/*
Here are various JS functions to enhance the website experience.

The basic website functionality should be available even for browser without
JS enabled.
*/

// Pricing page -  calculate estimation for services, with currency switcher
if (document.getElementsByClassName('cards-services').length) {
    var server_gbp = $('#server-gbp-price')[0].innerText
    var server_usd = $('#server-usd-price')[0].innerText
    var server_eur = $('#server-eur-price')[0].innerText

    var client_gbp = $('#client-gbp-price')[0].innerText
    var client_usd = $('#client-usd-price')[0].innerText
    var client_eur = $('#client-eur-price')[0].innerText

    var balancer_gbp = server_gbp * 0.6;
    var balancer_usd = server_usd * 0.6;
    var balancer_eur = server_eur * 0.6;

    var prices = {
        server_gbp: server_gbp,
        server_eur: server_eur,
        server_usd: server_usd,
        client_gbp: client_gbp,
        client_usd: client_usd,
        client_eur: client_eur,
        balancer_gbp: balancer_gbp,
        balancer_usd: balancer_usd,
        balancer_eur: balancer_eur,
    }

    function resetCheckboxes() {
      $('input[type="checkbox"]').each(function() {
          $(this).prop('checked', false)
      })
    }

    function isInArray(value, array) {
        return array.indexOf(value) > -1
    }

    resetCheckboxes()
    var services = []

    $(document).on('change', '.checkbox', function() {
        var currency = $('#currency-switcher .btn.active')[0].id
        var service = this.id.replace('add-', '')

        if (this.checked) {
            // Add Server if Load balancer is checked.
            if (service == 'balancer') {
                if (!isInArray('server', services)) {
                    services.push('server')
                    $('.server-subtotal').removeClass('d-none')
                    $('#add-server').prop('checked', true)
                }
            }
            services.push(service)

            $('.cards-services ').addClass('cards-services-transition')
            $('#price-estimation').removeClass('card-none').addClass('card-estimation')

            $('.' + service + '-subtotal').removeClass('d-none')

            var r = getTotalPrice(services, currency)
            onCurrencyChange(currency, r[0], r[1])
        } else {
            services = services.filter(function(e) {
                return e !== service
            })

            // If Server is unchecked, uncheck Load balancer also.
            if (service == 'server') {
                services = services.filter(function(e) {
                    return e !== 'balancer'
                })

                $('.balancer-subtotal').addClass('d-none')
                $('#add-balancer').prop('checked', false);
            }

            $('.' + service + '-subtotal').addClass('d-none')

            var r = getTotalPrice(services, currency)
            onCurrencyChange(currency, r[0], r[1])
            // Hide estimation card when checkboxes unchecked and Total is 0
            if (r[1] == 0) {
                $('#price-estimation').removeClass('card-estimation').addClass('card-none')
                $('.cards-services ').removeClass('cards-services-transition')

            }
        }
    });

    $('#currency-switcher > .btn').click(function(){
        $(this).addClass('active').siblings().removeClass('active');
        var new_currency = this.id
        var r = getTotalPrice(services, new_currency)
        // Update Support and Total
        onCurrencyChange(new_currency, r[0], r[1])
    });

    var getTotalPrice = function(services, currency) {
        var subtotal = 0
        var total = 0
        services.forEach(function(service) {
            subtotal = subtotal + parseInt(prices[service + '_' + currency])
        })

        var support = Math.ceil(subtotal * 0.2)
        var total = subtotal + support
        setEmailText()
        return [support, total]
    }

    var onCurrencyChange = function (currency, support, total) {
        if (!currency) {
            currency = 'gbp'
        }

        $('#server-cost').text(prices['server_'+currency])
        $('#balancer-cost').text(prices['balancer_'+currency])
        $('#client-cost').text(prices['client_'+currency])
        $('#support-cost').text(support)
        $('span.currency').text(currency.toUpperCase())
        $('#total-cost').text(total)
        setEmailText()
    }

    function setEmailText() {
        var email = $('#estimation')[0].innerText
        $('#estimation-request').text(email)
    }

}

/*
 JQuery integration
*/

$(document).ready(function() {

  let banner_code  = 'ua-war'
  // Show banner if it was not visited.
  if (window.localStorage.getItem('banner_visited_code') != banner_code) {
    $('#top-banner').removeClass('d-none');
  }

  $('#banner-link').click(function(){
    hideBanner(banner_code)
  });

  // Hide if link is visited by clicking on article not from banner.
  let banner_link = '/articles/2022/stand-with-ukraine.html'
  if (window.location.pathname == banner_link) {
    hideBanner(banner_code)
  }

  function hideBanner(banner_code) {
    window.localStorage.setItem('banner_visited_code', banner_code);
    $('#top-banner').addClass('d-none');
  }

  // Add a css class to all external text links
  $('main a:not(:has(img))')
    .filter(function () {
      return this.hostname && this.hostname !== location.hostname;
    })
    .addClass('external-link');

  // Handle articles tag/archive dropdown menus.
  let tags = document.getElementsByClassName('dropdown-tag');
  let years = document.getElementsByClassName('dropdown-year');
  let m = tags.length;
  let n = years.length;

  let pathname = window.location.pathname;
  let paths = pathname.split('/');
  // get year or tag name from URL
  let last = paths.pop();
  if (last == '') { // for tag the URL ends with a / so we take the next word
    last = paths.pop();
  }

  // Set tag and count text on dropdown
  for (let i = 0; i < m; i++) {
    if (tags[i].text.split(' ')[0] == last) {
      $('button#dropdown-tags').text(tags[i].text);
    }
  }

  // Set year and count text on dropdown
  for (let i = 0; i < n; i++) {
    if (years[i].text.split(' ')[0] == last) {
      $('button#dropdown-years').text(years[i].text);
    }
  }
});
