//Find the input search box
let search_version = document.getElementById("search-version");

//Find every item inside the dropdown
let items = document.getElementsByClassName("dropdown-item");

function buildDropDown() {
  if (typeof SFTPPLUS_RELEASED_VERSIONS === "undefined") {
    // Most probably we are not on a documentation page.
    return;
  }

  let values = SFTPPLUS_RELEASED_VERSIONS;

  let contents = [];

  //Group versions by major release tag number
  //eg. {4: ['4.7.0', '4.8.0'], 3: ['3.0.0']}
  const grouped_versions = values.reduce(function (result, version) {
    const number = version[0];
    result[number] = result[number] || [];
    result[number].push(version);
    return result;
  }, {});

  //Get list of major versions, eg. ['4', '3', '2']
  let major_versions = Object.keys(grouped_versions).reverse();
  //The latest major version is the max one
  let latest_version = Math.max.apply(Math, major_versions);

  major_versions.forEach(function(v) {
    let versions = grouped_versions[v];

    if (v == latest_version) {
      insert_versions(contents, v, versions, "Recent releases", true);
    } else {
      insert_versions(contents, v, versions, "Version " + v + " (end of life)");
    }
  });

  $("#versions-menu").append(contents.join(""));

  //Hide the row that shows no items were found
  $("#empty").hide();
}

//Capture the event when user types into the search box
window.addEventListener("input", function () {
  if(search_version){
    search_versions(search_version.value.trim());
  }
});

//Construct the lists of versions
function insert_versions(
  contents,
  version_key,
  values,
  title,
  latest
) {
  latest = typeof latest !== 'undefined' ? latest : false;

  contents.push(
    '<h6 class="dropdown-header text-dark v-' +
      version_key +
      '">' +
      title +
      "</h6>"
  );

  let url = "sftpplus";
  if (["2"].indexOf(version_key) >= 0) {
    url = "server";
  }

  let shown_values = values.slice(0, 5);

  values.forEach(function(version) {
    if (latest && version == shown_values[0]) {
      contents.push(
        '<a class="text-primary dropdown-item" \
          href="/documentation/' +
          url +
          "/v/" +
          version +
          '/">' +
          version +
          " (latest)</a>"
      );
    } else if (shown_values.indexOf(version) >= 0) {
      contents.push(
        '<a class="text-primary dropdown-item" \
          href="/documentation/' +
          url +
          "/v/" +
          version +
          '/">' +
          version +
          "</a>"
      );
    } else {
      contents.push(
        '<a class="text-primary dropdown-item version-hidden" \
          href="/documentation/' +
          url +
          "/v/" +
          version +
          '/">' +
          version +
          "</a>"
      );
    }
  });
  contents.push(
    '<p class="text-dark ml-3 mb-0 v-' +
      version_key +
      '">Use search for older releases</p>'
  );
}

// Polyfill for startsWith
if (!String.prototype.startsWith) {
  Object.defineProperty(String.prototype, "startsWith", {
    value: function (search, rawPos) {
      var pos = rawPos > 0 ? rawPos | 0 : 0;
      return this.substring(pos, pos + search.length) === search;
    },
  });
}

//For every number entered by the user, check if the version starts with that number
//If it does show the version, else hide it
function search_versions(number) {
  let length = items.length;
  let hidden = 0;

  for (let i = 0; i < length; i++) {
    if (items[i].text.startsWith(number)) {
      // reset css for headers and p's
      if (number == "") {
        $("#versions-menu h6").removeClass("d-none");
        $("#versions-menu p").removeClass("d-none");
        $("#versions-menu h6").removeClass("d-block");
        $("#versions-menu p").removeClass("d-block");
      }
      // display matching items
      $(items[i]).css("display", "block");
      $(".v-" + number[0]).removeClass("d-none");
      $(".v-" + number[0]).addClass("d-block");
    } else {
      // hide not matching items
      $(items[i]).css("display", "none");
      $("#versions-menu h6:not(.v-" + number[0] + ")").addClass("d-none");
      $("#versions-menu p:not(.v-" + number[0] + ")").addClass("d-none");
      hidden++;
    }
  }

  // If all items are hidden, show the empty view.
  if (hidden === length) {
    $("#empty").show();
    // Do not show any headers or p's for empty view.
    $("#versions-menu h6").hide();
    $("#versions-menu p").hide();
    $("#versions-menu h6").removeClass("d-block");
    $("#versions-menu p").removeClass("d-block");
  } else {
    $("#empty").hide();
    $("#versions-menu h6").show();
    $("#versions-menu p").show();
  }
}

buildDropDown();
