# Copyright (c) 2012 Adi Roiban.
# See LICENSE for details.
"""
System tests for `BrinkFilesystem`.
"""
import os

from brink.testing import BrinkTestCase
from brink.filesystem import BrinkFilesystem


class TestBrinkFilesystem(BrinkTestCase):
    """
    System tests for `BrinkFileSystem`.
    """

    def setUp(self):
        super(TestBrinkFilesystem, self).setUp()

        self.brink_fs = BrinkFilesystem()
        self.full_path = self.createFileInTemp(suffix='.bat')
        self.folder, self.file_name = os.path.split(self.full_path)

    def test_which_file_exists(self):
        """
        Returns the full path to the specified command if the file is
        found.
        """
        command = self.file_name
        folder = self.folder
        full_path = self.full_path
        extra_paths = [self.string(), folder]

        result = self.brink_fs.which(command, extra_paths=extra_paths)

        self.assertEqual(full_path, result)

    def test_which_file_exists_no_extension(self):
        """
        On Windows, it returns the full path to the specified command even
        when the executable extension was not provided.
        """
        if os.name != 'nt':
            raise self.skipTest("Windows specific test.")

        extra_paths = [self.string(), self.folder]
        command = self.file_name.replace('.bat', '')

        result = self.brink_fs.which(command, extra_paths=extra_paths)

        self.assertEqual(self.full_path, result)

    def test_which_not_exist(self):
        """
        Returns `None` if the specified command could not be found.
        """
        unknown_command = self.string()

        result = self.brink_fs.which(unknown_command)

        self.assertIsNone(result)
