# Copyright (c) 2010 Adi Roiban.
# See LICENSE for details.
"""
Chevah Server module.
"""
from __future__ import unicode_literals
from __future__ import absolute_import
import os
import sys
from chevah.slim_server.pool import spawn_pool
# This is imported for the side effect.
from chevah.server import side_effects
from chevah.server.helpers import (
    add_timeout,
    force_unicode,
    get_path_handling_module,
    native_string,
    )

MODULE_PATH = os.path.dirname(__file__)
MODULE_PATH

SCRIPTS_PATH = os.path.dirname(sys.executable)
if os.name == 'nt':
    # On Windows, we might start via the Windows Server and then
    # sys.executable is not reliable.
    # This is for V3 installed services.
    # In V4 we always start via the python.exe
    # (renamed to sftpplus-worker.exe).
    SCRIPTS_PATH = os.path.join(
        os.path.dirname(os.path.dirname(os.__file__)), 'Scripts')

from chevah.server.event.handler import (  # noqa
    EventDispatcher,
    emit_error_details,
    )

# Create EventDispatcher singleton and add method shortcuts.
# These imports needs to be at the end so that previous helpers
# are available.
event_dispatcher = EventDispatcher()
emit = event_dispatcher.emit
# Emit private as most code should just use emitEvent
_emit = event_dispatcher.emit
# FIXME:3231:
# Make sure these methods also have a creator.
emitFromError = event_dispatcher.emitFromError
emit_from_error = emitFromError

# Shut up the linter.
add_timeout
side_effects
emit_error_details
force_unicode
native_string
get_path_handling_module
spawn_pool


if __name__ == '__main__':  # noqa:cover
    """
    Start the server using command line options.
    """
    # Create the spawn processed ASAP to reduce their memory usage.

    from chevah.server.commons import component
    from chevah.server.commons.command import admin_commands
    component.register()
    admin_commands()
