# Copyright (c) 2015 Adi Roiban.
# See LICENSE for details.
"""
Authentication based on a list of denied accounts.
"""
from __future__ import unicode_literals
from __future__ import absolute_import

from zope.interface import implementer

from chevah.server.commons.exception import CredentialsForbidden
from chevah.server.commons.interface import IAuthentication
from chevah.server.commons.runnable import Runnable


@implementer(IAuthentication)
class DenyUsernameAuthentication(Runnable):
    """
    Simple authentication which deny based on a list of users.
    """
    PROPERTIES_REQUIRING_RESTART = ()

    def _onStart(self, avatar=None):
        """
        See: Runnable.
        """
        # No action is taken for start.
        pass

    def _onStop(self, avatar=None):
        """
        See: Runnable.
        """
        # Nothing is required for this auth method.
        pass

    def authenticate(self, credentials):
        """
        See: `IAuthentication`
        """
        if credentials.username.lower() in self.configuration.usernames:
            raise CredentialsForbidden('User in the deny list.')
        # If not in the list, allow other methods to continue.
        return None

    def authenticateAdministrator(self, credentials):
        """
        See: `IAuthentication`
        """
        return self.authenticate(credentials)
