# Copyright (c) 2014 Adi Roiban.
# See LICENSE for details.
"""
Authentication methods management.
"""
from __future__ import unicode_literals
from __future__ import absolute_import

from zope.interface import implementer

from chevah.server.commons.interface import (
    IAuthentication,
    IAuthenticationsManager,
    IOIDCAuthentication,
    )
from chevah.server.commons.runnable import (
    RunnablesManagerWithConfigurationType,
    )


@implementer(IAuthenticationsManager)
class AuthenticationsManager(RunnablesManagerWithConfigurationType):
    """
    Management of available authentication method components.
    """
    __type_interface__ = IAuthentication

    def _getRunnablesConfiguration(self):
        """
        See: `RunnablesManagerWithConfigurationSet`.
        """
        return self.parent.configuration.authentications

    def getOIDC(self, authentications, default):
        """
        Return the first authentication method that can handle OpenID connect
        requests.

        Return None if no OIDC is enabled.
        """
        if not authentications:
            # Fallback to default methods.
            authentications = default

        for method_id in authentications:
            method = self.getSection(method_id)

            if not method:
                # We will raise an error later if UUID is not found.
                continue

            if not IOIDCAuthentication.providedBy(method):
                # Not an OpenID connect authentication.
                continue

            if not method.started:
                # We ignore the methods that are not started.
                continue

            return method

        return None
