# Copyright (c) 2010 Adi Roiban.
# See LICENSE for details.
"""
Common code which we failed to organize in other way.
"""
from __future__ import absolute_import
from __future__ import unicode_literals
import datetime

from cryptography.hazmat.backends import default_backend

ZERO = datetime.timedelta(0)


class UTC(datetime.tzinfo):
    """UTC
    Identical to the reference UTC implementation given in Python docs except
    that it unpickles using the single module global instance defined beneath
    this class declaration.
    Also contains extra attributes and methods to match other pytz tzinfo
    instances.
    """
    zone = "UTC"

    def utcoffset(self, dt):
        return ZERO

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return ZERO

    def __reduce__(self):
        return UTC, ()

    def localize(self, dt, is_dst=False):
        '''Convert naive time to local time'''
        if dt.tzinfo is not None:
            raise ValueError('Not naive datetime (tzinfo is already set)')
        return dt.replace(tzinfo=self)

    def normalize(self, dt, is_dst=False):
        '''Correct the timezone information on the given datetime'''
        if dt.tzinfo is None:
            raise ValueError('Naive time - no tzinfo set')
        return dt.replace(tzinfo=self)

    def __repr__(self):
        return "<UTC>"

    def __str__(self):
        return "UTC"


# UTC is a singleton
UTC = UTC()

DEFAULT_BACKEND = default_backend()

OPENSSL_ED25519_SUPPORTED = DEFAULT_BACKEND.ed25519_supported()
