# Copyright (c) 2017 Adi Roiban.
# See LICENSE for details.
"""
CSV handling code.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

import csv
import six
from io import BytesIO, StringIO


def to_csv_row(fields):
    """
    Returns an CSV formatted Unicode row including the line separator.
    """
    if six.PY2:
        # FIXME:6162:
        # We should update all the code to only call with bytes.
        buffer = BytesIO()
    else:
        buffer = StringIO()

    to_write_row = []
    for item in fields:
        if not isinstance(item, six.text_type):
            raise ValueError('Row values should be text.')
        if six.PY2:
            item = item.encode('utf-8')
        to_write_row.append(item)

    writer = csv.writer(buffer)
    writer.writerow(to_write_row)
    result = buffer.getvalue()
    if not isinstance(result, six.text_type):
        result = result.decode('utf-8')
    return result


def from_csv_row(line):
    """
    Return a list of values in CSV line.
    """
    if six.PY2:
        # On Py2 CSV reader works internally as bytes
        # and on Py3 as text.
        line = line.encode('utf-8')
    reader = csv.reader([line], skipinitialspace=True)
    result = []
    for row in list(reader)[0]:
        if not isinstance(row, six.text_type):
            row = row.decode('utf-8')
        result.append(row)
    return result


class UnicodeCSVProducer(object):
    """
    A CSV row producer which will pass Unicode rows to `consumer` as
    UTF-8 encoding.
    """
    def __init__(self, consumer):
        self._consumer = consumer

    def makeRow(self, row):
        self._consumer(to_csv_row(row).encode('utf-8'))
