# Copyright (c) 2021 Adi Roiban.
# See LICENSE for details.
"""
Helper for internationalization.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

from babel.support import Translations

from chevah_compat import local_filesystem

from chevah.server.commons.constant import (
    HTTP_LANGUAGES,
    HTTP_STATIC_FOLDER_NAME,
    )
from chevah.server.commons.exception import ServerException

from chevah.server.commons.jinja import JinjaEnvironment


def get_translation(lang_code):
    """
    Return a new translation.
    """
    if lang_code not in HTTP_LANGUAGES:
        raise ServerException(
            'Unknown language code: "%s".' % (lang_code))

    dirname = local_filesystem.getEncodedPath(
        local_filesystem.getRealPathFromSegments(
            local_filesystem.installation_segments + ['lib', 'locale']))

    translation = Translations.load(
        dirname=dirname,
        locales=[lang_code],
        # For now the domain is hardcoded.
        # In the future we might have separate domains, but I don't think that
        # will happen soon.
        domain='http-service',
        )

    def install_translation(target):
        """
        Inject translation into target.
        """
        target._ = translation.ugettext
        target._translation = translation

    translation.install = install_translation

    # The template root folder is also hardcoded as for now we plan
    # to provide translation only for the HTTP service.
    jinja = JinjaEnvironment(HTTP_STATIC_FOLDER_NAME, 'jinja')
    jinja.uninstall_gettext_translations('ignored')
    jinja.install_gettext_translations(translation, newstyle=True)

    translation.get_template = jinja.get_template

    return translation


# A translation that does nothing.
NULL_TRANSLATION = get_translation('en')
