# Copyright (c) 2016 Adi Roiban.
# See LICENSE for details.
"""
Code to help dealing with lines.
"""

from __future__ import absolute_import
from __future__ import unicode_literals
import re

from chevah.server.commons.exception import ServerException


def get_normalized_lines(stream, max_line_length=16000):
    """
    Return line by line of the stream normalized to Unix new lines and
    checking for max line length.

    Raise `ServerException` if there is a line too long.
    """
    data = stream.read(max_line_length)
    buffer = b''
    while data:
        buffer += data
        while buffer:
            try:
                line, buffer = re.split(b'\r\n|\n', buffer, 1)
            except ValueError:
                if len(buffer) > max_line_length:
                    raise ServerException('Line is too long.')
                break
            # New line normalization to Unix new lines.
            yield line + b'\n'
        data = stream.read(max_line_length)
    if buffer:
        yield buffer
