# Copyright (c) 2019 Adi Roiban.
# See LICENSE for details.
"""
Event handler to be used as a placeholder for deprecated handlers.
"""
from __future__ import unicode_literals

from __future__ import absolute_import
from chevah.server.commons.exception import ServerException
from chevah.server.event.handler import EventHandlerWithFilter


class DeprecatedEventHandler(EventHandlerWithFilter):
    """
    Placeholder event handler.
    """
    PROPERTIES_REQUIRING_RESTART = ()

    def _onStart(self, avatar=None):
        """
        See: `Runnable`.

        It always fails.
        """
        raise ServerException(
            'Event handler of type "%s" is no longer supported. '
            'See the upgrade documentation.' % (
                self.configuration.type,))

    def _onStop(self, avatar=None):
        """
        See: `Runnable`.

        Does nothing.
        """

    def _onEvent(self, event):
        """
        See: EventHandlerWithFilterMixin.
        """
        raise ServerException('Event handler is no longer supported.')
