# Copyright (c) 2018 Adi Roiban.
# See LICENSE for details.
"""
Event handler formating the events for standard outputs.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

import sys
import six

from chevah.server.event.handler import EventHandlerWithFilter
from chevah.server.event.model import StringFormatter


class StandardStreamEventHandler(EventHandlerWithFilter):
    """
    Sends events to the standard output.

    For now, only the STDOUT is supported.
    """
    PROPERTIES_REQUIRING_RESTART = (
        'entry_content',
        )

    def _onStart(self, avatar=None):
        """
        See: `Runnable`.
        """
        self._formatter = StringFormatter(self.configuration.entry_content)

    def _onStop(self, avatar=None):
        """
        See: `Runnable`.
        """
        self._formatter = None

    def _onEvent(self, event):
        """
        See: `EventHandlerWithFilterMixin`.
        """
        output = self._formatter.format(event)
        if six.PY2:
            output = output.encode('utf-8')

        sys.stdout.write(output)
        sys.stdout.flush()
