# Copyright (c) 2015 Adi Roiban.
# See LICENSE for details.
"""
Event handler triggering Windows EventLog event hooks.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

import os

from chevah.server.event.stdlib_logging import (
    EventHandlerWithLogger,
    )


if os.name == 'nt':
    from logging.handlers import NTEventLogHandler

    class WindowsEventLogHandler(NTEventLogHandler):
        """
        Custom NTEventLogger for sending events.

        For now we don't use any special category or level.
        """

        def open(self):
            """
            EventLog has no connection.
            """
            pass

        def getMessageID(self, log_record):
            try:
                return int(log_record.event.id)
            except ValueError:
                return 1

        def handleError(self, record):
            """
            Will just raise the error so that it will be handled by
            the log handler adapter.
            """
            raise

else:
    WindowsEventLogHandler = None


class WindowsEventLogEventHandler(EventHandlerWithLogger):
    """
    Sends events to the Windows EventLog.
    """
    PROPERTIES_REQUIRING_RESTART = ('name',)

    def __init__(self, parent, configuration):
        super(WindowsEventLogEventHandler, self).__init__(
            parent, configuration)
        self._handler = None

    def _getLogHandler(self):
        """
        See: `EventHandlerWithLogger`.
        """
        if not WindowsEventLogHandler:
            raise RuntimeError(
                'Windows EventLog is only supported on Windows')
        return WindowsEventLogHandler(self._configuration.name)
