# Copyright (c) 2017 Adi Roiban.
# See LICENSE for details.
"""
Extensions which are part of the main distribution.
"""
from __future__ import absolute_import
from twisted.python.reflect import namedObject

from chevah.server.commons.exception import ServerException


def load_extension_object(name, default=None):
    """
    Return an object (class/function) from `name` which is passed as dotted
    Python reference, prefixed with `python:`.

    If `name` is not an API extension, it returns the `default` value.
    """
    if not name.startswith('python:'):
        if default is None:
            raise ServerException(
                'No extension defined for "%s".' % (name,))
        return default

    # Remove the `python:` prefix
    name = name[7:].strip()

    try:
        return namedObject(name)
    except Exception as error:
        raise ServerException('Failed to load extension "%s". %s' % (
            name, error))
