# Copyright (c) 2018 Adi Roiban.
# See LICENSE for details.
"""
Here are some simple entry content formatters.

Most of them are designed to fit a very specific need for one of our customers,
but are generic enough to be used by other customers with similar requirements.

Since for now there are few formatters, there is no public documentation.
We can document this once multiple customers make use of them.
"""
from __future__ import absolute_import, unicode_literals

from chevah.server import get_path_handling_module
from chevah.server.commons.csv import to_csv_row


def format_csv_date_and_filename(event):
    """
    Return a CSV string representation of `EventLogRecord` to be used for
    reporting date and only filename (no full pat) of uploaded files via a
    client-side transfer.

    It is designed to be used with event ID 60012,
    which have 'destination_path'.

    It will generate an output like:

    2017-06-28 09:20:58, file_T1.csv
    2017-06-28 09:20:58, file_T000032.csv
    """
    path = event.data.get('destination_path', 'has/no/no-associated.path')

    path_module = get_path_handling_module(path)
    filename = path_module.basename(path)

    if not filename:
        filename = 'event-associated-with-folder-path'

    fields = [event.timestamp.cwa_14051, filename]

    return to_csv_row(fields)
