# Copyright (c) 2014 Adi Roiban.
# See LICENSE for details.
"""
Entry point for the client-shell.

The client-shell is an interactive command-line shell serving as an interface
to remote locations.
"""

from __future__ import absolute_import
from __future__ import unicode_literals
import signal
import sys
from chevah.server.commons.exception import ServerException


def start_client_shell():
    """
    Start the location shell using command line options.
    """
    from chevah.server.commons import component
    from chevah.server.location.shell import ClientShell
    component.register()
    shell = ClientShell()

    def stop_client_shell(signum, frame):
        """
        Signal handler for keyboard interrupt and process termination.
        """
        shell._cmd._stop()
        # The sys.exit() should be moved inside ClientShell
        # and create a public ClientShell.stop() method.
        sys.exit(0)

    if (signal.getsignal(signal.SIGINT) == signal.default_int_handler):
        signal.signal(signal.SIGINT, stop_client_shell)
    else:  # noqa:cover
        # If there was already a signal handler this should not be setting
        # a new one.
        # https://github.com/chevah/server/pull/1365#discussion_r164729291
        raise ServerException(
            'A signal handler is already defined for SIGINT: %s' %
            signal.getsignal(signal.SIGINT))

    shell.start()


if __name__ == '__main__':  # noqa:cover
    start_client_shell()
