# Copyright (c) 2017 Adi Roiban.
# See LICENSE for details.
"""
Locations for a remote AS2 server.

This is just the location adapter. The actual client-side protocol is found
in the chevah.server.http package.
"""
from zope.interface import implementer

from chevah.server.commons.constant import (
    LOCATION_AS2_DEFAULTS,
    )
from chevah.server.commons.interface import (
    IAS2LocationConfiguration,
    )
from chevah.server.commons.ssl import (
    SSLClientMixin,
    SSLConfigurationSectionMixin,
    )
from chevah.server.configuration.model import (
    WritableString,
    WritableStringCaseInsensitive,
    )
from chevah.server.configuration.option import (
    URLOptions,
    UsernamePasswordPublicOptionsMixin,
    )

from chevah.server.http.as2_send import AS2Client

from chevah.server.location.commons import SSLLocationBase
from chevah.server.location.configuration import _LocationConfigurationBase


@implementer(IAS2LocationConfiguration)
class AS2LocationConfiguration(
        _LocationConfigurationBase,
        UsernamePasswordPublicOptionsMixin,
        SSLConfigurationSectionMixin,
        URLOptions,
        ):
    """
    Configuration for a AS2 HTTP/HTTPS location.
    """
    as2_partner_identifier = WritableString('as2_partner_identifier')
    as2_partner_certificates = WritableString('as2_partner_certificates')

    as2_own_identifier = WritableString('as2_own_identifier')
    as2_own_certificate = WritableString('as2_own_certificate')
    as2_own_private_key = WritableString('as2_own_private_key')

    as2_send_security = WritableStringCaseInsensitive(
        'as2_send_security', default=LOCATION_AS2_DEFAULTS)
    as2_mdn_receipt = WritableStringCaseInsensitive(
        'as2_mdn_receipt', default=LOCATION_AS2_DEFAULTS)
    as2_encryption_algorithm = WritableStringCaseInsensitive(
        'as2_encryption_algorithm', default=LOCATION_AS2_DEFAULTS)
    as2_signature_algorithm = WritableStringCaseInsensitive(
        'as2_signature_algorithm', default=LOCATION_AS2_DEFAULTS)

    as2_content_type = WritableStringCaseInsensitive(
        'as2_content_type', default=LOCATION_AS2_DEFAULTS)


class AS2Location(SSLLocationBase, SSLClientMixin):
    """
    Location for a remote AS2 server for sending files to.

    Receiving files from remote AS2 is not supported.
    """

    PROPERTIES_REQUIRING_RESTART = (
        SSLConfigurationSectionMixin.PROPERTIES_REQUIRING_RESTART +
        (
            'url',
            'password_update',
            'username',
            'idle_connection_timeout',
            'idle_connection_keepalive_interval',
            'connection_retry_count',
            'connection_retry_interval',
            'as2_partner_certificates',
            'as2_own_certificate',
            'as2_own_private_key',
            ))

    _HTTPClient = AS2Client
