# Copyright (c) 2018 Adi Roiban.
# See LICENSE for details.
"""
Locations for a remote Azure BLOB storage.

This is just the location adapter.
The actual client-side protocol is found in the chevah.server.http package.
"""
from __future__ import unicode_literals

from __future__ import absolute_import
from zope.interface import implementer

from chevah.server.commons.exception import ServerException
from chevah.server.commons.interface import (
    IAzureBLOBLocationConfiguration,
    )
from chevah.server.http.azure_blob import AzureBLOBClient
from chevah.server.location.azure_storage import (
    _AzureStorageLocation,
    _AzureStorageAccountLocationConfiguration,
    )


@implementer(IAzureBLOBLocationConfiguration)
class AzureBLOBLocationConfiguration(
        _AzureStorageAccountLocationConfiguration):
    """
    Configuration for a Azure BLOB Service location.
    """

    @property
    def address(self):
        if not self.username:
            return 'no-address-yet'
        return '%s.%s' % (self.username, AzureBLOBClient.SERVICE_DNS_BASE)


class AzureBLOBLocation(_AzureStorageLocation):
    """
    Location to a Azure BLOB storage.

    It has SSL configuration option, but for now are private.
    """

    _HTTPClient = AzureBLOBClient

    def getStreamForAppending(self, path):
        """
        See: ILocation.
        """
        raise ServerException('Appending is not supported in Azure Blob.')
