# Copyright (c) 2018 Adi Roiban.
# See LICENSE for details.
"""
Locations for a remote Azure File Service.

This is just the location adapter.
The actual client-side protocol is found in the chevah.server.http package.
"""
from __future__ import unicode_literals

from __future__ import absolute_import
from zope.interface import implementer

from chevah.server.commons.exception import ServerException
from chevah.server.commons.interface import (
    IAzureFileLocationConfiguration,
    )
from chevah.server.http.azure_file import AzureFileClient
from chevah.server.location.azure_storage import (
    _AzureStorageLocation,
    _AzureStorageAccountLocationConfiguration,
    )


@implementer(IAzureFileLocationConfiguration)
class AzureFileLocationConfiguration(
        _AzureStorageAccountLocationConfiguration):
    """
    Configuration for a Azure File Service location.
    """

    @property
    def address(self):
        if not self.username:
            return 'no-address-yet'
        return '%s.%s' % (self.username, AzureFileClient.SERVICE_DNS_BASE)


class AzureFileLocation(_AzureStorageLocation):
    """
    Location to a Azure File storage.
    """

    _HTTPClient = AzureFileClient

    def getStreamForAppending(self, path):
        """
        See: ILocation.
        """
        raise ServerException('Appending is not supported in Azure File.')
