# Copyright (c) 2014 Adi Roiban.
# See LICENSE for details.
"""
Location runnables management.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

from twisted.internet import defer
from zope.interface import implementer

from chevah.server.commons.interface import (
    ILocalLocation,
    ILocation,
    ILocationsManager,
    )
from chevah.server.commons.runnable import (
    RunnablesManagerWithConfigurationType,
    )
from chevah.server.location.commons import (
    LocalLocationAuditor,
    LocationAuditor,
    )


@implementer(ILocationsManager)
class LocationsManager(RunnablesManagerWithConfigurationType):
    """
    Management of available locations.
    """
    __type_interface__ = ILocation

    def _getRunnablesConfiguration(self):
        """
        See: `RunnablesManagerWithConfigurationType`.
        """
        return self.parent.configuration.locations

    def _createRunnable(self, configuration):
        """
        Called when a new location should be created.
        """
        location = super(LocationsManager, self)._createRunnable(configuration)
        # Wrap the location into an auditor.
        if ILocalLocation.providedBy(location):
            return LocalLocationAuditor(context=location)
        else:
            return LocationAuditor(context=location)

    def start(self):
        """
        See: IRunnablesManager.

        Resources are not auto-started.
        """
        return defer.succeed(None)
