# Copyright (c) 2014 Adi Roiban.
# See LICENSE for details.
"""
Entry point for the admin-shell.

The admin-shell is an interactive command-line shell serving as an interface
to the local manager service.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

import signal
from chevah.server.commons.exception import ServerException


def start_admin_shell():
    """
    Start the location shell using command line options.
    """
    from chevah.server.manager.shell import AdminShell
    shell = AdminShell()

    def stop_admin_shell(signum, frame):
        """
        Signal handler for keyboard interrupt and process termination.
        """
        shell.stop()

    if (signal.getsignal(signal.SIGINT) == signal.default_int_handler):
        signal.signal(signal.SIGINT, stop_admin_shell)
    else:  # noqa:cover
        # If there was already a signal handler this should not be setting
        # a new one.
        # https://github.com/chevah/server/pull/1365#discussion_r164729291
        raise ServerException(
            'A signal handler is already defined for SIGINT: %s' %
            signal.getsignal(signal.SIGINT))

    shell.start()


if __name__ == '__main__':  # noqa:cover
    start_admin_shell()
