# Copyright (c) 2011 Adi Roiban.
# See LICENSE for details.
"""
Configuration for management service.
"""
from __future__ import unicode_literals
from __future__ import absolute_import
from zope.interface import implementer

from chevah.server.commons.interface import (
    ILocalManagerServiceOptions,
    )
from chevah.server.commons.service_configuration import (
    ConfigurationWithConnections,
    )

from chevah.server.commons.ssl import (
    SSLConfigurationSectionMixin,
    )
from chevah.server.configuration.model import (
    WritableString,
    WritableStringCaseInsensitive,
    WritableCSVLinesUnique,
    WritableCSVUniqueInsensitive,
    )


@implementer(ILocalManagerServiceOptions)
class LocalManagerServiceConfigurationSection(
        ConfigurationWithConnections, SSLConfigurationSectionMixin):
    """
    Configurations for local manager service.
    """
    PROPERTIES_REQUIRING_RESTART = (
        SSLConfigurationSectionMixin.PROPERTIES_REQUIRING_RESTART +
        ('headers', 'accepted_origins', 'base_url_path')
        )

    headers = WritableCSVLinesUnique('headers')
    accepted_origins = WritableCSVUniqueInsensitive(
        'accepted_origins')
    client_forwarded_header = WritableStringCaseInsensitive(
        'client_forwarded_header', default='')
    base_url_path = WritableString('base_url_path')
