# Copyright (c) 2011 Adi Roiban.
# See LICENSE for details.
"""
Local manager realm.

This received valid and already authenticated configuration for an account.
"""
from __future__ import unicode_literals
from __future__ import absolute_import
from twisted.internet import defer

from chevah.server.commons.interface import (
    IApplicationLocalManagerAvatar,
    IOSAdministratorConfiguration,
    IOSLocalManagerAvatar,
    )
from chevah.server.manager.avatar import (
    ApplicationLocalManagerAvatar,
    OSLocalManagerAvatar,
    )
from chevah.server.commons.realm import RealmAbstract


class LocalManagerRealm(RealmAbstract):
    """
    Realm for local manager service.
    """

    def _requestAvatar(self, authentication_result, mind=None, *interfaces):
        """
        Return a deferred that returns the avatar based on
        result credentials validation.
        """
        administrator, roles, credentials = authentication_result

        if IOSAdministratorConfiguration.providedBy(administrator):
            interface = IOSLocalManagerAvatar
            klass = OSLocalManagerAvatar
        else:
            interface = IApplicationLocalManagerAvatar
            klass = ApplicationLocalManagerAvatar

        avatar = klass(
            uuid=administrator.uuid,
            name=administrator.name,
            roles=roles,
            peer=credentials.peer,
            component=credentials.creator,
            )
        deferred = defer.succeed(None)
        deferred.addCallback(
            lambda _: self._analytics.administratorAuthenticated(avatar))
        deferred.addCallback(lambda _: (interface, avatar))
        return deferred
