# Copyright (c) 2015 Adi Roiban.
# See LICENSE for details.
"""
Component registration for resources.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

from chevah.server.commons.constant import TYPE_NAME
from chevah.server.commons.interface import (
    IResourceConfiguration,
    IResourceRunnable,
    )
from chevah.server.resource.analytics import (
    AnalyticsConfiguration,
    AnalyticsResource,
    DeprecatedProcessMonitorConfiguration,
    DeprecatedProcessMonitorResource,
    )
from chevah.server.resource.email import (
    EmailClientConfiguration,
    EmailClientResource,
    )
from chevah.server.resource.lets_encrypt import (
    LetsEncryptClientConfiguration,
    LetsEncryptClientResource,
    )
from chevah.server.resource.monitor import (
    MonitorConfiguration,
    MonitorResource,
    )
from chevah.server.resource.sqlite import (
    SQLiteConfiguration,
    SQLiteResource,
    )


def register_resources():
    """
    Register all configuration and runnable for the resources.
    """
    # General components are calling this method so we need to avoid
    # the circular import.
    from chevah.server.commons.component import registry

    # SQLIte Db.
    registry.registerUtility(
        SQLiteConfiguration,
        IResourceConfiguration,
        name=TYPE_NAME.SQLITE,
        )
    registry.registerUtility(
        SQLiteResource,
        IResourceRunnable,
        name=TYPE_NAME.SQLITE,
        )

    # Email client.
    registry.registerUtility(
        EmailClientConfiguration,
        IResourceConfiguration,
        name=TYPE_NAME.EMAIL_CLIENT,
        )
    registry.registerUtility(
        EmailClientResource,
        IResourceRunnable,
        name=TYPE_NAME.EMAIL_CLIENT,
        )

    # Let's Encrypt client.
    registry.registerUtility(
        LetsEncryptClientConfiguration,
        IResourceConfiguration,
        name=TYPE_NAME.LETS_ENCRYPT,
        )
    registry.registerUtility(
        LetsEncryptClientResource,
        IResourceRunnable,
        name=TYPE_NAME.LETS_ENCRYPT,
        )

    # Filesystem monitor.
    registry.registerUtility(
        MonitorConfiguration,
        IResourceConfiguration,
        name=TYPE_NAME.MONITOR,
        )
    registry.registerUtility(
        MonitorResource,
        IResourceRunnable,
        name=TYPE_NAME.MONITOR,
        )

    # Deprecated process monitor.
    registry.registerUtility(
        DeprecatedProcessMonitorConfiguration,
        IResourceConfiguration,
        name=TYPE_NAME.PROCESS_MONITOR,
        )
    registry.registerUtility(
        DeprecatedProcessMonitorResource,
        IResourceRunnable,
        name=TYPE_NAME.PROCESS_MONITOR,
        )

    # Analytics configuration and runnable.
    registry.registerUtility(
        AnalyticsConfiguration,
        IResourceConfiguration,
        name=TYPE_NAME.ANALYTICS,
        )
    registry.registerUtility(
        AnalyticsResource,
        IResourceRunnable,
        name=TYPE_NAME.ANALYTICS,
        )
