# Copyright (c) 2015 Adi Roiban.
# See LICENSE for details.
"""
Root manager for resources.
"""
from __future__ import absolute_import
from __future__ import unicode_literals
from zope.interface import implementer

from chevah.server.commons.constant import (
    DEFAULT_ANALYTICS_UUID,
    DEFAULT_EMAIL_CLIENT_UUID,
    DEFAULT_LETS_ENCRYPT_UUID,
    DEFAULT_SQLITE_DB_UUID,
    )
from chevah.server.commons.exception import ServerException
from chevah.server.commons.interface import (
    IResourceRunnable,
    IRunnablesManager,
    )
from chevah.server.commons.runnable import (
    RunnablesManagerWithConfigurationType,
    )


@implementer(IRunnablesManager)
class ResourcesManager(RunnablesManagerWithConfigurationType):
    """
    Management of available resources.
    """
    __type_interface__ = IResourceRunnable

    def _getRunnablesConfiguration(self):
        """
        See: `RunnablesManagerWithConfigurationType`.
        """
        return self.parent.configuration.resources

    def getDatabase(self):
        """
        Return the database resource.
        """
        result = self.getSection(DEFAULT_SQLITE_DB_UUID)
        if result is not None:  # noqa:cover
            return result

        # This should not happen as when the process is started the
        # default DB should be auto-created if missing.
        raise ServerException('Missing embedded database.')  # noqa:cover

    def getAnalytics(self):
        """
        Return the Analytics resource.
        """
        result = self.getSection(DEFAULT_ANALYTICS_UUID)
        if result is not None:  # noqa:cover
            return result

        # This should not happen as when the process is started the
        # default DB should be auto-created if missing.
        raise ServerException('Missing embedded database.')  # noqa:cover

    def getEmailClient(self):
        """
        Return the email client resource.
        """
        result = self.getSection(DEFAULT_EMAIL_CLIENT_UUID)
        if result is not None:  # noqa:cover
            return result

        # This should not happen as when the process is started the
        # default email client should be auto-created if missing.
        raise ServerException('Missing email client.')  # noqa:cover

    def getLetsEncryptClient(self):
        """
        Return the let's encrypt client resource.
        """
        result = self.getSection(DEFAULT_LETS_ENCRYPT_UUID)
        if result is not None:  # noqa:cover
            return result

        # This should not happen as when the process is started the
        # default client should be auto-created if missing.
        raise ServerException('Missing Let\'s Encrypt  client.')  # noqa:cover
