# Copyright (c) 2015 Adi Roiban.
# See LICENSE for details.
"""
Email resources.
"""
from __future__ import absolute_import, unicode_literals

from zope.interface import implementer

from chevah.server.commons.interface import (
    ISQLiteConfiguration,
    ISQLiteResource,
    )
from chevah.server.commons.runnable import (
    Runnable,
    )
from chevah.server.configuration.option import (
    LocalPathConfigurationMixin,
    )
from chevah.server.db.interaction import DBInteractionSQLite
from chevah.server.resource.configuration import _ResourceConfigurationBase


@implementer(ISQLiteConfiguration)
class SQLiteConfiguration(
        _ResourceConfigurationBase,
        LocalPathConfigurationMixin,
        ):
    """
    Configurations for the SQLite resource.
    """


@implementer(ISQLiteResource)
class SQLiteResource(Runnable, DBInteractionSQLite):
    """
    An SQLite connection as a resource.
    """
    PROPERTIES_REQUIRING_RESTART = (
        'path',
        )

    def _onStart(self, avatar=None):
        """
        See `Runnable`.
        """
        return self._startDB()

    def _onStop(self, avatar=None):
        """
        See `Runnable`.
        """
        return self._stopDB()
