/*
JS code for folder listing of HTTP service.

HTTP code should minimize the usage of JS.

There is a lot of progressive things going on here.
When JS is enabled some features are disabled while others are enabled.
Check __init__
*/

/*
 Called after document is loaded.
*/
var __init__ = function() {
    patch_ie_logout_button()

    // Make the main content a dropzone.
    var dropzone = new Dropzone('div.dropzone-area', {
        // Setting maxFilesize to null has a bug and will limit at 256MB.
        // This is why we set a huge value.
        // Value is in MegaBytes.
        'maxFilesize': 1000000,
        // Have no timeout.
        'timeout': 0,
        'url': '.',
        'paramName': 'upload-file',
        'params': {
            'action': 'upload-file'
            },
        'headers': {
            'X-Upload-JS': 'v1'
        },
        'autoQueue': false,
        'parallelUploads': 1,
        'previewsContainer': '#dropzone-previews',
        'previewTemplate': document.querySelector(
            '#dropzone-template').innerHTML,
        'clickable': '.dz-add-files'
    })

    // When we have JS enabled we use the dropzone drag and drop upload.
    // Disable default form action, and insert the drag-and-drop compatible
    // action.
    var upload_selected = document.querySelector('#js-upload-selected-files')
    if (upload_selected) {
      upload_selected.type = 'button'
      upload_selected.onclick = function() {
          dropzone.enqueueFiles(dropzone.getFilesWithStatus(Dropzone.ADDED))
      }
    }

    // Hide/show button for js usage.
    $('#non-js-upload').hide()
    $('[data-js="upload-add-files"]').removeClass('hidden')
    $('[data-js="empty-add-files"]').removeClass('hidden')
    $('#js-drag-zone').removeClass('hidden')
    // Hide non-JS upload button as we hae JS active.
    $('#upload-selected-files').addClass('hidden')
    // Also hide the JS upload button, since at the start there is no file
    // to upload.
    $('#js-upload-selected-files').addClass('hidden')

    $('#js-clear-all-files').on('click', function() {
        dropzone.removeAllFiles(true)
        $('#js-clear-all-files').addClass('hidden')
    })

    // Called when a file is added.
    // Prepare the action for start.
    dropzone.on('addedfile', function(file) {
      file.previewElement.querySelector('.start-upload').onclick = function() {
          dropzone.enqueueFile(file)
      }

      if (file.fullPath) {
          // We have a recursive folder upload request.
          file.upload.filename = file.fullPath
          var node = file.previewElement.querySelectorAll("[data-dz-name]")
          node[0].textContent = file.fullPath
      }

      $('#js-clear-all-files').removeClass('hidden')
      $('#js-upload-selected-files').removeClass('hidden')
    })

    dropzone.on('removedfile', function(file) {
      if (dropzone.getAddedFiles().length > 0) {
        // Still have files to add.
        return
      }
      $('#js-clear-all-files').addClass('hidden')
      $('#js-upload-selected-files').addClass('hidden')
    })

    // Called before sending each file.
    // This can be used to update the multi-part.
    dropzone.on("sending", function(file, xhr, formData) {

      // If file is actually a folder
      if(file.fullPath){
        formData.append("fullPath", file.fullPath);
      }
    })

    // Called for each file which was uploaded.
    dropzone.on('success', function(file) {
        // The row which will be added, or updated if file already exists.
        var new_row
        var new_name = file.name
        if (file.fullPath) {
          new_name = file.fullPath
        }
        var existing_row = document.getElementById(new_name)

        if (existing_row == null) {
            // A row does not already exists.
            // Create one by cloning the first row.
            var first_row = $('#__row-template__')
            new_row = first_row.clone()
        } else {
            // There is already a row for this file.
            // File was just updated.
            new_row = $(existing_row)
        }

        new_row.addClass('new-file-row')

        // New row ID and select box.
        new_row.attr('id', new_name)
        new_row.find('input').attr('value', new_name)

        // New link.
        var new_link = new_row.find('.member-name a')
        var new_member = new_row.find('.member-name #js-member-name')
        new_link.attr('href', new_name)
        new_member.text(new_name)

        // Add size.
        var new_size = new_row.find('.member-size span')
        new_size.html(this.filesize(file.size))
        new_size.attr('title', file.size)

        // Add date.
        var new_date = new Date(file.lastModified)
        // On JS upload we don't reload the page,
        // We just show a "placeholder" member if it is a new file.
        // The placeholder member is also listed at the top of the file table
        // to make it easier for users to see that their file was uploaded.
        new_row.find('td.member-modified').text(
            new_date.toLocaleString('en-CA', {
                year: 'numeric',
                month: '2-digit',
                day: '2-digit',
                hour: '2-digit',
                hour12: false,
                minute: '2-digit',
                second: '2-digit'
                }
            )
        )

        if (existing_row == null) {
            // File was not already part of the folder.
            // Put it to the top of the listing for easy view.
            $('#folder-content-listing').prepend(new_row)
        }

        // Make sure the info about empty folder is not visible.
        $('#folder-content-listing-empty').hide()

    })

}


/*
Return IE version or False if not running on IE.
This function is a duplicate method here to support gen1 UI.
*/
var running_on_ie = function() {
  var user_agent = navigator.userAgent.toLowerCase()
  if (user_agent.indexOf('trident') != -1) {
     return true
  } else {
     return false
  }
}


/*
Patch logout button for IE 8 and above.

IE does not support username and password in URL [1], but luckily it
provides a dedicated JS command: `ClearAuthenticationCache`.
[1] https://support.microsoft.com/kb/834489
*/
var patch_ie_logout_button = function() {
  if (!running_on_ie()) {
    return
  }

  var logout_button = $('#button-logout')
  var path_parts = window.location.pathname.split('/')
  var no_credentials_url = (
    window.location.protocol + '//' + window.location.host +
    '/' + path_parts[1] + '/')
  logout_button.on('click', function() {
    try {
      // Call dedicated function and then go to start page.
      logout_result = document.execCommand('ClearAuthenticationCache')
      document.location.href = no_credentials_url
    } catch (e) {}
    if (!logout_result) {
      alert('Logout is not supported on your browser.')
    }
  })
}

$(document).ready(__init__)
