/*
All kind of hacks for JS used by both client and management UI.

This file should be loaded first.

It contains some polifyls for JS prototypes.

It also exports some functions, that should be moved into the `utils``
angular service.

This is also the place were some "improvements" are made to JS standard
library.
*/

if (typeof exports === 'undefined') {
    // We are in the browser.
    var require = function(node_path, browser_name) {return this[browser_name]}

    // Poor's man namespace initialization.
    this.services = {}
    this.controllers = {}
    this.mixins = {}

}


(function(exports){

/*
Return type of something after this table
Object  'object'
Array   'array' (js 'object')
Function    'function'
String  'string'
Number  'number'
Boolean 'boolean'
null    'null' (js 'object')
undefined   'undefined'

http://javascript.crockford.com/remedial.html
*/
exports.typeOf = function(value) {
    var s = typeof value
    if (s === 'object') {
        if (value) {
            if (Object.prototype.toString.call(value) == '[object Array]') {
                s = 'array'
            }
            // We can have a string and typeof report an "object", when
            // calling typeOf(this) from a prototype
            if (Object.prototype.toString.call(this) == '[object String]') {
                return 'string'
            }
        } else {
            s = 'null';
        }
    }
    return s
}


/*
True if something is empty.

Something can be: array, object, string

http://javascript.crockford.com/remedial.html
*/
exports.isEmpty = function(o) {

    var i, v;
    var type = exports.typeOf(o)
    if (type === 'object') {
        for (i in o) {
            v = o[i]
            if (v !== undefined && exports.typeOf(v) !== 'function') {
                return false
            }
        }
        return true
    } else if (type === 'string' || type === 'array') {
        return !o.length
    }

    throw new Error('isEmpty not defined for ' + type)
}


/*
Return length of object.
*/
exports.objectLength = function(input) {
    if (!input) {
        return 0
    }
    var input_type = exports.typeOf(input)

    if (input_type === 'array') {
        return input.length
    }

    if (input_type === 'string') {
        return input.length
    }

    if (input_type === 'object') {
        return Object.keys(input).length
    }

    return 0
}


/*
Helper function for Array.sort().

[{'id': 1}, {'id': 2}].sort(sort_by('id', true))
[{'id': '1'}, {'id': '2'}].sort(sort_by('id', true, parseInt))
[{'id': 'a'}, {'id': 'B'}].sort(sort_by('id', true, function(value) {
    return value.toUpperCase()))
*/
exports.sort_by = function(field, reverse, primer) {

    if (exports.typeOf(reverse) === 'undefined') {
        reverse = false
    }

    // Helper for getting key from object.
    var get_key = function(x) {
        return primer ? primer(x[field]) : x[field]
    }

    // Compare a and b.
    var compare = function(a, b) {
        if (a < b) {
            return -1
        }
        if (a > b) {
            return 1
        }
        return 0
    }

    return function(a, b) {
        var result = compare(get_key(a), get_key(b))
        if (reverse) {
            return result * -1
        } else {
            return result
        }
    }
}


/*
Calls toUpperCase in the context of a generic function.
*/
exports.toUpperCaseFn = function(value) {
    if (!value) {
        // No value.
        return value
    }
    if (!value.toUpperCase) {
        // Not a string.
        return value
    }
    return value.toUpperCase()
}


/*
String interpolation.

"%(media)slogo.gif".supplant({media: 'path/media/'}) -> "path/media/logo.gif

http://javascript.crockford.com/remedial.html
*/

if (!String.prototype.supplant) {
    String.prototype.supplant = function(o) {
        return this.replace(
            /%\(([^()]*)\)s/g,
            function(a, b) {
                var r = o[b]
                if (exports.typeOf(r) === 'null') {
                  return 'None'
                }

                return typeof r === 'string' || typeof r === 'number' ? r : a
            }
        )
    }
}

/*
Remove spaces from start and end of a string.

http://javascript.crockford.com/remedial.html
*/
if (!String.prototype.trim) {
    String.prototype.trim = function() {
        return this.replace(/^\s*(\S*(?:\s+\S+)*)\s*$/, '$1');
    }
}


/*
True if string starts with something.
*/
if (!String.prototype.startsWith) {
    String.prototype.startsWith = function(str) {
        return this.slice(0, str.length) === str
    }
}


/*
True if string ends with something.
*/
if (!String.prototype.endsWith) {
    String.prototype.endsWith = function(str) {
        var can_it_end_with = this.length >= str.length
        return can_it_end_with && this.substr(this.length - str.length) === str
    }
}


/*
Implementation of indexOf in case it is missing.
*/
if (!Array.prototype.indexOf) {
    Array.prototype.indexOf = function(
obj, fromIndex) {
        if (fromIndex === null) {
            fromIndex = 0;
        } else if (fromIndex < 0) {
            fromIndex = Math.max(0, this.length + fromIndex);
        }
        for (var i = fromIndex; i < this.length; i++) {
            if (this[i] === obj)
                return i;
        }
        return -1;
    };
}


if (!Array.prototype.lastIndexOf) {
    Array.prototype.lastIndexOf = function(
obj, fromIndex) {
        if (fromIndex === null) {
            fromIndex = this.length - 1;
        } else if (fromIndex < 0) {
            fromIndex = Math.max(0, this.length + fromIndex);
        }
        for (var i = fromIndex; i >= 0; i--) {
            if (this[i] === obj)
                return i;
        }
        return -1;
    }
}

Array.prototype.contains = function(
obj) {
    return this.indexOf(obj) != -1;
}

String.prototype.contains = function(
obj) {
    return this.indexOf(obj) != -1;
}


Array.prototype.insertAt = function(
obj, i) {
    this.splice(i, 0, obj)
}


Array.prototype.removeAt = function(
i) {
    this.splice(i, 1)
}


Array.prototype.remove = function(
obj) {
    var i = this.indexOf(obj)
    if (i != -1) {
        this.splice(i, 1)
    }
}

Array.prototype.extend = function (other_array) {
    for (var i = 0; i < other_array.length; i++) {
        this.push(other_array[i])
    }
}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.init = {} : exports))
