/*
* Tests for service initialization.
*
*/

describe('commons.__init__', function() {

  var init = require('./__init__.js')

    /*
    As a side-efect it will update the prototy for array to include exted
    */
    test('array_extend', function() {
        var sut = [1, 5, 9]
        sut.extend([3, 4, 8, 10])
        expect(sut).toEqual([1, 5, 9, 3, 4, 8, 10])
    })

  describe('typeOf()', function() {

    var typeOf = init.typeOf

    test('Objects are "object".', function() {
      expect(typeOf({})).toBe('object')
    })

    test('Arrays are "array".', function() {
      expect(typeOf([])).toBe('array')
    })

    test('null is "null".', function() {
      expect(typeOf(null)).toBe('null')
    })

    test('Undefined properties are "undefined".', function() {
      var test_object = {}
      expect(typeOf(test_object.no_such)).toBe('undefined')
    })

    test('Function is "function".', function() {
      function test_type() {}
      expect(typeOf(function() {})).toBe('function')
      expect(typeOf(test_type)).toBe('function')
    })

    test('String is "string".', function() {
      expect(typeOf('')).toBe('string')
      expect(typeOf('a')).toBe('string')
    })

    test('Number is "number".', function() {
      expect(typeOf(1)).toBe('number')
      expect(typeOf(1.1)).toBe('number')
    })

    test('Boolean is "boolean".', function() {
      expect(typeOf(true)).toBe('boolean')
      expect(typeOf(false)).toBe('boolean')
    })
  })


  describe('isEmpty()', function() {

    var isEmpty = init.isEmpty

    test('True for empty arrays.', function() {
      expect(isEmpty([])).toBeTrue()
    })

    test('False for not empty arrays.', function() {
      expect(isEmpty([''])).toBeFalse()
      expect(isEmpty([1])).toBeFalse()
    })

    test('True for empty strings.', function() {
      expect(isEmpty('')).toBeTrue()
    })

    test('False for not empty strings.', function() {
      expect(isEmpty('a')).toBeFalse()
    })

    test('True for non objects.', function() {
      expect(isEmpty({})).toBeTrue()
    })

    test('True for objects with only function definitions.', function() {
      var object = {f: function() {}}
      expect(isEmpty(object)).toBeTrue()
    })

    test('False for non objects.', function() {
      expect(isEmpty({a: 'b', c: function() {}})).toBeFalse()
    })

  })


  describe('String.supplant()', function() {

    test(
    'Does not change text with no placeholders.',
    function() {
      expect('abc'.supplant({})).toBe('abc')
      expect('abc%(def)s'.supplant({})).toBe('abc%(def)s')
      expect('abc'.supplant({abc: 'cda'})).toBe('abc')
    })

    test(
    'Update text with placeholders.',
    function() {
      expect('c%(def)sb'.supplant({'def': null})).toBe('cNoneb')
      expect(
        '%(abc)slogo%(def)s bla'.supplant({abc: 'ABC', def: 'DEF'})
        ).toBe(
        'ABClogoDEF bla')
    })

  })


  describe('String.trim()', function() {

    test('Does not change text with spaces.', function() {
      expect('abc'.trim()).toBe('abc')
    })

    test('Trims spaces.', function() {
      expect('    abc    '.trim()).toBe('abc')
    })

  })


  describe('String.startsWith()', function() {

    test('True if starts with empty string .', function() {
      expect('something'.startsWith('')).toBeTrue()
    })

    test('True if starts with less than string.', function() {
      expect('something'.startsWith('some')).toBeTrue()
    })

    test('True if starts with same string.', function() {
      expect('something'.startsWith('something')).toBeTrue()
    })

    test('False if starts with more.', function() {
      expect('something'.startsWith('somethingmoe')).toBeFalse()
    })

  })


  describe('String.endsWith()', function() {

    test('True if ends with empty string .', function() {
      expect('something'.endsWith('')).toBeTrue()
    })

    test('True if ends with less than string.', function() {
      expect('something'.endsWith('thing')).toBeTrue()
    })

    test('True if ends with same string.', function() {
      expect('something'.endsWith('something')).toBeTrue()
    })

    test('False if ends with more.', function() {
      expect('something'.endsWith('moresomething')).toBeFalse()
    })

  })


  describe('sort_by', function() {

    var sort_by = init.sort_by

    test(
    'By default it will sort in ascending order.',
    function() {
      var result = [{'id': 1}, {'id': 3}, {'id': 4}, {'id': 2}]

      result.sort(sort_by('id'))

      expect(result[1]['id']).toBe(2)
      expect(result[3]['id']).toBe(4)
    })

    test(
    'Reverse sort order can be requested.',
    function() {
      var result = [{'id': 1}, {'id': 3}, {'id': 4}, {'id': 2}]

      result.sort(sort_by('id', true))

      expect(result[1]['id']).toBe(3)
      expect(result[3]['id']).toBe(1)
    })

    test(
    'A custom function can be used for getting sort value.',
    function() {
      var result = [{'id': '1'}, {'id': '111'}, {'id': '2'}, {'id': '11'}]

      result.sort(sort_by('id', false, parseInt))

      expect(result[1]['id']).toBe('2')
      expect(result[3]['id']).toBe('111')
    })

  })


    describe('objectLength', function() {

        var objectLength = init.objectLength

        test(
        'Return 0 for undefined or types without length.',
        function() {
            expect(objectLength()).toBe(0)
            expect(objectLength(2)).toBe(0)
        })

        test(
        'Return 0 for empty object.',
        function() {
            expect(objectLength('')).toBe(0)
            expect(objectLength([])).toBe(0)
            expect(objectLength({})).toBe(0)
        })

        test(
        'Return length for non object.',
        function() {
            expect(objectLength('abc')).toBe(3)
            expect(objectLength([1, 2, 3, 5])).toBe(4)
            expect(objectLength({'k': 'v', 'a': 'b'})).toBe(2)
        })

    })

})
