/*
Custom Jest matchers used by the chevah project.
*/

/*
Expects that the angular element to have attribute with optional value.
*/
expect.extend({toHaveAttribute(element, name, expected_value) {
    if (!name) {
        throw new Error('an attribute name to check is required')
    }

    var value = element.attr(name)
    if (!value) {
        return {
            pass: false,
            message: `Expected ${name} attribute.`,
        }
    }

    if (!expected_value) {
        return {
            pass: true,
            message: `Expected ${name} attribute exists.`,
        }
    }

    if (expected_value != value) {
        return {
            pass: false,
            message: `Expected ${expected_value} attribute got ${value}.`,
        }
    }
    return {
        pass: true,
        message: `Expected ${expected_value} attribute valid.`,
    }


},})

/*
Expects that the angular element to have CSS classes.
*/
expect.extend({toHaveClass(element, expected_value) {
    if (!expected_value) {
        throw new Error('at least one CSS class is required')
    }

    var css_values = element.attr('class')
    if (!css_values) {
        return {
            pass: false,
            message: `No CSS class.`
        }
    }

    var expected_css = expected_value.split(' ')
    try {
        expect(css_values.split(' ')).toEqual(
            expect.arrayContaining(expected_css))
    } catch (error) {
        return {
            pass: false,
            message: function() { return `CSS ${expected_css} not found.`}
        }
    }
    return {
        pass: true,
        message: `CSS ${expected_css} found.`
    }

},})

/*
Expects that the angular element has no hidden CSS classe.
*/
expect.extend({toBeVisible(element) {

    try {
        expect(element).not.toHaveClass('hidden')
        expect(element).not.toHaveClass('ng-hide')
        expect(element).not.toHaveClass('hide')
    } catch (error) {
        return {
            pass: false,
            message: function() {return 'Element is not visible.'}
        }
    }
    return {
        pass: true,
        message: 'Element is visible.'
    }

},})
