(function(exports){

/*
Used for the footer of a table.

It takes 2 objects:

* table-ui - for UI config part
* table-options - for the JSON-RPC requests.

<table-footer
  table-ui="table_ui"
  table-options="table_options"
  >
</table-footer>
*/
exports.TableFooterBindings = {
    // Things that are not exchanged with the JSON-API
    'tableUi': '=',
    // Things that are sent to the JSON-API.
    'tableOptions': '=',
}

exports.TableFooterCtrl = function() {

    var $ctrl = this

    $ctrl.onGoToPage = function(page) {
        if (page < 0) {
            var page = 0
        }
        $ctrl.tableOptions.page = page
    }

}

/*
This is a sort UI for a table.
It received as input an object with:

sort_options = {
    'default_column': 'DB_COLUMN_NAME,
    'default_order': 'desc | asc',
    'columns': [
        {
            'value': _ACCOUNT_COLUMN,
            'text': 'Username',
        },
    ],
    // These will be updated by the component and can be used by the
    // user controller to detect the current sort order and reset the sorting.
    'reset': function() {},
    'sort_up': {},
    'sort_down': {},
}

Usage:
<table-sort
    sort-options="sort_options"
    table-options="table_options"
    >
</table-sort>
*/
exports.TableSortBindings = {
    // This is defined as "input",
    // but the controller will update the input object with methods that
    // are used to control the sort behaviour.
    // * sort_up - called to trigger sort ascending
    // * sort_down - called to trigger sort descending.
    // * reset - called to trigger a reset of the sort criteria
    'sortOptions': '<',
    // Things that are sent to the JSON-API.
    'tableOptions': '=',
}

exports.TableSortCtrl = function() {

    var $ctrl = this

    // This is the reference to the Dropdown UI.
    // We start with a placeholder, while the real dropdown is injected
    // by the external dropdown JS code.
    $ctrl.sort_dropdown = {'hide': function() {}}
    // The available options in the UI.
    // These are expanded based on the input configuration.
    $ctrl.options = []

    $ctrl.$onInit = function() {

        angular.forEach($ctrl.sortOptions.columns, function(option) {
            option['type'] = 'col'
            option['active'] = false
            if (option['value'] == $ctrl.sortOptions.default_column) {
                option['active'] = true
            }
            $ctrl.options.push(option)
        })
        // Add the separator.
        $ctrl.options.push({
            'type': 'sep',
            'value': 'sep',
            'text': '',
            'active': false,
        })

        // Add sort order.
        $ctrl.sort_up = {
            'type': 'order',
            'value': 'asc',
            'text': 'Ascending',
            'active': $ctrl.sortOptions.default_order == 'asc',
        }
        $ctrl.sort_down = {
            'type': 'order',
            'value': 'desc',
            'text': 'Descending',
            'active': $ctrl.sortOptions.default_order != 'asc',
        }
        $ctrl.options.push($ctrl.sort_up)

        $ctrl.options.push($ctrl.sort_down)

        // Populate the parent input.
        $ctrl.sortOptions.reset = resetSorting
        $ctrl.sortOptions.sort_up = $ctrl.sort_up
        $ctrl.sortOptions.sort_down = $ctrl.sort_down
    }
    /*
    Called when the a sort option is selected.
    */
    $ctrl.onSort = function(sort_option) {
        // These should always be updated,
        // but initialized with some sane value just in case.
        var active_column = $ctrl.sortOptions.default_column
        var active_order = $ctrl.sortOptions.default_order

        angular.forEach($ctrl.options, function(option) {
            if (option.type == sort_option.type) {
                // We need to the current active
                option.active = option.value == sort_option.value
            }
            if (option.active) {
                if (option.type == 'order') {
                    active_order = option.value
                }
                if (option.type == 'col') {
                    active_column = option.value
                }
            }
        })
        $ctrl.tableOptions.sorting = {}
        $ctrl.tableOptions.sorting[active_column] = active_order
        $ctrl.sort_dropdown.hide()
    }

    var resetSorting = function() {
        $ctrl.onSort({
            'type': 'col',
            'value': $ctrl.sortOptions.default_column
            })
        $ctrl.onSort({
            'type': 'order',
            'value': $ctrl.sortOptions.default_order
        })    }


}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.table_ui = {} : exports))
