/*
 * The application handing the main admin page.
 */

/*
Main application definition.
*/
var app = angular.module(
  'app',
  [
    'flow',
    'ngCookies',
    'ngRoute',
    'ngSanitize',
    'ui.bootstrap'
  ]
  )


// While this is already global, we define it as dependency to avoid
// using the global value when possible.
if (window && window.jasmine) {
  /* c8 ignore start */
  // During unit test we use as simple value to make assertion simpler.
  // The real value is used in the Selenium test.
  app.constant('CACHE_BUSTER', 'TEST_BUSTER')
  /* c8 ignore end */
} else if (CONSTANTS.PRODUCT_VERSION.indexOf('.dev0') == -1) {
  /* c8 ignore start */
  // Most of the time we run the tests in DEV mode,
  // so this production branch is not tested that often.
  // It is do tested for the release PRs,
  // so we do have coverate, not not for every PR.
  // It should be fine.
  app.constant('CACHE_BUSTER', CONSTANTS.PRODUCT_VERSION)
  /* c8 ignore end */
} else {
  // For development, try to invalidate the cache as often as possible.
  app.constant('CACHE_BUSTER', CONSTANTS.PRODUCT_VERSION + Date.now())
}
// Use dayjs as a constant to make it easier to use it.
// Later we might want to have it as a service to make it easier to fake
// the time.
var dayjs = require('dayjs', 'dayjs')
if (typeof exports !== 'undefined') {
  // On nodejs days is loaded without any plugin.
  dayjs.extend(require('dayjs/plugin/relativeTime'))
}
app.constant('dayjs', dayjs)

// Services definition.
var utils = require('./services/utils.js', 'utils')
app.factory('utils', utils.Utils)
var ng_table_mixin = require('./services/ng_table_mixin.js', 'ng_table_mixin')
app.factory('ng_table_mixin', ng_table_mixin.Mixins)

/*
Add controller with `name` from `file`.
*/
var add_controller = function(name, file) {
  var ctrl_module = require('./controllers/' + file, name)
  app.controller(name, ctrl_module[name])
}

// Controllers definition.
add_controller('mainCtrl', 'main.js')
add_controller('headerCtrl', 'header.js')
add_controller('statusCtrl', 'status.js')
add_controller('serverCtrl', 'server.js')
add_controller('servicesCtrl', 'services.js')
add_controller('serviceCtrl', 'service.js')
add_controller('transferCtrl', 'transfer.js')
add_controller('transfersCtrl', 'transfers.js')
add_controller('accountsCtrl', 'accounts.js')
add_controller('accountCtrl', 'account.js')
add_controller('rolesCtrl', 'roles.js')
add_controller('roleCtrl', 'role.js')
add_controller('administratorsCtrl', 'administrators.js')
add_controller('administratorCtrl', 'administrator.js')
add_controller('authenticationCtrl', 'authentication.js')
add_controller('authenticationsCtrl', 'authentications.js')
add_controller('groupsCtrl', 'groups.js')
add_controller('groupCtrl', 'group.js')
add_controller('locationCtrl', 'location.js')
add_controller('locationsCtrl', 'locations.js')
add_controller('eventHandlersCtrl', 'event_handlers.js')
add_controller('eventHandlerCtrl', 'event_handler.js')
add_controller('resourceCtrl', 'resource.js')
add_controller('resourcesCtrl', 'resources.js')
add_controller('activityLogCtrl', 'past_events.js')
add_controller('reviewCtrl', 'review.js')
add_controller('reportsCtrl', 'reports.js')
add_controller('keyCertCtrl', 'key_cert.js')

var trial_warning = require(
    '../component/trial_warning.js', 'trial_warning')
app.component('trialWarning', {
    templateUrl: 'component/trial_warning.html',
    controller: trial_warning.TrialWarningCtrl,
    bindings: {}
})
var table_ui = require('../component/table_ui.js', 'table_ui')
app.component('tableFooter', {
  templateUrl: 'component/table_footer.html',
  controller: table_ui.TableFooterCtrl,
  bindings: table_ui.TableFooterBindings
})
app.component('tableSort', {
  templateUrl: 'component/table_sort.html',
  controller: table_ui.TableSortCtrl,
  bindings: table_ui.TableSortBindings
})


// Filters definition.
var filters = require('./services/filters.js', 'filters')

app.filter('confirmation', filters.confirmation)
app.filter('checkDefined', filters.checkDefined)
app.filter('enableState', filters.enableState)
app.filter('dictToArray', filters.dictToArray)
app.filter('length', filters.length)
app.filter('textDiff', filters.textDiff)
app.filter('cssForState', filters.cssForState)
app.filter('highlight', filters.highlight)
app.filter('fromNow', ['dayjs', filters.fromNow])
app.filter('iso8601', ['dayjs', filters.iso8601])
app.filter('componentType', ['utils', filters.componentType])

// Directive definition.
var directives = require('../directive/directives.js', 'directives')

// For log-rotate-on
app.directive('logRotateOn', directives.logRotateOn)
// For log-rotate-on-calendar-type
app.directive(
  'logRotateOnCalendarType', directives.logRotateOnCalendarType)
app.directive('integer', directives.integer)
// For float-or-integer
app.directive('floatOrInteger', directives.floatOrInteger)
// For default-empty
app.directive(
  'defaultEmpty',
  ['utils', directives.defaultEmpty])
// For loading-overlay
app.directive(
  'loadingOverlay', directives.loadingOverlay)
// For multi-line
app.directive(
  'multiLine', ['utils', directives.multiLine])
// For multi-line-ssv
app.directive(
  'multiLineSsv', ['utils', directives.multiLineSSV])
// For comma-separated-fields
app.directive(
  'commaSeparatedFields', ['utils', directives.commaSeparatedFields])
// For comma-separated-textarea
app.directive(
  'commaSeparatedTextarea',
  ['utils', directives.commaSeparatedTextarea])
app.directive(
  'tagify', [
  'utils', directives.tagify])
// For ssl-ciphers
app.directive(
  'sslCiphers',
  ['utils', directives.sslCiphers])
// For input-trigger
app.directive(
  'inputTrigger',
  ['utils', '$timeout', directives.inputTrigger])
// For disable-overridden-options
app.directive(
  'disableOverriddenOptions',
  [directives.disableOverriddenOptions])
// For dropdown
app.directive('dropdown', ['utils', directives.dropdown])
// For config-seconds
app.directive(
  'configSeconds', ['utils', 'CACHE_BUSTER', directives.configSeconds])



// Route configuration.
app.config(function(
  $routeProvider,
  CACHE_BUSTER
) {


  var dash_to_camel = function(text) {
    return text.replace(
      /-([a-z])/g,
      function(m, w) {
        return w.toUpperCase()
        }
      )
    }

  var add_router = function(name, uuid, template) {
    var path = '/' + name
    if (uuid) {
        path += '/' + uuid
    }

    if (!template) {
        template = name
    }

    var template_url = 'partials/%(template)s.html?v=%(cache)s'.supplant({
        'template': template,
        'cache': CACHE_BUSTER
    })

    var controller = dash_to_camel(template)

    $routeProvider.when(path, {
        'templateUrl': template_url,
        'controller': controller + 'Ctrl'
    })

    // When runing the tests we don't want to always load the default router.
    if (window && window.jasmine) {
        $routeProvider.eagerInstantiationEnabled(false)
    }

  }

  add_router('status')
  add_router('server')
  add_router('server', ':tab')
  add_router('services')
  add_router('services', ':uuid', 'service')
  add_router('transfers')
  add_router('transfers', ':uuid', 'transfer')
  add_router('accounts')
  add_router('accounts', ':uuid', 'account')
  add_router('roles')
  add_router('roles', ':uuid', 'role')
  add_router('administrators')
  add_router('administrators', ':uuid', 'administrator')
  add_router('authentications')
  add_router('authentications', ':uuid', 'authentication')
  add_router('groups')
  add_router('groups', ':uuid', 'group')
  add_router('locations')
  add_router('locations', ':uuid', 'location')
  add_router('event-handlers')
  add_router('event-handlers', ':uuid', 'event-handler')
  add_router('activity-log')
  add_router('reports')
  add_router('resources')
  add_router('resources', ':uuid', 'resource')
  add_router('review')
  add_router('key-cert')

  $routeProvider.otherwise({redirectTo: '/status'})

  })
