(function(exports){

exports.accountCtrl = function($scope, $routeParams, dayjs, utils) {

    var sending_email = false

    var set_qrcode = function() {
        var value = $scope.account.multi_factor_authentication
        var qr_html = '<img src="/images/qrcode_blur.png">'

        var qrcode_element = document.getElementById('qrcode')
        if (!qrcode_element) {
            // We don't have a valid DOM.
            return
        }

        if (!value) {
            // No TOTP set.
            qrcode_element.innerHTML = ''
            return
        }

        if ($scope.totp_show) {
            // TOTP not requested to be visible.
            qr_html = get_qr(value).createImgTag(6)
        }
        qrcode_element.innerHTML = qr_html
    }


    /* Return a QR object for value */
    var get_qr = function(value) {
        var typeNumber = 0
        var errorCorrectionLevel = 'H'
        var qr = qrcode(typeNumber, errorCorrectionLevel)
        qr.addData(value)
        qr.make()
        return qr
    }

    /*
    The the list of home folder structures for this account.
    */
    var set_home_folder_structure = function() {
        $scope.home_folder_structure = []
        var all_groups = container.getGroups()
        angular.forEach($scope.account.group, function(group_uuid) {
            var group = all_groups[group_uuid]
            if (!group) {
                return
            }
            var home_folder_structure = group.home_folder_structure
            if (!home_folder_structure) {
                return
            }
            $scope.home_folder_structure.extend(group.home_folder_structure)
        })
    }

    $scope.show_disable_on_inactivity = true
    $scope.show_clear_password = false
    $scope.totp_show = false
    $scope.qrcode_toggle_text = 'Show secret'
    $scope.totp_base32 = 'Hidden'

    $scope.show_account_type = false

    $scope.disable_on_inactivity_group = ''
    $scope.file_write_size_group = ''
    $scope.password_lifetime_group = ''
    $scope.home_folder_structure = []

    $scope.groups_location = ''
    $scope.section_uuid = ''
    // Account primary group
    $scope.group = ''

    $scope.kinds = [
        {'type': 'application', 'name': 'Application Account'},
        {'type': 'os', 'name': 'Operating System Account'}
        ]

    $scope.account = {}
    // The place where accounts are stored.
    var container

    // Options and value for account.expire_datatime UI time and date picker.
    $scope.expire_datetime_ui = {
        // Widget options
        startingDay: 1,
        showWeeks: false,
        // Widget state.
        opened: false,
        // Widget model.
        // default to no date selected.
        date_value: null,
        time_value: null
    }

    /*
    Check changes in account type.
    */
    $scope.application_account = true

    $scope.$watch('account.type', function(current, old) {
        if (!current) {
            return
        }
        if (current === 'os') {
            delete $scope.account['password_update']
            delete $scope.account['last_password_update']
            // Is OS account.
            $scope.application_account = false
        } else {
            // Is APPLICATION account.
            $scope.application_account = true
        }
    })

    /*
    Check changes in account primary group selection.
    */
    $scope.$watch('account.group', function(current, old) {
        if (!current) {
            return
        }
        var primary_group_uuid = current[0]
        var primary_group = ''

        angular.forEach(
            $scope.getAllGroups(),
            function(value, key) {
                if (value.uuid == primary_group_uuid) {
                    primary_group = value
                }
            }
        )

        $scope.group = primary_group

        $scope.disable_on_inactivity_group = ''
        $scope.file_write_size_group = ''
        $scope.password_lifetime_group = ''

        if (utils.isInherited($scope.account.disable_on_inactivity)) {
            $scope.disable_on_inactivity_group = (
                $scope.group.disable_on_inactivity)
            if (!$scope.disable_on_inactivity_group) {
                $scope.disable_on_inactivity_group = 'unlimited'
            }
        }
        if (utils.isInherited($scope.account.file_write_size)) {
            $scope.file_write_size_group = $scope.group.file_write_size
            if (!$scope.file_write_size_group) {
                $scope.file_write_size_group = 'unlimited'
            }
        }
        if (utils.isInherited($scope.account.password_lifetime)) {
            $scope.password_lifetime_group = (
                $scope.group.password_lifetime)
            if (!$scope.password_lifetime_group) {
                $scope.password_lifetime_group = 'unlimited'
            }
        }

    })

    // Convert from custom text ISO format, to JS Date() object.
    // This is called once at init.
    // This is model -> view.
    var _get_date = function(iso_text) {
        if (!iso_text) {
            // The model has empty string, while the UI needs null.
            return null
        }
         // maybe using dayjs to parse the low-level config.
        return new Date(iso_text)
    }

    // Called every time the date view is updated for date.
    // Covert view -> model.
    $scope.$watch('expire_datetime_ui.date_value', function(current, old) {
        if (current == old) {
            // Nothing changed.
            return
        }

        if (!current) {
            // Date disabled.
            // Update the model and the time UI.
            $scope.account.expire_datetime = ''
            $scope.expire_datetime_ui.time_value = null
            return
        }

        var existing_datetime = $scope.expire_datetime_ui.time_value
        if (existing_datetime) {
            // Use the configured hours/second/minutes.
            current.setHours(existing_datetime.getHours())
            current.setMinutes(existing_datetime.getMinutes())
            current.setSeconds(existing_datetime.getSeconds())
        }

        // Upate the model.
        $scope.account.expire_datetime = current.toISOString()
    })

    // Called every time the time view is updated for time.
    // Covert view -> model.
    $scope.$watch('expire_datetime_ui.time_value', function(current, old) {
        if (current == old) {
            // Nothing changed.
            return
        }

        var datetime = $scope.expire_datetime_ui.date_value
        if (!datetime) {
            // No date select yet.
            // Don't update anything yet.
            $scope.account.expire_datetime = ''
            return
        }

        if (!current) {
            // Time disabled, but we have a date.
            // Set the start of the day.
            datetime.setHours(0)
            datetime.setMinutes(0)
            datetime.setSeconds(0)
        } else {
            // Sync the date UI with latest time.
            datetime.setHours(current.getHours())
            datetime.setMinutes(current.getMinutes())
            datetime.setSeconds(current.getSeconds())
        }

        // Update the model.
        $scope.account.expire_datetime = datetime.toISOString()
    })

    $scope.password_meter = {
        'score': 0
    }

    $scope.$watch(
        'account.password_update',
        utils.setPasswordMeter($scope, 'account')
        )

    $scope.$watch('account.multi_factor_authentication', set_qrcode)

    $scope.__init__ = function() {
        var search = $scope.getEntities('accounts')
        var current_account
        var current_groups

        if (search.length > 0) {
            container = $scope.$parent.component_categories.accounts

            current_account = $scope.$parent._findEntity(
                search, $routeParams.uuid)
            current_groups = 'groups'

            if (current_account && !current_account.uuid) {
                // We have an account in main configuration which is not yet
                // created.
                // So we show the type selection.
                $scope.show_account_type = true
            }
        }

        if (!current_account) {
            // Account is not in main configuration.
            // Search in one of the authentication methods.
            angular.forEach(
                $scope.$parent.configuration.authentications,
                function(value, key) {
                    if (current_account) {
                        // Stop searching for more accounts.
                        return
                    }
                    search = $scope.getEntities(
                      'authentications/' + value.uuid + '/accounts')
                    current_groups = 'authentications/' + value.uuid + '/groups'
                    current_account = $scope.$parent._findEntity(
                        search, $routeParams.uuid)
                    container = $scope._getIdentityContainer(value, 'accounts')
                    $scope.show_disable_on_inactivity = false
                })
        }

        if (!current_account) {
            // No account with this URL.
            $scope.goToPage('/accounts')
            return
        }

        $scope.$parent.component = current_account
        $scope.account = current_account
        $scope.groups_location = current_groups
        $scope.expire_datetime_ui.date_value = _get_date(
            $scope.account.expire_datetime)
        $scope.expire_datetime_ui.time_value = _get_date(
            $scope.account.expire_datetime)

        var password_update_details
        if (!current_account.last_password_update) {
            password_update_details = (
                'Date and time when password was last updated is unknown.')
        } else {
            password_update_details = (
                'Password was last updated on ' +
                dayjs.unix(current_account.last_password_update).format(
                    'YYYY-MM-DD HH:mm:ss')
                )

        }
        $scope.last_password_update_description = password_update_details

        var uuid = current_account.uuid
        if (uuid) {
            $scope.section_uuid = 'accounts/' + uuid
        }

        set_home_folder_structure()

        set_qrcode()
    }

    /*
    Return all the groups that this account can use.
    */
    $scope.getAllGroups = function() {
        var result = []
        angular.forEach(
          $scope.getEntities($scope.groups_location),
          function(value, key) {
            result.push(value)
          })

        return result
    }

    /*
    Delete an account.
    */
    $scope.onDeleteAccount = function() {
        container.delete($scope.account)
        $scope.goToPage('/accounts')
    }


    $scope.onGenerateSecret = function() {
        var server_name = $scope.configuration.server.name
        var new_secret = utils.get_new_otpauth_url(
            $scope.account.name, server_name)
        $scope.account.multi_factor_authentication = new_secret
        if ($scope.totp_show) {
            $scope.totp_base32 = new_secret
        }
    }

    $scope.onDisableQRCode = function() {
        $scope.account.multi_factor_authentication = ''
        $scope.totp_base32 = 'No TOTP secret defined.'
    }

    $scope.onToggleQRCode = function() {
        $scope.totp_show = !$scope.totp_show

        if (!$scope.totp_show) {
            $scope.qrcode_toggle_text = 'Show secret'
            $scope.totp_base32 = 'TOTP hidden'
        } else {
            $scope.qrcode_toggle_text = 'Hide secret'
            $scope.totp_base32 = (
                $scope.account.multi_factor_authentication
                )
        }

        if (!$scope.account.multi_factor_authentication) {
            $scope.totp_base32 = 'No TOTP secret defined.'
        }
        set_qrcode()
    }

    // Return the email for the current account.
    // Return empty if no email is defined.
    var get_email = function() {

        if (!$scope.account) {
            return ''
        }

        if ($scope.account.email) {
            // We have a dedicated email address.
            return $scope.account.email
        }

        if (!$scope.account.name) {
            return ''
        }

        if ($scope.account.name.contains('@')) {
            // Account name looks like an email.
            return $scope.account.name
        }
        return ''
    }

    // Return True if TOTP code can't be sent.
    $scope.blockTOTPEmail = function() {

        if (!$scope.account) {
            return true
        }

        if (!get_email()) {
            return true
        }
        if (!$scope.account.multi_factor_authentication) {
            return true
        }
        return false
    }

    // Return True if password can't be sent.
    $scope.blockPasswordEmail = function() {
        if (!$scope.account) {
            return true
        }

        if (!get_email()) {
            return true
        }

        if (!$scope.account.password_update) {
            return true
        }

        return false
    }

    /*
    Called when the credentails shoudl be sent over email.
    */
    $scope.onEmailCredentials = function() {

        if (sending_email) {
            return
        }
        sending_email = true

        var attachments = {}
        var name = $scope.account.name
        var recipient = get_email()
        var totp_code = $scope.account.multi_factor_authentication
        var password = $scope.account.password_update
        var password_message = ''
        var totp_message = ''

        if (totp_code) {
            totp_message = (
                '\r\n\r\nThe TOTP secret for your account is: \r\n\r\n' +
                totp_code +
                '\r\n\r\n' +
                'The QRcode is attached to this email.\r\n\r\n'
                )
            var qr_data = get_qr(totp_code).createDataURL(6)
            attachments = {'totp-code': qr_data}
        }

        if (password) {
            password_message = (
                'The password is: ' + password + '\r\n\r\n'
                )
        }

        var processing_alert = {
            'type': 'info',
            'msg': 'Sending email...'
        }

        $scope.alerts.push(processing_alert)

        utils.jsonrpc({
            'method': 'send_email',
            'params': {
                'to': [recipient],
                'subject': 'Credentials for your file transfer account',
                'body': (
                    'This message should include credentials for your ' +
                    'account %(account)s on the file transfer server.' +
                    '\r\n\r\n' + password_message + totp_message +
                    '\r\n-- \r\n' +
                    'Automated email sent by your file transfer administrator.'
                    ).supplant({'account': name}),
                'attachments': attachments
                },
            /*
            jscpd:ignore-start

            The error handling is similar to other email sending code.
            It should be ok for now.
            */
            'success': function(data, status) {
                sending_email = false
                $scope.alerts.remove(processing_alert)
                },
            'error': function(data, status) {
                sending_email = false
                $scope.clearAlerts()
                $scope.alertError(data['error'])
                },
            'alerts': $scope.alerts
            /* jscpd:ignore-end */
        })

    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.accountCtrl = {} : exports))
