(function(exports){

exports.accountsCtrl = function($scope, utils, $timeout) {

    var init = require('../../../js_commons/__init__.js', 'init')
    // The current list of filtered and sorted accounts
    // Will be initialized later.
    $scope.filtered_accounts = []
    // Used by the search input tag.
    $scope.filter_input = ''

    $scope.tabs = []

    $scope.$parent.component = {
        'name': 'Accounts',
        'summary': (
            'The accounts are used for accessing the file transfer services.'),
    }

    var update_tabs = function(current, old) {
        $scope.tabs = $scope.createIdentityTabs('accounts')
    }
    $scope.$watch('process.authentications', update_tabs, true)

    // Handle tab switching.
    $scope.active_tab = 0
    $scope.onTabSelect = function(tab_index) {
        $scope.active_tab = tab_index
        $scope.filter_input = ''
        $scope.__init__()
    }

    /*
    This is called each time the page is accessed,
    including when using the browser back/forward.
    */
    $scope.__init__ = function() {
        if ($scope.redirectWithoutConfiguration()) {
            // FIXME:5878:
            // The page can only be loaded when the config is already loaded,
            // due to external authentication tabs.
            return
        }
        // Trigger an initial filter that will show all.
        $scope.onFilterAccounts($scope.filter_input)

        // The configuration might not be loaded so we force a dumb reload
        // We should find a better way to auto-reload the page but for
        // now it should be ok.
        // In an ideal case, we should use a watch on the configuration,
        // but that might have some side-effects and performance issues.
        // Most of the times, users should not land directly on the accounts
        // page, so this hack should be ok.
        if (init.isEmpty($scope.filtered_accounts)) {
            $timeout(function() {$scope.onFilterAccounts()}, 3000)
        }
    }

    /*
    Called the update the list of accounts to be displayed on the page.
    */
    $scope.onFilterAccounts = function(filter_value) {
        var view_model
        $scope.filtered_accounts = []

        if (!$scope.configuration.accounts) {
            // Configuration not yet loaded.
            return
        }
        if (!$scope.tabs) {
            // Configuration not yet loaded.
            return
        }

        var current_tab = $scope.tabs[$scope.active_tab]
        if (!current_tab) {
            // Tabs not yet loaded.
            return
        }

        // Set tab
        $scope.tab = current_tab
        var current_accounts = $scope.getEntities(current_tab.accounts_path)

        angular.forEach(
            current_accounts,
            function(account) {
                if (isFilteredOut(filter_value, account, current_tab)) {
                    return
                }
                view_model = getAccountView(account)
                $scope.filtered_accounts.push(view_model)
            })
    }

    /*
    Update the `accounts` based on the current filter.
    */
    var getAccountView = function(configuration) {
        var uuid = configuration.uuid

        return {
            'uuid': uuid,  // To help with testing to identify the row.
            'config': configuration,
            'details': getAccountDetails(configuration),
            'group_names': getAccountGroups(configuration),
        }
    }

    /*
    Return the list of details to show for an account on the index page.
    */
    var getAccountDetails = function(configuration) {
        return [
            {
                'key': 'Type',
                'value': getAccountType(configuration)
            },
            {
                'key': '',
                'value': configuration.description
            },
            {
                'key': 'Source IPs',
                'value': resolveSourceIPs(
                    configuration.group, configuration.source_ip_filter)
            },
            {
                'key': 'Home folder',
                'value': configuration.home_folder_path
            },
            {
                'key': 'TOTP enabled',
                'value':
                    configuration.multi_factor_authentication ? 'Yes' : 'No'
            },
            {
                'key': 'Password expiration',
                'value':
                    configuration.expire_datetime
            },
            {
                'key': 'SSH keys',
                'value':
                    configuration.ssh_authorized_keys_content ? 'Yes' : 'No'
            },
            {
                'key': 'SSL cert',
                'value': resolveSSLCert(
                    configuration.allow_certificate_authentication,
                    configuration.group)
            },
        ]
    }


    /*
    Return true if the entry should be filtered out.
    */
    var isFilteredOut = function(filter_value, configuration, tab) {
        if (!filter_value) {
            return false
        }

        if (utils.containsInsensitive(configuration.name, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(
                configuration.description, filter_value)) {
            return false
        }

        // Account might be configured without a group.
        if (configuration.group && utils.containsInsensitive(
                $scope.getGroupName(
                    tab.container.getGroups, configuration.group[0]
                ),
                filter_value)) {
            return false
        }

        return true
    }

    /*
    Return the allowed source IPs for each group associated with an account.

    FIXME:5729:
    Better join for multi lines.
    Share this code with roles
    */
    var resolveSourceIPs = function(groups, ips) {
        var result = []

        // Add direct IPs
        angular.forEach(ips, function(value) {
            result.push(value[0] + '-' + value[1])
        })

        angular.forEach(groups, function(group_uuid) {
            var group = $scope.resolveGroup(group_uuid)
            angular.forEach(group.source_ip_filter, function(value) {
                result.push('(inherited)' + value[0] + '-' + value[1])
            })
        })

        return result.join(' ')
    }

    /*
    Return SSL cert value.
    */
    var resolveSSLCert = function(value, groups) {
        var result = value ? 'Yes' : 'No'

        if (value == 'inherit' && groups) {
            result = ['(inherited)']
            var group = $scope.resolveGroup(groups[0], $scope.tab)
            var v = group.allow_certificate_authentication ? 'Yes' : 'No'
            result.push(v)
            return result.join(' ')
        }

        return result
    }

    /*
    Return the group names of each account.
    */
    var getAccountGroups = function(configuration) {
        var group_names = []
        angular.forEach(
            configuration.group,
            function(uuid) {
                var group = $scope.resolveGroup(uuid, $scope.tab)
                var group_text = group.name
                if (!group_text) {
                    // If group has no text, use UUID
                    group_text = group.uuid
                }
                if (!group_text) {
                    // If no text or UUID just skip.
                    return
                }
                group_names.push(group_text)
            }
        )
        return [
            {
                'key': '',
                'value': group_names.join(', ')
            }
        ]
    }

    // Delete the account from the `container` indentity holder.
    $scope.onDeleteAccount = function(container, config) {
        container.delete(config)
        $scope.onFilterAccounts($scope.filter_input)
    }

    /*
    Can add an account, we have at least 1 group to add them to.
    */
    $scope.haveAddAccount = function(container) {
        var target = container.name.replace('accounts', 'groups')
        return $scope.getEntities(target).length > 0
    }

    /*
    Create a new account configuration.
    Set the default value in the fields that need one.
    */
    $scope.onAddAccount = function(tab) {
        var account = {}
        var container = tab.container
        account['name'] = utils.getRandomMark('New Account ')

        account['enabled'] = true

        angular.forEach(
            utils.constants['CONFIGURATION_ACCOUNT_DEFAULTS'],
            function(value, key) {
                // We skip password field.
                if (key === 'password') {
                    return
                }
                // Default boolean values are stored as string.
                value = utils.maybeParseBoolean(value)
                account[key] = value
            })

        // `type` field is not part of the interface but is
        // required for creating a new account.
        account['type'] = 'application'
        account['group'] = tab.default_group_uuids
        account['source_ip_filter'] = []
        account['permissions'] = [['inherit']]
        account['virtual_folders'] = [['inherit']]
        account['allow_certificate_authentication'] = 'inherit'
        account['amend_write_name'] = 'inherit'
        account['create_home_folder'] = 'inherit'
        account['lock_in_home_folder'] = 'inherit'
        account['description'] = ''
        account['as2_require_http_authentication'] = 'inherit'
        account['required_credentials'] = ['inherit']
        account['expire_datetime'] = ''
        account['disable_on_inactivity'] = 'inherit'
        account['file_write_size'] = 'inherit'
        account['password_lifetime'] = 'inherit'

        container.add(account)
        container.goToConfiguration(account)
    }

    /*
    Return the type for account.

    Existing accounts use the read_only `kind_name` while new accounts use
    the write only 'type'.
    */
    var getAccountType = function(account) {
        if (account['kind_name'] == 'os') {
            return account['kind_name'].toUpperCase()
        }
        return ''
    }

    /*
    Return the name of group with `uuid`.
    */
    $scope.getGroupName = function(group_getter, uuid) {
        groups = group_getter()

        if (!groups) {
            return 'UNKNOWN'
        }

        var group = groups[uuid]

        if (!group) {
            return 'UNKNOWN'
        }

        return group['name']
    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.accountsCtrl = {} : exports))
