(function(exports){

exports.administratorCtrl = function(
    $scope,
    $routeParams,
    utils
) {
    var category = $scope.$parent.component_categories.administrators

    var set_qrcode = function() {
        var value = $scope.administrator.multi_factor_authentication
        var qr_html = '<img src="/images/qrcode_blur.png">'
        if (!value) {
            // This code should be refactored not to touch the element,
            // but instead use angularjs to update the page.
            // No TOTP set.
            var element = document.getElementById('qrcode')
            if (!element) {
                return
            }
            element.innerHTML = ''
            return
        }

        if ($scope.totp_show) {
            // TOTP not requested to be visible.
            var typeNumber = 0
            var errorCorrectionLevel = 'H'
            var qr = qrcode(typeNumber, errorCorrectionLevel)
            qr.addData(value)
            qr.make()
            qr_html = qr.createImgTag(6)
        }
        document.getElementById('qrcode').innerHTML = qr_html
    }

    $scope.show_clear_password = false
    $scope.totp_show = false
    $scope.qrcode_toggle_text = 'Show secret'
    $scope.totp_base32 = 'Hidden'

    $scope.administrator = {}

    $scope.password_meter = {
        'score': 0
    }
    $scope.$watch(
        'administrator.password_update',
        utils.setPasswordMeter($scope, 'administrator')
        )

    $scope.$watch(
        'administrator.multi_factor_authentication',
        set_qrcode
        )

    $scope.__init__ = function() {
        var all_administrators = $scope.$parent.getAllAdministrators()
        if (all_administrators.length < 1) {
            $scope.goToPage('/')
            return
        }
        var current_administrator = $scope.$parent._findEntity(
                all_administrators, $routeParams.uuid)

        if (!current_administrator) {
            $scope.$parent.goToPage('/administrators')
            return
        }
        $scope.is_delete_blocked = utils.isDeleteBlocked(
            current_administrator, 'administrator')

        $scope.administrator = current_administrator
        $scope.$parent.component = current_administrator

        set_qrcode()
    }

    /*
    Go to role configuration page.
    */
    $scope.onGoToRoleConfiguration = function(role_uuid) {
        var category = $scope.$parent.component_categories.roles
        category.goToConfiguration({'uuid': role_uuid})
    }

    /*
    Delete an administrator.
    */
    $scope.onDeleteAdministrator = function() {
        category.delete($scope.administrator)
        $scope.goToPage('/administrators')
    }

    $scope.onGenerateSecret = function() {
        var server_name = $scope.configuration.server.name
        var new_secret = utils.get_new_otpauth_url(
            $scope.administrator.name, server_name)
        $scope.administrator.multi_factor_authentication = new_secret
        if ($scope.totp_show) {
            $scope.totp_base32 = new_secret
        }
    }

    // jscpd:ignore-start
    $scope.onDisableQRCode = function() {
        $scope.administrator.multi_factor_authentication = ''
        $scope.totp_base32 = 'No TOTP secret defined.'
    }

    $scope.onToggleQRCode = function() {
        $scope.totp_show = !$scope.totp_show

        if (!$scope.totp_show) {
            $scope.qrcode_toggle_text = 'Show secret'
            $scope.totp_base32 = 'TOTP hidden'
        } else {
            $scope.qrcode_toggle_text = 'Hide secret'
            $scope.totp_base32 = (
                $scope.administrator.multi_factor_authentication
                )
        }

        if (!$scope.administrator.multi_factor_authentication) {
            $scope.totp_base32 = 'No TOTP secret defined.'
        }
        set_qrcode()
    }
    // jscpd:ignore-end


}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.administratorCtrl = {} : exports))
