/*
Tests for administrator edit page controller.
*/
describe('AdministratorCtrl', function() {


    /*
    When on the page for a new admin that is not yet saved in the config,
    it will remove it from the pending changes.
    */
    test('onDeleteAdministrator_new', function() {
        var sut = mk.getController('administratorCtrl')

        sut.scope.$parent.configuration.administrators = {
            'some-old-uuid': {'name': 'admin in config'}}
        sut.scope.$parent.new_entities.administrators = [{
            'name': 'some-new-admin'
        }]
        sut.route.uuid = 'some-new-admin'
        sut.scope.__init__()

        sut.scope.onDeleteAdministrator()

        // The new admin is removed.
        expect(sut.scope.$parent.new_entities.administrators).toHaveLength(0)
        // Existing admin is left.
        expect(sut.scope.$parent.configuration.administrators).toContainKey(
            'some-old-uuid')
    })

    /*
    For an admin that is found in the config, it will remove it from the
    configuration object.
    */
    test('onDeleteAdministrator_existing', function() {
        var sut = mk.getController('administratorCtrl')

        sut.scope.$parent.configuration.administrators = {
            'some-old-uuid': {'name': 'old admin', 'uuid': 'some-old-uuid'}}
        sut.scope.$parent.new_entities.administrators = [{
            'name': 'some-new-admin'
        }]
        sut.route.uuid = 'some-old-uuid'
        sut.scope.__init__()

        sut.scope.onDeleteAdministrator()

        // The new admin is removed.
        expect(sut.scope.$parent.new_entities.administrators).toHaveLength(1)
        // Existing admin is left.
        expect(sut.scope.$parent.configuration.administrators).not.toContainKey(
            'some-old-uuid')
    })

})
