(function(exports){

    exports.administratorsCtrl = function($scope, utils, $timeout) {
    var category = $scope.$parent.component_categories.administrators
    $scope.$parent.component = {
        'name': 'Administrators',
        'summary': (
            "These are the users of the web manager console. " +
            "They only have access to the configuration and management of " +
            "SFTPPlus. They don't have access to the user's files."
        ),
    }
    $scope.filtered_administrators = []
    $scope.STATE = utils.constants['STATE']
    // Used by the search input tag.
    $scope.filter_input = ''

    $scope.__init__ = function() {
        // Trigger an initial filter that will show all.
        $scope.onFilterAdministrators()

        // The configuration might not be loaded so we force a dumb reload
        // We should find a better way to auto-reload the page but for
        // now it should be ok.
        // In an ideal case, we should use a watch on the configuration,
        // but that might have some side-effects and performance issues.
        // Most of the times, users should not land directly on the
        // administrators page, so this hack should be ok.
        if (init.isEmpty($scope.filtered_administrators)) {
            $timeout(function() {$scope.onFilterAdministrators()}, 3000)
        }
    }

    /*
    Called the update the list of administrators to be displayed on the page.
    */
    $scope.onFilterAdministrators = function(filter_value) {
        var view_model
        $scope.filtered_administrators = []

        if (!$scope.configuration.administrators) {
            // Configuration not yet loaded.
            return
        }

        angular.forEach(
            $scope.getAllAdministrators(),
            function(administrator) {
                if (utils.isComponentExcluded(filter_value, administrator)) {
                    return
                }
                view_model = getAdministratorView(administrator)
                $scope.filtered_administrators.push(view_model)
            })
    }

    /*
    Update the `administrators` based on the current filter.
    */
    var getAdministratorView = function(configuration) {
        var uuid = configuration.uuid

        return {
            'uuid': uuid,  // To help with testing to identify the row.
            'details': getAdministratorDetails(configuration),
            'role_names': getAdministratorRoles(configuration),
            'is_delete_blocked': utils.isDeleteBlocked(
                configuration, 'administrator'),
            'config': configuration,
        }
    }

    /*
    Return the details to be visible in the UI for each administrator type.
    */
    var getAdministratorDetails = function(configuration) {

        return [
            {
                'key': 'Source IPs',
                'value': resolveSourceIPs(
                    configuration.roles, configuration.source_ip_filter)
            },
            {
                'key': 'TOTP enabled',
                'value':
                    configuration.multi_factor_authentication ? 'Yes' : 'No'
            }
        ]
    }

    /*
    Return the role names of each administrator.
    */
    var getAdministratorRoles = function(configuration) {
        var role_names = []
        angular.forEach(
            configuration.roles,
            function(uuid) {
                var role = $scope.resolveRole(uuid)
                var role_text = role.name
                if (!role_text) {
                    // If role has no text, use UUID
                    role_text = role.uuid
                }
                if (!role_text) {
                    // If no text or UUID just skip.
                    return
                }
                role_names.push(role_text)
            }
        )
        return [
            {
                'key': '',
                'value': role_names.join(', ')
            }
        ]
    }

    /*
    Return the allowed source IPs for each administrator role associated with
    an administrator.

    FIXME:5729:
    Better join for multi lines.
    */
    var resolveSourceIPs = function(roles, ips) {
        var result = []

        // Add direct IPs
        angular.forEach(ips, function(value) {
            result.push(value[0] + '-' + value[1])
        })

        angular.forEach(roles, function(role_uuid) {
            var role = $scope.resolveRole(role_uuid)
            angular.forEach(role.source_ip_filter, function(value) {
                result.push('(inherited)' + value[0] + '-' + value[1])
            })
        })

        return result.join(' ')
    }

    /*
    Create a new empty administrator configuration.
    */
    $scope.onAddAdministrator = function() {
        var administrator = {}
        administrator['name'] = utils.getRandomMark('New Administrator ')
        administrator['enabled'] = true
        administrator['roles'] = ['DEFAULT-ROLE']
        administrator['description'] = utils.maybeParseBoolean(
            utils.constants['CONFIGURATION_DEFAULTS']['description'])
        administrator['source_ip_filter'] = []
        administrator['multi_factor_authentication'] = ''
        administrator['node_variables'] = ''

        category.add(administrator)
        category.goToConfiguration(administrator)
    }

    /*
    Go to configuration page.
    */
    $scope.onGoToConfiguration = function(administrator) {
        category.goToConfiguration(administrator)
    }

    /*
    Delete an administrator.
    */
    $scope.onDeleteAdministrator = function(administrator) {
        category.delete(administrator)
        // Update the table after one entry was removed.
        $scope.onFilterAdministrators($scope.filter_input)
    }
}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.administratorsCtrl = {} : exports))
