(function(exports){

    exports.authenticationsCtrl = function(
        $scope, utils, componentTypeFilter, $timeout) {

    var init = require('../../../js_commons/__init__.js', 'init')
    // The current list of filtered and sorted authentication
    // Will be initialized later.
    $scope.filtered_authentications = []
    // Used by the search input tag.
    $scope.filter_input = ''

    $scope.authentication_types = []
    // Link to the general server configuration section
    $scope.server = {}

    $scope.$parent.component = {
        'name': 'Authentications',
        'summary': 'Sources used to authenticate users.',
    }

    var details_getter = {}

    var category = $scope.$parent.component_categories.authentications

    /*
    This is called each time the page is accessed,
    including when using the browser back/forward.
    */
    $scope.__init__ = function() {

        if ($scope.redirectWithoutConfiguration()) {
            // If the full configuration is not ready we redirect to start
            // page since there are many things on this page
            // that require the full configuration to be ready.
            return
        }

        $scope.authentication_types = _getAvailableAuthenticationTypes()
        // Trigger an initial filter that will show all.
        $scope.onFilterAuthentications()

        $scope.server = $scope.configuration.server

        // The configuration might not be loaded so we force a dumb reload
        // We should find a better way to auto-reload the page but for
        // now it should be ok.
        // In an ideal case, we should use a watch on the configuration,
        // but that might have some side-effects and performance issues.
        // Most of the times, users should not land directly on the
        // authentications page, so this hack should be ok.
        if (init.isEmpty($scope.filtered_authentications)) {
            $timeout(function() {$scope.onFilterAuthentications()}, 3000)
        }
    }

    $scope.onGoToLocalManager = function() {
        var manager = $scope.configuration.services[
            utils.constants.DEFAULT_LOCAL_MANAGER_UUID]
        $scope.onGoToService(manager.uuid)
    }

    /*
    Return a list with authentication methods that are already available
    on the server.
    */
    $scope.getExistingAuthentications = function() {
        var result = []
        angular.forEach(
            $scope.getAllAuthentications(),
            function(authentication) {
                if (!authentication.uuid) {
                    return
                }
                result.push(authentication)
            })
        return result
    }

    /*
    Called the update the list of authentications to be displayed on the page.
    */
    $scope.onFilterAuthentications = function(filter_value) {
        var view_model
        $scope.filtered_authentications = []

        if (!$scope.configuration.authentications) {
            // Configuration not yet loaded.
            return
        }

        angular.forEach(
            $scope.getAllAuthentications(),
            function(authentication) {
                if (utils.isComponentExcluded(filter_value, authentication)) {
                    return
                }
                view_model = getAuthenticationView(
                    authentication, $scope.process.runnables.authentications
                    )
                $scope.filtered_authentications.push(view_model)
            })
    }

    /*
    Update the `authentications` based on the current filter.
    */
    var getAuthenticationView = function(configuration, runnables) {
        var view_model = utils.getBaseViewModel(configuration, runnables)
        var component_usage = isUsed(configuration.uuid)
        var is_delete_blocked = utils.isDeleteBlocked(
            configuration, 'authentication')
        if (!is_delete_blocked && component_usage.length > 0) {
            is_delete_blocked = 'Used by: ' + component_usage.join(', ') + '.'
        }

        return {
            'uuid': view_model.uuid,
            'details': getAuthenticationDetails(configuration, component_usage),
            'is_delete_blocked': is_delete_blocked,
            'config': configuration,
            'runnable': view_model.runnable,
            'just_created': view_model.just_created
        }
    }

    /*
    Return the details to be visible in the UI for each authentication type.

    Return empty list when no extra details are required for the UI.
    */
    var getAuthenticationDetails = function(configuration, component_usage) {

        var result = [{
            'key': 'Type',
            'value': componentTypeFilter(
                configuration.type, 'authentication'),
        }]

        var usage = 'Not used by any service'
        if (component_usage.length > 0) {
            usage = component_usage.join(', ')
        }

        result.push({
            'key': 'Usage',
            'value': usage
        })

        var getter = details_getter[configuration.type]
        if (getter) {
            Array.prototype.push.apply(
                result, getter(configuration))
        }

        return result
    }

    /*
    Return a list of component names for which the authentication is used.
    */
    var isUsed = function(uuid) {

        if (!uuid) {
            // Not yet created methods are not yet in use.
            return []
        }

        // This is alwasy called when we have the configuration.
        // No need for error handling here.
        var server = $scope.configuration.server

        var used_by_service = []

        if (server.authentications.contains(uuid)) {
            used_by_service.push('Default authentication')
        }

        angular.forEach(
            $scope.configuration.services,
            function(service) {
                if (service.authentications.contains(uuid)) {
                    used_by_service.push(service.name)
                }
            }
        )

        return used_by_service
    }

    details_getter.anonymous = function(configuration) {
        return [
            {'key': 'Account',
            'value': $scope.resolveAccount(
                configuration.anonymous_account_uuid).name
            }
        ]
    }

    details_getter['deny-username'] = function(configuration) {
        return [
            {'key': 'Usernames',
            'value': configuration.usernames
            }
        ]
    }

    details_getter['ip-time-ban'] = function(configuration) {
        return [
            {'key': 'Trigger',
            'value': configuration.ban_after_count
            },
            {'key': 'Duration',
            'value': configuration.ban_interval
            }
        ]
    }

    details_getter.http = function(configuration) {
        return [
            {'key': 'URL',
            'value': configuration.url
            },
            {'key': 'Proxy',
            'value': configuration. proxy
            }
        ]
    }

    details_getter.ldap = function(configuration) {
        return [
            {'key': 'Address',
            'value': configuration.address + ':' + configuration.port
            },
            {'key': 'Base DN',
            'value': configuration.bind_dn
            },
            {'key': 'Secure',
            'value': configuration.secure_connection ? 'Yes' : 'No'
            }
        ]
    }

    details_getter.radius = function(configuration) {
        return [
            {'key': 'Address',
            'value': configuration.address + ':' + configuration.port
            },
            {'key': 'Auth type',
            'value': configuration.authentication_type
            }
        ]
    }

    details_getter.application = function(configuration) {

        var group_names = []
        angular.forEach(
            configuration.allowed_groups,
            function(uuid) {
                var group = $scope.resolveGroup(uuid)
                var group_text = group.name
                if (!group_text) {
                    // If group has no text, use UUID
                    group_text = group.uuid
                }
                if (!group_text) {
                    // If no text or UUID just skip.
                    return
                }
                group_names.push(group_text)
            }
        )
        return [
            {'key': 'Allowed groups',
            'value': group_names.join(', ')
            }
        ]
    }

    details_getter.os = function(configuration) {
        // Here we have OS group names, not SFTPPlus group UUIDs.
        return [
            {'key': 'User groups',
            'value': configuration.allowed_groups
            },
            {'key': 'Admin groups',
            'value': configuration.manager_allowed_groups
            }
        ]
    }

    details_getter['local-file'] = function(configuration) {
        return [
            {'key': 'Path',
            'value': configuration.path
            },
            {'key': 'External',
            'value': configuration.external_management ? 'Yes' : 'No'
            }
        ]
    }

    /*
    Return default configuration for a method.
    */
    $scope._getDefault = function(handler_type) {
        var result
        var map = {
            'http': $scope._getDefaultHTTP,
            'ldap': $scope._getDefaultLDAP,
            'radius': $scope._getDefaultRADIUS,
            'local-file': $scope._getDefaultLocalFile,
            'entra-id': $scope._getDefaultEntraID,
            'os': $scope._getDefaultOS
            }[handler_type]

        if (map) {
            result = map()
        } else {
            result = {}
        }
        result['type'] = handler_type
        result['enabled'] = true
        return result
    }

    $scope._getDefaultHTTP = function() {
        var result = {
            'timeout': 120,
            'url': ['http://ws.example.org/auth-handler']
            }
        angular.extend(result, utils.constants.SSL_DEFAULTS)
        // The direct password field read only.
        delete result['ssl_key_password']
        return result
    }

    $scope._getDefaultLDAP = function() {
        var result = {
            'address': utils.constants.CONFIGURATION_DEFAULTS.address,
            'port': 389,
            'bind_dn': ['CN=Users,dc=domain,dc=example,dc=com'],
            'bind_dn_type': 'parent',
            'username_attribute': 'cn',
            'email_attribute': 'mail',
            'home_folder_attribute': ['homeDirectory']
            }
        return result
    }

    $scope._getDefaultEntraID = function() {
        var result = {
            'directory_id': 'YOUR-ENTRA-ID-DIRECTORY-ID',
            'application_id': 'YOUR-APP-ID',
            'base_groups': ['DEFAULT_GROUP'],
            'group_association': 'base-groups',
            'remove_username_suffix': [],
            'api_scopes': [],
            }
        return result
    }

    $scope._getDefaultRADIUS = function() {
        var result = {
            'address': 'radius.example.com',
            'port': 1812,
            'password_update': '',
            'authentication_type': 'ms-chap-v2',
            'nas_port': 0,
            'debug': false,
            'timeout': 60,
            'continue_authentication': false
            }
        return result
    }

    $scope._getDefaultOS = function() {
        var result = {
            'allowed_groups': ['example-os-group-name', 'other-example-group'],
            'manager_allowed_groups': [
                'admin-example-group3', 'admin-example-group4'],
            'base_groups': ['DEFAULT_GROUP'],
            'group_association': 'base-groups',
            'pam_usage': 'fallback',
            'pam_service': 'login'
            }
        return result
    }

    $scope._getDefaultLocalFile = function() {
        var result = {
            'path': 'path/to/some/local-file.ini',
            'external_management': true
            }
        return result
    }

    /*
    Called when add event button is triggered.
    */
    $scope.onAddAuthentication = function(method_type) {
        var authentication = {}

        authentication['name'] = 'New %(type)s method %(random)s'.supplant({
            'type': method_type,
            'random': utils.getRandomMark()
            })

        angular.extend(authentication, $scope._getDefault(method_type));

        category.add(authentication)
        category.goToConfiguration(authentication)
    }

    /*
    Return a list of all event handler types which are available to be
    created.
    */
    var _getAvailableAuthenticationTypes = function() {
        var types = utils.getFamilyTypes(
            utils.constants.TYPE_NAME.AUTHENTICATION)
        var os_type = utils.constants.TYPE_NAME.OS

        if ($scope.process && $scope.process.runnables.status) {
            var os_name = $scope.process.runnables.status.os_name
            if (os_name == utils.constants.TYPE_NAME.OSX) {
                types.remove(os_type)
            }
        }
        return types
    }

    /*
    Go to configuration page.
    */
    $scope.onGoToConfiguration = function(method) {
        category.goToConfiguration(method)
    }

    /*
    Delete an authentication method.
    */
    $scope.onDeleteAuthentication = function(method) {
        category.delete(method)
        // Update the table after one entry was removed.
        $scope.onFilterAuthentications($scope.filter_input)
    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.authenticationsCtrl = {} : exports))
