(function(exports){

exports.eventHandlerCtrl = function($scope, $routeParams, utils) {

    $scope.handler = null
    // Requried to enable auto-disabling of overridden values.
    $scope.section_uuid = ''

    // Alias so that we can share the same SSL config between multiple
    // components.
    $scope.ssl_configuration = null

    $scope.template_path = null
    $scope.email_body_default_empty =
        utils.constants.CONFIGURATION_DEFAULTS.email_body
    var methods = utils.constants.EMAIL_ASSOCIATED_FILES_NAME
    var associated_files_methods = []
    for (var option in methods) {
        associated_files_methods.push({'id': option, 'value': methods[option]})
    }
    $scope.associated_files_methods = associated_files_methods

    $scope.__init__ = function() {
        var all_handlers = $scope.$parent.getAllEventHandlers()
        var current_handler = $scope.$parent._findEntity(
            all_handlers, $routeParams.uuid)

        if (!current_handler) {
            $scope.$parent.goToPage('/event-handlers')
            return
        }

        $scope.digest_algorithms = utils.keyValueArray(
            utils.constants.DIGEST_NAMES)
        $scope.output_formats = utils.keyValueArray(
            utils.constants.DIGEST_FORMATS)

        $scope.handler = current_handler
        $scope.$parent.component = current_handler

        $scope.ssl_configuration = current_handler
        $scope.template_path = $scope.$parent.getTemplatePath(
            'event-handler', $scope.handler.type)

        $scope.dispatch_rules = utils.multiSequenceToText(
            $scope.handler.dispatch_rules)

        $scope.fallback_rule = ''
        if ($scope.handler.fallback_rule) {
            $scope.fallback_rule = utils.multiSequenceToText(
                [$scope.handler.fallback_rule])
        }
        var uuid = current_handler.uuid
        if (uuid) {
            // Only handlers that are saved on the server have runnables.
            $scope.$parent.runnable = (
                $scope.process.runnables.event_handlers[uuid])
            $scope.section_uuid = 'events-handlers/' + uuid
        }

    }

    /*
    Return `true` if changing event handler name will trigger a restart.
    */
    $scope.doesNameChangeRequiresRestart = function() {
        if (!$scope.handler) {
            return false
        }
        if ($scope.handler['type'] ==
                utils.constants.TYPE_NAME.WINDOWS_EVENTLOG) {
            return true
        }
        return false
    }


    /*
    Called on blur event on dispatch_rules.

    `event` contains the new value.
    */
    $scope.onFileDispatchRulesBlur = function(event) {
        // Model is not yet updated so we get the value from the event.
        var raw_rules = event.target.value
        var new_rules = []
        var error = false
        angular.forEach(
            utils.textToMultiSequence(raw_rules),
            function(rule) {
                if (
                    rule.length == 2 &&
                    rule[0].trim() == 'delete'
                    ) {
                    // All good.
                } else if (
                    rule.length == 2 &&
                    rule[0].trim() == 'ignore'
                    ) {
                    // All good. Ignore operation has no configuration.
                } else if (
                    rule.length > 2 &&
                    rule[0].trim() == 'delete'
                    ) {
                    // Delete only supports 2 parts.
                    error = true
                    return
                } else if (rule.length < 3) {
                    // Skip invalid line
                    error = true
                    return
                }
                new_rules.push(rule.map(function(item) {
                    return item.trim()}))
            })

        $scope.handler.dispatch_rules = new_rules
        $scope.eventHandlerEditForm.dispatch_rules = {'$invalid': error}
        return true
    }

    /*
    Called on blur event on fallback_rule.

    `event` contains the new rule.
    */
    $scope.onFallbackRuleBlur = function(event) {
        // Model is not yet updated so we get the value from the event.
        var raw_rule = event.target.value
        var new_rule = []
        var error = false

        if (raw_rule) {
            var rule = utils.textToMultiSequence(raw_rule)
            // Here we have a list of lines, but we only care for the first.
            if (rule) { rule = rule[0]}
            if (rule.length == 1 && rule[0].trim() == 'delete') {
                // All good as delete don't need paths.
            } else if (rule.length > 1 && rule[0].trim() == 'delete') {
                // Delete method does not need a path.
                error = true
            } else if (rule.length < 2) {
                // Non delete methods needs at least one path.
                error = true
            }
            if (!error) {
                new_rule = rule.map(function(item) {
                    return item.trim()})
            }
        }

        $scope.handler.fallback_rule = new_rule
        $scope.eventHandlerEditForm.fallback_rule = {'$invalid': error}
        return true
    }

    /*
    Delete an event handler.
    */
    $scope.onDeleteEventHandler = function() {
        var category = $scope.$parent.component_categories.event_handlers
        category.delete($scope.handler)
        $scope.goToPage('/event-handlers')
    }

    /*
    Get all suggestions for email recipients.
    */
    $scope.getAllEmailSuggestions = function() {
        var result = []
        angular.forEach(
            $scope.configuration.groups,
            function(value, key) {
                result.push(value)
            }
        )

        angular.forEach(
            $scope.configuration.accounts,
            function(value, key) {
                if (!value.email) {
                    // User has no email.
                    // No need to suggest it.
                    return
                }
                result.push(value)
            }
        )

        return result
    }

    /*
    Return all the groups that this event handler can use.
    */
    $scope.getAllEventGroups = function() {
        var result = []
        angular.forEach(
          utils.constants.EVENTS_DB.groups,
          function(value, key) {
            result.push({
                'uuid': key,
                'name': key,
            })
          })

        return result
    }

    /*
    Return all the accounts that this event handler can use.
    */
    $scope.getUsernames = function() {
        var result = []
        angular.forEach(
            $scope.getAllAccounts(),
          function(value, key) {
            result.push({
                // Event handler filters are configured with the actual
                // username. It is not an account UUID, as it can be
                // configured with external users.
                'uuid': value.name,
                'name': value.name,
            })
          }
        )
        return result
    }

    /*
    Return all the components that this event handler can use.
    */
    $scope.getComponents = function() {
        var result = []

        angular.forEach(
            $scope.getEventComponents(),
            function(value, key) {
                if (!value.uuid) {
                    // This is a newly created component.
                    // We ignore it since it has no UUID yet.
                    return
                }
                result.push({
                    'uuid': value.uuid,
                    'name': value.name + ' (' + value.family + ')',
                })
            }
        )
        return result
    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.eventHandlerCtrl = {} : exports))
