/*
Tests for event handler controller.
*/
describe('EventHandlerCtrl', function() {

    var compiler
    var scope
    var utils
    var ctrl
    var result
    var routeParams
    var viewUpdate
    var $location

    require('../app.js')
    beforeEach(angular.mock.module('app'))
    beforeEach(inject(function(
        $injector, $compile, $rootScope, $controller, $route) {

        // FIXME:3932:
        // See how to test without removing the default route.
        delete $route.routes[null]

        utils = mk.servicesUtils($injector)

        compiler = $compile
        var parent_scope = $rootScope.$new();
        ctrl = $controller(
            'mainCtrl', {
            $scope: parent_scope,
            utils: utils
            })

        routeParams = {}
        scope = parent_scope.$new();
        ctrl = $controller(
            'eventHandlerCtrl', {
            $routeParams: routeParams,
            $scope: scope,
            utils: utils
            })
        viewUpdate = mk.getViewUpdate(scope)
        $location = $injector.get('$location')
        // Run an initial digest to 'settle' all watchers.
        scope.$digest()
    }))

    /*
    Update the config so that it has an event handler.

    It returns the configuration for the new event handler.
    */
    createEventHandler = function(name, type) {
        var handler = {
            'uuid' : 'some-uuid',
            'name': name,
            'type': type,
            'target': ['123', '345']
            }
        scope.$parent.configuration = {'event_handlers': {'some-uuid': handler}}
        scope.$parent.process.runnables = {
            'event_handlers': {'some-uuid': {'uuid': 'some-uuid'}}
        }
        routeParams['uuid'] = 'some-uuid'
        return handler
    }

    describe('__init__', function() {

        test(
        'Go to event handlers page if could not find the requested handler.',
        function() {
            scope.$parent.configuration = {'event_handlers': []}
            routeParams['uuid'] = 'some-name'

            scope.__init__()

            expect($location.path()).toBe('/event-handlers')
        })

        test(
        'Shows event handler details.',
        function() {
            var new_handler = {
                'name': 'some-name',
                'type': 'some-type',
                'target': ['123', '345'],
                'usernames': ['test_user'],
                'components': ['some-uuid']
                }
            scope.$parent.configuration = {'event_handlers': []}
            scope.$parent.new_entities['event_handlers'] = [new_handler]
            routeParams['uuid'] = 'some-name'

            scope.__init__()

            expect(scope.handler).toBe(new_handler)
            expect(scope.template_path).toEqual(
                'partials/event-handler_some-type.html?v=%(cache)s')
        })

    })


    /*
    Return a element which is already set as we use it in the page
    and has the handler attached as its model.
    */
    var getInputElement = function(handler, template, target) {
        // Set the ID so that it will be selected on page load.
        handler['uuid'] = 'uuid-12-23'

        // Create the configuration based on the passed handler.
        scope.$parent.configuration = {
            'event_handlers': {"uuid-12-23": handler}}
        scope.$parent.new_entities['event_handlers'] = []
        scope.$parent.process.runnables = {
            'event_handlers': {"uuid-12-23": {'uuid': "uuid-12-23"}}
        }
        routeParams['uuid'] = 'uuid-12-23'

        scope.__init__()
        var dom = compiler(template)(scope)
        return dom.find(target)[0]
    }

    describe('onFileDispatchRulesBlur', function() {

        // Return the textarea element already set with model to `handler`.
        var getTextArea = function(handler) {

            var template = (
                '<div ng-form="eventHandlerEditForm">' +
                '<textarea ' +
                'type="text" ' +
                'ng-blur="onFileDispatchRulesBlur($event)" ' +
                'ng-model="dispatch_rules" ' +
                '></textarea>' +
                '</div>'
                )

            var element = getInputElement(handler, template, 'textarea')
            return element
        }

        test(
        'Set empty list without errors on the form when no value is provided.',
        function() {
            var new_handler = {
                'dispatch_rules': [['move', '*.pdf', '/some/dest']]
                }
            var input_field = getTextArea(new_handler)

            viewUpdate(input_field, '', 'blur')

            expect(scope.handler['dispatch_rules']).toEqual([])
            expect(
                scope.eventHandlerEditForm.dispatch_rules['$invalid']
                ).toBeFalse()
        })

        test(
        'Set an error and reset the model when input value has fewer ' +
        'values than expected.',
        function() {
            // At the start model has a valid value.
            var new_handler = {
                'dispatch_rules': [['move', '*.pdf', '/some/dest']]
                }
            var input_field = getTextArea(new_handler)

            viewUpdate(input_field, 'move, *.psf', 'blur')

            expect(
                scope.eventHandlerEditForm.dispatch_rules['$invalid']
                ).toBeTrue()
            // After the invalid input it is reset to empty value.
            expect(new_handler['dispatch_rules']).toEqual([])
        })

        test(
        'Will set the model to list of list, when input value has a ' +
        'minimum of 3 elements and is a single line.',
        function() {
            var new_handler = {
                'dispatch_rules': [['other', '*.pdf', '/some/dest']]
                }
            var input_field = getTextArea(new_handler)

            viewUpdate(
                input_field,
                'move-with-timestamp , *.txt , /other/dest  ', 'blur')

            expect(
                scope.eventHandlerEditForm.dispatch_rules['$invalid']
                ).toBeFalse()
            expect(new_handler['dispatch_rules']).toEqual([
                ['move-with-timestamp', '*.txt', '/other/dest']
                ])
        })

        test(
        'Will accept a line with only 2 elements when the action is delete.',
        function() {
            var new_handler = {
                'dispatch_rules': [['other', '*.pdf', '/some/dest']]
                }
            var input_field = getTextArea(new_handler)

            viewUpdate(input_field, 'delete , \t*.txt ', 'blur')

            expect(
                scope.eventHandlerEditForm.dispatch_rules['$invalid']
                ).toBeFalse()
            expect(new_handler['dispatch_rules']).toEqual([
                ['delete', '*.txt']
                ])
        })

        test(
        'Will reject input for delete action with more than 2 elements.',
        function() {
            var new_handler = {
                'dispatch_rules': [['other', '*.pdf', '/some/dest']]
                }
            var input_field = getTextArea(new_handler)

            viewUpdate(
                input_field, 'delete , \t*.txt, /extra/destination', 'blur')

            expect(
                scope.eventHandlerEditForm.dispatch_rules['$invalid']
                ).toBeTrue()
            expect(new_handler['dispatch_rules']).toEqual([])
        })

        test(
        'Accept multi-line input and will create a rule for each line.',
        function() {
            var new_handler = {
                'dispatch_rules': [['other', '*.pdf', '/some/dest']]
                }
            var input_field = getTextArea(new_handler)

            var value = (
                ' delete , \t*.txt \n' +
                '\tmove, m/.*pdf/i , /move-dest  \r\n' +
                'move\t, *.csv, /other-dest'
                )
            viewUpdate(input_field, value, 'blur')

            expect(
                scope.eventHandlerEditForm.dispatch_rules['$invalid']
                ).toBeFalse()
            expect(new_handler['dispatch_rules']).toEqual([
                ['delete', '*.txt'],
                ['move', 'm/.*pdf/i', '/move-dest'],
                ['move', '*.csv', '/other-dest']
                ])
        })

        test(
        'Will accept multi-line input when one rule is invalid, ignoring ' +
        'the rules which are invalid',
        function() {
            var new_handler = {
                'dispatch_rules': [['other', '*.pdf', '/some/dest']]
                }
            var input_field = getTextArea(new_handler)

            var value = (
                ' delete , \t*.txt \n' +
                '\tmove, m/.*pdf/i \r\n' +
                'move\t, *.csv, /other-dest'
                )
            viewUpdate(input_field, value, 'blur')


            expect(
                scope.eventHandlerEditForm.dispatch_rules['$invalid']
                ).toBeTrue()
            expect(new_handler['dispatch_rules']).toEqual([
                ['delete', '*.txt'],
                ['move', '*.csv', '/other-dest']
                ])
        })

    })

    describe('onFallbackRuleBlur', function() {

        // Return the text input element already set with model to `handler`.
        var getTextInput = function(handler) {

            var template = (
                '<div ng-form="eventHandlerEditForm">' +
                '<input ' +
                'type="text" ' +
                'ng-blur="onFallbackRuleBlur($event)" ' +
                'ng-model="fallback_rule" ' +
                '/>' +
                '</div>'
                )

            var element = getInputElement(handler, template, 'input')
            return element
        }

        test(
        'Set empty list without errors on the form when no value is provided.',
        function() {
            var new_handler = {
                'fallback_rule': ['move', '/some/dest']
                }
            var input_field = getTextInput(new_handler)

            viewUpdate(input_field, '', 'blur')

            expect(
                scope.eventHandlerEditForm.fallback_rule['$invalid']
                ).toBeFalse()
            expect(new_handler['fallback_rule']).toEqual([])
        })

        test(
        'Set the value on model, without errors on the form when the value ' +
        'is not delete and has at least one destination.',
        function() {
            var new_handler = {
                'fallback_rule': ['move', '/some/dest']
                }
            var input_field = getTextInput(new_handler)

            viewUpdate(
                input_field, 'move-with-timestamp\t,  /other/path / ', 'blur')

            expect(
                scope.eventHandlerEditForm.fallback_rule['$invalid']
                ).toBeFalse()
            expect(new_handler['fallback_rule']).toEqual([
                'move-with-timestamp', '/other/path /'])
        })

        test(
        'Set the value on model, without errors on the form when the value ' +
        'is delete and has no destination.',
        function() {
            var new_handler = {
                'fallback_rule': ['move', '/some/dest']
                }
            var input_field = getTextInput(new_handler)

            viewUpdate(input_field, '  delete\t ', 'blur')

            expect(
                scope.eventHandlerEditForm.fallback_rule['$invalid']
                ).toBeFalse()
            expect(new_handler['fallback_rule']).toEqual(['delete'])
        })

        test(
        'Set empty list on model, and error on the form when the value ' +
        'is delete and has destination.',
        function() {
            var new_handler = {
                'fallback_rule': ['move', '/some/dest']
                }
            var input_field = getTextInput(new_handler)

            viewUpdate(input_field, 'delete, /a/dest', 'blur')

            expect(
                scope.eventHandlerEditForm.fallback_rule['$invalid']
                ).toBeTrue()
            expect(new_handler['fallback_rule']).toEqual([])
        })


        test(
        'Set empty list on model, and error on the form when the value ' +
        'is not delete and has no destination.',
        function() {
            var new_handler = {
                'fallback_rule': ['move', '/some/dest']
                }
            var input_field = getTextInput(new_handler)

            viewUpdate(input_field, 'move-with-timestamp', 'blur')

            expect(
                scope.eventHandlerEditForm.fallback_rule['$invalid']
                ).toBeTrue()
            expect(new_handler['fallback_rule']).toEqual([])
        })

    })

    /*
    It will remove the current handler from the config and
    move to event handlers index page.
    */
    test('onDeleteEventHandler_existing', function() {
        var handler = createEventHandler('h1', 'local-file')
        var handlers = scope.$parent.configuration.event_handlers
        scope.__init__()
        expect(handlers).toContainKey(handler.uuid)

        scope.onDeleteEventHandler()

        expect(handlers).not.toContainKey(handler.uuid)
        expect($location.path()).toBe('/event-handlers')
    })

    /*
    Returns the objects for all groups and all accounts with emails.
    */
    test('getAllEmailSuggestions', function() {
        var group1 = {
            'uuid': 'group1-uuid',
            'name': 'group 1',
            'key': 'value',
        }
        var group2 = {
            'uuid': 'group2-uuid',
            'name': 'group 2',
        }
        var account1 = {
            'uuid': 'account1-uuid',
            'name': 'account 1',
            'email': 'user1@acme.com',
        }
        var account2 = {
            'uuid': 'account2-uuid',
            'name': 'account 2',
            'email': '',
        }
        scope.$parent.configuration = {
            'groups': {
                'group1-uuid': group1,
                'group2-uuid': group2,
            },
            'accounts': {
                'account1-uuid': account1,
                'account2-uuid': account2,
            },
        }

        result = scope.getAllEmailSuggestions()

        // Account 2 is excluded as it has no email.
        expect(result).toIncludeSameMembers([group1, group2, account1])

    })

    /*
    Returns a list with all the defined groups as a dict to be used for Tagiy.
    */
    test('getAllEventGroups', function() {
        result = scope.getAllEventGroups()

        expect(result).toIncludeAllMembers([
            {'name': 'process', 'uuid': 'process'},
            {'name': 'monitor', 'uuid': 'monitor'},
        ])
    })

    /*
    Returns a list with all the defined accounts as a dict to be used for Tagiy.
    */
    test('getUsernames', function() {
        scope.configuration.accounts = {
            'user1-uuid': {
                'uuid': 'user1-uuid',
                'name': 'User 1',
            },
            'user2-uuid': {
                'uuid': 'user2-uuid',
                'name': 'User 2',
            }
        }

        result = scope.getUsernames()

        expect(result).toIncludeSameMembers([
            {'name': 'User 1', 'uuid': 'User 1'},
            {'name': 'User 2', 'uuid': 'User 2'},
        ])

    })

    /*
    Returns a list with all the components as a dict to be used for Tagiy.

    Only components that can be filtered are listed.
    So accounts or newly created components are ignored.

    The name contains the component type.
    */
    test('getComponents', function() {
        scope.configuration.accounts = {
            'user1-uuid': {
                'uuid': 'user1-uuid',
                'name': 'User 1',
            },
        }
        scope.configuration.transfers = {
            'transfer1-uuid': {
                'uuid': 'transfer1-uuid',
                'name': 'Push ACME',
            },
        }

        scope.configuration.services = {
            'service1-uuid': {
                'uuid': 'sevice1-uuid',
                'name': 'HTTPS Server',
            },
        }
        scope.new_entities.services = {'name': 'new service'}

        result = scope.getComponents()

        expect(result).toIncludeSameMembers([
            {'name': 'Push ACME (transfer)', 'uuid': 'transfer1-uuid'},
            {'name': 'HTTPS Server (service)', 'uuid': 'sevice1-uuid'},
        ])

    })


})
