(function(exports){

exports.eventHandlersCtrl = function(
    $scope, utils, componentTypeFilter, $timeout) {

    var init = require('../../../js_commons/__init__.js', 'init')
    var details_getter = {}
    var defaults_getter = {}
    // The current list of filtered and sorted handlers
    // Will be initialized later.
    $scope.filtered_handlers = []
    // Used by the search input tag.
    $scope.filter_input = ''
    $scope.all_components = []
    // Will be updated with all the types that can be added.
    $scope.event_handler_types = []

    $scope.$parent.component = {
        'name': 'Event handlers',
        'summary': 'Configure actions triggered by application activity.',
    }

    var category = $scope.$parent.component_categories.event_handlers

    /*
    This is called each time the page is accessed,
    including when using the browser back/forward.
    */
    $scope.__init__ = function() {
        // Trigger an initial filter that will show all.
        $scope.event_handler_types = _getCreatableEventHandlersTypes()
        $scope.onFilterHandlers()
        // The configuration might not be loaded so we force a dumb reload
        // We should find a better way to auto-reload the page but for
        // now it should be ok.
        // In an ideal case, we should use a watch on the configuration,
        // but that might have some side-effects and performance issues.
        // Most of the times, users should not land directly on the handlers
        // page, so this hack should be ok.
        if (init.isEmpty($scope.filtered_handlers)) {
            $timeout(function() {$scope.onFilterHandlers()}, 3000)
        }
    }

    /*
    Called the update the list of handlers to be displayed on the page.
    */
    $scope.onFilterHandlers = function(filter_value) {
        var view_model
        $scope.filtered_handlers = []
        $scope.all_components = $scope.getEventComponents()

        if (!$scope.configuration.event_handlers) {
            // Configuration not yet loaded.
            return
        }

        angular.forEach(
            _getAllHandlers(),
            function(handler) {
                if (isFilteredOut(filter_value, handler)) {
                    return
                }
                view_model = _getHandlerView(
                    handler, $scope.process.runnables.event_handlers)
                $scope.filtered_handlers.push(view_model)
            })
    }

    /*
    Return a list with configuration for all the handlers.
    */
    var _getAllHandlers = function() {
        return $scope.getEntities('event_handlers')
    }

    /*
    Update the `handlers` based on the current filter.
    */
    var _getHandlerView = function(configuration, runnables) {

        var view_model = utils.getBaseViewModel(configuration, runnables)
        return {
            'uuid': view_model.uuid,
            'enabled_text': $scope.getEntityStartupText(configuration),
            'filters': getHandlerFilters(configuration),
            'details': getHandlerDetails(configuration),
            'config': configuration,
            'runnable': view_model.runnable,
            'just_created': view_model.just_created
        }
    }

    /*
    Return true if the entry should by filtered out.
    */
    var isFilteredOut = function(filter_value, configuration) {
        if (!filter_value) {
            return false
        }

        if (utils.containsInsensitive(configuration.name, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(
                configuration.description, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(
            componentTypeFilter(
                configuration.type, 'event_handler'), filter_value)) {
            return false
        }

        return true
    }



    /*
    Return the filters to be visible in the UI for each event handler.

    Return empty list if no extra details are required for the UI.
    */
    var getHandlerFilters = function(configuration) {

        var raw_render = function(list, separator) {
            var separator = separator
            if (!list) {
                return ''
            }
            if (!separator) {
                separator = ', '
            }
            return list.join(separator)
        }

        var name_render = function(list) {
            var result = []
            if (!list) {
                return ''
            }

            angular.forEach(list, function(value) {
                angular.forEach($scope.all_components, function(component) {
                    if (value != component.uuid) {
                        return
                    }
                    result.push(component.name)
                })
            })

            return result.join(', ')
        }

        var data_filter = ''
        if (configuration.data_filter) {
            data_filter = raw_render(
                configuration.data_filter.map(
                    function(rule) {return raw_render(rule, ' = ')}),
                ' AND ')
        }

        return [
            {
                'key': 'Events',
                'value': raw_render(configuration.target),
            },
            {
                'key': 'Event groups',
                'value': raw_render(configuration.groups),
            },
            {
                'key': 'Accounts',
                'value': raw_render(configuration.usernames),
            },
            {
                'key': 'Components',
                'value': name_render(configuration.components),
            },
            {
                'key': 'Source IPs',
                'value': raw_render(configuration.source_addresses),
            },
            {
                'key': 'Data',
                'value': data_filter,
            },
            {
                'key': 'Stop after errors',
                'value': configuration.fail_after_errors,
            }
        ]
    }

    /*
    Return the options to be visible in the UI for each event handler type.

    Return empty list no extra options are required for the UI.
    */
    var getHandlerDetails = function(configuration) {

        var result = [{
            'key': 'Type',
            'value': componentTypeFilter(configuration.type, 'event_handler'),
        }]
        var get_details = details_getter[configuration.type]
        if (get_details) {
            Array.prototype.push.apply(
                result, get_details(configuration))
        }
        return result
    }

    details_getter['account-interaction'] = function(configuration) {
        return [
            {
                'key': 'Operations',
                'value':  configuration.monitored_operations.join(', ')
            }
        ]
    }


    details_getter['external-executable'] = function(configuration) {
        return [
            {
                'key': 'Path',
                'value':  configuration.executable_path
            },
            {
                'key': 'Arguments',
                'value':  configuration.executable_arguments
            }
        ]
    }

    details_getter.extension = function(configuration) {
        return [
            {
                'key': 'Entry point',
                'value':  configuration.entry_point
            }
        ]
    }

    details_getter['file-dispatcher'] = function(configuration) {
        return [
            {
                'key': 'Overwrite rule',
                'value':  configuration.overwrite_rule
            },
            {
                'key': 'Retry count',
                'value':  configuration.retry_count
            },
            {
                'key': 'Retry wait',
                'value':  configuration.retry_wait
            },
            {
                'key': 'Dispatch delay',
                'value':  configuration.dispatch_delay
            },
            {
                'key': 'Temporary suffix',
                'value':  configuration.destination_temporary_suffix
            }
        ]
    }

    details_getter['file-digest'] = function(configuration) {
        return [
            {
                'key': 'Algorithm',
                'value':  configuration.digest_algorithm
            },
            {
                'key': 'Source attribute',
                'value':  configuration.source_attribute
            }
        ]
    }

    details_getter.http = function(configuration) {
        return [
            {
                'key': 'URL',
                'value':  configuration.url.join(' ')
            },
            {
                'key': 'Content',
                'value':  configuration.http_content_type
            }
        ]
    }

    details_getter.rabbitmq = function(configuration) {
        return [
            {
                'key': 'URL',
                'value':  configuration.url
            },
            {
                'key': 'Exchange',
                'value':  configuration.exchange
            },
            {
                'key': 'Routing',
                'value':  configuration.routing_key
            }
        ]
    }

    details_getter.database = function(configuration) {
        return [
            {
                'key': 'Path',
                'value':  configuration.path
            },
            {
                'key': 'Auto-delete',
                'value':  '' + configuration.auto_delete + ' days'
            }
        ]
    }

    details_getter.syslog = function(configuration) {
        return [
            {
                'key': 'URL',
                'value':  configuration.url
            }
        ]
    }

    details_getter['create-archive'] = function(configuration) {
        return [
            {
                'key': 'Dest',
                'value':  configuration.destination_path
            },
            {
                'key': 'Format',
                'value':  configuration.archive_format
            },
            {
                'key': 'Delete source',
                'value':  configuration.delete_source_on_success ? 'Yes' : 'No'
            },
            {
                'key': 'Overwrite',
                'value':  configuration.overwrite_rule
            }
        ]
    }

    details_getter['extract-archive'] = function(configuration) {
        return [
            {
                'key': 'Dest',
                'value':  configuration.destination_path
            },
            {
                'key': 'Delete source',
                'value':  configuration.delete_source_on_success ? 'Yes' : 'No'
            },
            {
                'key': 'Overwrite',
                'value':  configuration.overwrite_rule
            }
        ]
    }

    details_getter['local-file'] = function(configuration) {
        return [
            {
                'key': 'Path',
                'value':  configuration.path
            },
            {
                'key': 'Rotate on',
                'value':  configuration.rotate_on
            },
            {
                'key': 'Rotate at size',
                'value':  '' + configuration.rotate_at_size + ' bytes'
            },
            {
                'key': 'Rotate count',
                'value':  configuration.rotate_count
            }
        ]
    }

    details_getter['email-sender'] = function(configuration) {
        return [
            {
                'key': 'Subject',
                'value':  configuration.email_subject
            },
            {
                'key': 'Recipients',
                'value':  configuration.email_to_recipients.join(', ')
            }
        ]
    }

    /*
    Return default configuration for a handler.
    */
    $scope._getDefault = function(handler_type) {
        var result = {
            'enabled': true,
            'type': handler_type,
            'target': ['123456'],
            'usernames': [],
            'components': [],
            'fail_after_errors': 10
            }

        var get_defaults = defaults_getter[handler_type]
        if (get_defaults) {
            angular.extend(result, get_defaults())
        }

        return result
    }

    defaults_getter['account-interaction'] = function() {
        var result = {
            'monitored_operations': [utils.constants.EVENT_TYPE.CREATED],
            }
        return result
    }

    defaults_getter['create-archive'] = function() {
        var defaults = utils.constants.CONFIGURATION_DEFAULTS
        var result = {
            'archive_format': utils.constants.TYPE_NAME.ZIP,
            'archive_name': ['{fullname}{archive_extension}'],
            'source_attribute': defaults.source_attribute,
            'overwrite_rule': defaults.overwrite_rule,
            'delete_source_on_success': defaults.delete_source_on_success
            }
        return result
    }

    defaults_getter.database = function() {
        var result = {
            'auto_delete': utils.constants.CONFIGURATION_DEFAULTS.auto_delete
            }
        return result
    }

    defaults_getter['email-sender'] = function() {
        var defaults = utils.constants.CONFIGURATION_DEFAULTS
        var result = {
            'email_subject': defaults.email_subject,
            'email_body': defaults.email_body,
            'email_associated_files': defaults.email_associated_files,
            'email_cc': [],
            'email_bcc': [],
            'email_to_recipients': []
            }
        return result
    }

    defaults_getter['external-executable'] = function() {
        var result = {
            'executable_path': 'no/path/defined',
            'executable_arguments': '"{data.real_path}"',
            'executable_output_format': utils.constants.TYPE_NAME.TEXT,
            'executable_timeout': 30,
            'concurrent_processes': 2
            }
        return result
    }

    defaults_getter['extract-archive'] = function() {
        var defaults = utils.constants.CONFIGURATION_DEFAULTS
        var result = {
            'source_attribute': defaults.source_attribute,
            'overwrite_rule': defaults.overwrite_rule,
            'delete_source_on_success': defaults.delete_source_on_success
            }
        return result
    }

    defaults_getter['file-dispatcher'] = function() {
        var result = {
            'dispatch_attribute': 'real_path',
            'dispatch_rules': [[
                'move', '*.txt', 'c:\\some-path', '/other/path']],
            'fallback_rule': '',
            'retry_count': 2,
            'dispatch_delay': 5,
            'executable_timeout': 60,
            'create_destination_folder': ''
            }
        return result
    }

    defaults_getter['file-digest'] = function() {
        var result = {
            'source_attribute': 'real_path',
            'digest_algorithm': 'sha512',
            'output_format': 'gnu'
            }
        return result
    }

    defaults_getter.http = function() {
        var result = {
            'retry_count': 2,
            'retry_increase': 10,
            'url': ['http://ws.example.org/handler'],
            'extra_data': '{"text": "{message}"}',
            'timeout': 120,
            'http_content_type': utils.constants.TYPE_NAME.JSON
            }
        return result
    }

    defaults_getter.openpgp = function() {
        var defaults = utils.constants['CONFIGURATION_DEFAULTS']

        var result = {
            'encryption_extension': defaults.encryption_extension,
            'encryption_cipher': defaults.encryption_cipher,
            'source_attribute': defaults.source_attribute,
            'overwrite_rule': defaults.overwrite_rule,
            'delete_source_on_success': defaults.delete_source_on_success
            }
        return result
    }

    defaults_getter.rabbitmq = function() {
        var result = {
            'retry_count': 2,
            'retry_increase': 10,
            'url': 'amqps://YOUR.RABBITMQ.HOST/VIRTUAL_HOST_NAME',
            'routing_key': 'test-queue-name',
            'payload_template': 'New message {id} {data_json}'
            }
        return result
    }

    defaults_getter['standard-stream'] = function() {
        var result = {
            'entry_content':
                utils.constants['CONFIGURATION_DEFAULTS']['entry_content']
            }
        return result
    }

    defaults_getter['local-file'] = function() {
        var defaults = utils.constants['CONFIGURATION_DEFAULTS']
        var fields = defaults['structured_fields']

        var structured_fields = utils.textToMultiSequence(fields)
        if (structured_fields) { structured_fields = structured_fields[0]}
        var result = {
            'entry_content': defaults.entry_content,
            'structured_fields': structured_fields,
            'rotate_at_size': defaults.rotate_at_size,
            'path': 'log/server.log',
            'rotate_count': defaults.rotate_count
            }
        return result
    }

    /*
    Return default configuration for Syslog.
    */
    defaults_getter.syslog = function() {
        var result = {
            'url': 'udp://127.0.0.1:514'
            }
        return result
    }

    /*
    Called when add event button is triggered.
    */
    $scope.onAddEventHandler = function(handler_type) {
        var handler = {}

        handler['name'] = 'New %(type)s handler %(random)s'.supplant({
            'type': handler_type,
            'random': utils.getRandomMark()
            })

        angular.extend(handler, $scope._getDefault(handler_type));

        category.add(handler)
        category.goToConfiguration(handler)
    }

    /*
    Return a list of all event handler types which are available to be
    created.
    */
    var _getCreatableEventHandlersTypes = function() {
        var types = utils.getFamilyTypes(
            utils.constants.TYPE_NAME.EVENT_HANDLER)
        var eventlog_type = utils.constants.TYPE_NAME.WINDOWS_EVENTLOG
        var os_family

        if ($scope.process && $scope.process.runnables.status) {
            os_family = $scope.process.runnables.status.os_family
            if (os_family == 'Unix') {
                types.remove(eventlog_type)
            }
        }
        return types
    }

    /*
    Go to configuration page.
    */
    $scope.onGoToConfiguration = function(handler) {
        category.goToConfiguration(handler)
    }

    /*
    Delete an event handler.
    */
    $scope.onDeleteEventHandler = function(handler) {
        category.delete(handler)
        // Update the table after one entry was removed.
        $scope.onFilterHandlers($scope.filter_input)
    }
}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.eventHandlersCtrl = {} : exports))
