(function(exports){

    exports.groupsCtrl = function($scope, utils, $timeout) {

    var init = require('../../../js_commons/__init__.js', 'init')
    // The current list of filtered and sorted groups
    // Will be initialized later.
    $scope.filtered_groups = []
    // Used by the group input tag.
    $scope.filter_input = ''
    $scope.tabs = []
    $scope.$parent.component = {
        'name': 'Groups',
        'summary': (
            'Groups are used for applying similar configuration to multiple ' +
            'accounts.'
        ),
    }

    /*
    This is similar to accounts code.
    jscpd:ignore-start
    */
    var update_tabs = function(current, old) {
        $scope.tabs = $scope.createIdentityTabs('groups')
    }
    $scope.$watch('process.authentications', update_tabs, true)

    // Handle tab switching.
    $scope.active_tab = 0
    $scope.onTabSelect = function(tab_index) {
        $scope.active_tab = tab_index
        $scope.filter_input = ''
        $scope.__init__()
    }
    // jscpd:ignore-end

    /*
    This is called each time the page is accessed,
    including when using the browser back/forward.
    */
    $scope.__init__ = function() {
        if ($scope.redirectWithoutConfiguration()) {
            // FIXME:5878:
            // The page can only be loaded when the config is already loaded,
            // due to external authentication tabs.
            return
        }
        // Trigger an initial filter that will show all.
        $scope.onFilterGroups($scope.filter_input)
        // The configuration might not be loaded so we force a dumb reload
        // We should find a better way to auto-reload the page but for
        // now it should be ok.
        // In an ideal case, we should use a watch on the configuration,
        // but that might have some side-effects and performance issues.
        // Most of the times, users should not land directly on the groups
        // page, so this hack should be ok.
        if (init.isEmpty($scope.filtered_groups)) {
            $timeout(function() {$scope.onFilterGroups()}, 3000)
        }
    }

    /*
    Called the update the list of groups to be displayed on the page.
    */
    $scope.onFilterGroups = function(filter_value) {
        var view_model
        $scope.filtered_groups = []

        if (!$scope.configuration.groups) {
            // Configuration not yet loaded.
            return
        }

        if (!$scope.tabs) {
            // Configuration not yet loaded.
            return
        }
        var current_tab = $scope.tabs[$scope.active_tab]

        if (!current_tab) {
            // Tabs not yet loaded.
            return
        }
        var current_groups = $scope.getEntities(current_tab.groups_path)
        var all_accounts = $scope.getEntities(current_tab.accounts_path)

        angular.forEach(
            current_groups,
            function(group) {
                if (utils.isComponentExcluded(filter_value, group)) {
                    return
                }
                view_model = getGroupView(group, all_accounts)
                $scope.filtered_groups.push(view_model)
            })
    }

    // Delete the group from the `container` indentity holder.
    $scope.onDeleteGroup = function(container, config) {
        container.delete(config)
        $scope.onFilterGroups($scope.filter_input)
    }

    /*
    Update the `groups` based on the current filter.
    */
    var getGroupView = function(configuration, all_accounts) {
        var uuid = configuration.uuid

        return {
            'uuid': uuid,  // To help with testing to identify the row.
            'user_count': getAccounsCount(uuid, all_accounts),
            'is_delete_blocked': utils.isDeleteBlocked(configuration, 'group'),
            'config': configuration,
        }
    }

    /*
    Return the number of accounts associated with group_uuid.
    */
    var getAccounsCount = function(group_uuid, all_accounts) {
        var count = 0
        angular.forEach(
            all_accounts,
            function(account) {
                var group_uuids = account.group
                if (!group_uuids) {
                    return
                }
                angular.forEach(
                    group_uuids,
                    function(uuid) {
                        if (uuid == group_uuid) {
                            count = count + 1
                        }
                    })
        })

        return count
    }

    /*
    Create a new group and go to edit page for it.
    */
    $scope.onAddGroup = function(parent) {
        var group = {}
        var account_defaults = (
            utils.constants['CONFIGURATION_ACCOUNT_DEFAULTS'])
        group['name'] = utils.getRandomMark('New Group ')

        // Set valid default values.
        group['enabled'] = true
        group['ssh_authorized_keys_path'] = ''
        group['home_folder_path'] = ''
        group['lock_in_home_folder'] = true
        group['allow_certificate_authentication'] = false
        group['allow_own_password_change'] = false
        group['create_home_folder'] = false
        group['create_home_folder_owner'] = (
            account_defaults['create_home_folder_owner'])
        group['create_home_folder_group'] = (
            account_defaults['create_home_folder_group'])
        group['source_ip_filter'] = []
        group['permissions'] = [['allow-full-control']]
        group['virtual_folders'] = []
        group['required_credentials'] = ['any']
        group['password_lifetime'] = 0
        group['disable_on_inactivity'] = 0
        group['file_write_size'] = 0
        group['ssh_delete_delay'] = 0

        parent.add(group)
        parent.goToConfiguration(group)
    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.groupsCtrl = {} : exports))
