/*
Tests for groups controller.
*/


describe('GroupsCtrl', function() {

    /*
    Will redirect to the start page when there is no configuration.
    The start page can wait longer for the configuration to be available.
    */
    test('__init__no_config', function() {
        var sut = mk.getController('groupsCtrl')
        sut.$location.path('/groups')
        expect(sut.$location.path()).toEqual('/groups')

        sut.scope.__init__()

        expect(sut.$location.path()).toEqual('/')
    })

    /*
    Will remain on the same page if there is a configuration.
    */
    test('__init__with_config', function() {
        var sut = mk.getController('groupsCtrl')
        sut.$location.path('/groups')
        sut.scope.configuration.server = {'uuid': 'some-id'}
        expect(sut.$location.path()).toEqual('/groups')

        sut.scope.__init__()

        expect(sut.$location.path()).toEqual('/groups')
    })


    /*
    Will add a new group in new group cache and then ' +
    'go to the page for the new group.',
    */
    test('onAddGroup', function() {
        var sut = mk.getController('groupsCtrl')
        sut.utils.constants['CONFIGURATION_ACCOUNT_DEFAULTS'] = {
            'home_folder_path': 'inherit',
            'create_home_folder': 'No',
            'create_home_folder_owner': 'default-owner',
            'create_home_folder_group': 'default-group',
            'lock_in_home_folder': 'Yes',
            'allow_certificate_authentication': 'Yes'
            }
        sut.scope.$parent.new_entities['groups'] = []
        var new_groups = sut.scope.$parent.new_entities['groups']
        var parent = sut.scope.$parent.component_categories.groups

        sut.scope.onAddGroup(parent)

        expect(new_groups.length).toBe(1)
        expect(new_groups[0]['enabled']).toBeTrue()
        expect(new_groups[0]['uuid']).toBeUndefined()
        expect(new_groups[0]['home_folder_path']).toBe('')
        expect(new_groups[0]['ssh_authorized_keys_path']).toBe('')
        expect(new_groups[0]['create_home_folder']).toBeFalse()
        expect(new_groups[0]['lock_in_home_folder']).toBeTrue()
        expect(new_groups[0]['name']).toStartWith('New Group ')
        expect(sut.$location.path()).toStartWith('/groups/New Group ')
    })

})
