/*
Tests for header controller.
*/

describe('headerCtrl', function() {

    var result
    /*
    Triggers the browers JS history back call.
    */
    test('onGoBack', function() {
        var sut = mk.getController('headerCtrl')
        var old_state = {'key': 'old-state'}
        var new_state = {'key': 'new-state'}
        window.history.pushState(old_state, 'old-title', 'old-url.html')
        window.history.pushState(new_state, 'new-title', 'new-url.html')

        sut.scope.onGoBack()

        // I think there is a bug with JSDOM and this is why the state is not
        // updated.
        // Here we should expect to have the `old_state` after going back.
        expect(window.history.state).toBe(new_state)
    })

    /*
    Return empty string if no roles configuration were loaded yet.
    */
    test('getAdminRoleName_no_config_yet', function() {
        var sut = mk.getController('headerCtrl')

        sut.scope.$parent.configuration = {}
        result = sut.scope.getAdminRoleName()
        expect(result).toBe('')
    })

    /*
    Return empty string if the session avatar is not yet ready.
    */
    test('getAdminRoleName_no_session_yet', function() {
        var sut = mk.getController('headerCtrl')

        sut.scope.$parent.configuration = {'roles': {'uuid1': {}}}
        sut.scope.avatar.roles_uuid = []
        result = sut.scope.getAdminRoleName()
        expect(result).toBe('')
    })

    /*
    Return the name of the admin based on the current avatar.
    */
    test('getAdminRoleName_found', function() {
        var sut = mk.getController('headerCtrl')

        sut.scope.avatar = {
            'uuid': 'some-uuid',
            'roles_uuid': ['primary-uuid', 'role-2'],
        }
        sut.scope.$parent.configuration.roles = {'primary-uuid': {
            'name': 'Role Test primary',
        }}
        result = sut.scope.getAdminRoleName()
        expect(result).toBe('Role Test primary')
    })

    /*
    It navigates to the current admin configuration page and closes
    the dropdown menu.
    */
    test('onGoToCurrentAdmin', function() {
        var sut = mk.getController('headerCtrl')
        sut.scope.avatar = {'uuid': 'some-uuid', 'name': 'Some Name'}
        sut.scope.configuration.administrators = {'some-uuid': {}}
        sut.scope.admin_menu = mk.Dropdown()

        sut.scope.onGoToCurrentAdmin()

        expect(sut.$location.path()).toBe('/administrators/some-uuid')
    })

    /*
    onLogout lets the backend know about the logout and redirects the UI to
    login.
    */
    test('onLogout_ok', function() {
        var sut = mk.getController('headerCtrl')

        sut.changeRoute('/bla')
        expect(sut.$window.location.href).toBe('/bla')

        sut.$httpBackend.expectPOST(
            '/rp/path/json', {
            'jsonrpc': '2.0',
            'method': 'logout',
                'params': {},
            'id': 1
            }
            ).respond({result: true})

        sut.scope.onLogout()
        sut.$httpBackend.flush()

        expect(sut.$window.location.href).toBe('/rp/path/')
    })

    /*
    When onLogout gets a JSON-RPC response, but the response is an error,
    it still redirects to the login page.
    */
    test('onLogout_failure', function() {
        var sut = mk.getController('headerCtrl')

        sut.changeRoute('/other')

        sut.$httpBackend.expectPOST(
            '/rp/path/json', {
            'jsonrpc': '2.0',
            'method': 'logout',
                'params': {},
            'id': 1
            }
        ).respond({error: {'id': 123, 'message': 'anything'} , result: null})

        sut.scope.onLogout()

        sut.$httpBackend.flush()

        expect(sut.$window.location.href).toBe('/rp/path/')
    })


})
