(function(exports){

exports.keyCertCtrl = function($scope, utils, $timeout) {

    $scope.$parent.component = {
        'name': 'Keys and Certificates',
        'summary': (
            'Utilities to manage TLS, SSH or PGP keys.'
        ),
    }

    /*
    Trigger a browser file download for `content`.

    Return an array of 2 elements (SUCCESS_STATUS, ERROR_MESSAGE)
    */
    var saveTextAs = function(content, filename, charset) {
        filename = filename || 'download.txt'
        charset = charset || 'utf-8'

        // Try opening an new frame in as many ways as possible.
        var saveTxtWindow = window.frames.saveTxtWindow
        if (!saveTxtWindow) {
            saveTxtWindow = document.createElement('iframe')
            saveTxtWindow.id = 'saveTxtWindow'
            saveTxtWindow.style.display = 'none'
            document.body.insertBefore(saveTxtWindow, null)
            saveTxtWindow = window.frames.saveTxtWindow
        }
        if (!saveTxtWindow) {
            saveTxtWindow = window.open('', '_temp', 'width=100,height=100')
        }

        if (!saveTxtWindow) {
            return [false, 'Fail to create file download window.']
        }

        if (!saveTxtWindow.document) {
            return [
            false,
            'The current web browser does not support downloading files ' +
            'from JavaScript.']
        }

        var doc = saveTxtWindow.document
        doc.open('text/html', 'replace')
        doc.charset = charset
        if (filename.endsWith('.html')) {
            doc.close()
            doc.body.innerHTML = '\r\n' + content + '\r\n'
        } else {
            // File needs to end with .txt as otherwise download is refused.
            if (!filename.endsWith('.txt')) {
                filename += '.txt'
            }
            doc.write(content)
            doc.close()
        }

        var is_success = doc.execCommand('SaveAs', null, filename)
        saveTxtWindow.close()
        return [is_success, 'Failure in file saving operation.']
    }



    var maximum_size = utils.constants['MAXIMUM_IN_MEMORY_RECEIVED_DATA']
    // Current form to be sent to the server.
    $scope.form = {}
    // Form already sent to the server, used to generated the current key.
    $scope.sent_form = {}
    // SSH Key exported in all formats, as returned by server.
    $scope.key = {}
    // Data from SSL related operations.
    $scope.ssl = {}
    // Data for PKCS 12 conversion.
    $scope.pkcs12 = {}

    $scope.ssh_key_types = [
        {'type': 'rsa', 'name': 'RSA'},
        {'type': 'dsa', 'name': 'DSA'},
        {'type': 'ecdsa', 'name': 'ECDSA'}
        ]
    if (utils.constants.OPENSSL_ED25519_SUPPORTED) {
        $scope.ssh_key_types.push({'type': 'ed25519', 'name': 'Ed25519'})
        }
    $scope.pgp_key_types = [
        {'type': 'rsa', 'name': 'RSA'},
        {'type': 'dsa', 'name': 'DSA / DSS'}
        ]

    var rsa_key_sizes = [
        {'size': 1024, 'name': '1024'},
        {'size': 2048, 'name': '2048'},
        {'size': 3072, 'name': '3072'},
        {'size': 4096, 'name': '4096'},
        {'size': 8192, 'name': '8192'}
        ]
    var dsa_key_sizes = [
        {'size': 1024, 'name': '1024'},
        {'size': 2048, 'name': '2048'},
        {'size': 3072, 'name': '3072'}
    ]
    var ssh_ecdsa_key_sizes = [
        {'size': 256, 'name': '256'},
        {'size': 384, 'name': '384'},
        {'size': 521, 'name': '521'}
    ]
    var ssh_ed25519_key_sizes = [
        {'size': 256, 'name': '256'}
    ]
    $scope.ssh_key_sizes = rsa_key_sizes
    $scope.ssl_key_sizes = rsa_key_sizes
    $scope.pgp_key_sizes = rsa_key_sizes

    $scope.sign_algorithms = [
        {'type': 'sha256', 'name': 'SHA256'},
        {'type': 'sha512', 'name': 'SHA512'},
        {'type': 'sha1', 'name': 'SHA1'}
        ]

    $scope.countries = utils.countries

    $scope.__init__ = function() {

        // Configure the file uploader.
        angular.extend($scope.$flow.opts, {
            'target': utils.constants.BASE_URL_PATH + '/in-memory-upload',
            // By default upload requests are done without cookies.
            'withCredentials': true,
            // Don't check if chunk was already uploaded.
            'testChunks': false,
            'simultaneousUploads': 1,
            'chunkSize': maximum_size,
            'maxChunkRetries': 0,
            'singleFile': true
            })
        $scope._resetForm()
    }

    /*
    Set default values for the form.
    */
    $scope._resetForm = function() {
        $scope.pgp_key_sizes = rsa_key_sizes
        $scope.form = {
            'upload_action': 'ssh_upload',
            'ssl_new_key': 'csr',
            'sign_algorithm': 'sha256',
            'constraints': 'CA:FALSE',
            'key_usage': (
                'critical,digital-signature,' +
                'key-agreement,server-authentication'
                ),
            'generate_type': 'rsa',
            'generate_size': 3072,
            'pgp_generate_type': 'rsa',
            'pgp_generate_size': 3072,
            'pgp_generate_name': 'Autogenerated Key',
            'pgp_generate_email': 'email@example.com',
            'pgp_generate_comment': 'Created by SFTPPlus'
            }
    }

    /*
    Called when a new file was chose, but before being added to the queue.

    Return `false` to reject the file.
    */
    $scope.onFileAdded = function(new_file) {
        // We don't allow empty files and those larger than 100kb (1024 * 100)
        $scope.clearAlerts()

        // For old browsers, accept any file.
        if (!new_file.flowObj.support) {
            return true
        }

        if (new_file.size < 1) {
            $scope.addErrorAlert(
                'File is empty. Please upload a file with some content.')
            return false
        }

        if (new_file.size > maximum_size) {
            $scope.addErrorAlert(
                'File limit is 100KB. ' +
                'If your key is bigger, please contact support.'
                )
            return false
        }

        return true
    }

    /*
    Called when file upload failed.

    This is only called for modern browser, old browser will always succeed.
    */
    $scope.onUploadError = function(file, response_content) {
        var message = 'Server has not accepted the file. Maybe it is too big.'
        if (response_content) {
            message = response_content
        }
        $scope.clearPage()
        $scope.addErrorAlert(message)
    }

    /*
    Called when the file was uploaded with success.
    */
    $scope.onFileSuccess = function(file, response_content) {
        // With old browser, request will always success, but response will
        // not contain the uuid.
        $scope.$flow.cancel()
        var message = (
            'ERROR: Server has not accepted the file. Maybe it is too big.')
        if (response_content) {
            message = response_content
        }

        if (message.startsWith('ERROR: ')) {
            $scope.addErrorAlert(message)
            return
        }

        if ($scope.form.upload_action == 'ssh_upload') {
            // Trigger export of the file that was just uploaded.
            utils.jsonrpc({
                'method': 'ssh_key_export',
                'params': {
                    'uuid': response_content,
                    'import_password': $scope.form['import_password'],
                    'export_password': $scope.form['export_password']
                    },
                'success': $scope._cbSSHKeyExport,
                'error': function(data, status) {
                    $scope.clearPage()
                    $scope.$parent.alertError(data['error'])
                    },
                'alerts': $scope.$parent.alerts
            })
            return
        }

        if ($scope.form['upload_action'] == 'ssl_upload') {
            // Trigger PCKS12 conversion for the file that was just uploaded.
            $scope.pkcs12 = {}
            utils.jsonrpc({
                'method': 'ssl_convert_pkcs12',
                'params': {
                    'uuid': response_content,
                    'import_password': $scope.form['ssl_convert_password']
                    },
                'success': $scope._cbSSLConvert,
                'error': function(data, status) {
                    $scope.clearPage()
                    $scope.$parent.alertError(data['error'])
                    },
                'alerts': $scope.$parent.alerts
            })
            return
        }

    }

    /* Called when a file should be uploaded. */
    $scope.triggerFileUpload = function(action) {
        $scope.form.upload_action = action
        if ($scope.$flow.files.length != 1) {
            $scope.addErrorAlert('No valid file was selected.')
            $scope.key = {}
            return $scope._resetForm()
        }

        // We got files so we can start the upload process.
        $scope.clearAlerts()
        $scope.alerts.push({'type': 'info', 'msg': 'Sending file...'})
        $scope.$flow.upload()
    }

    /*
    Called when we got the exported key.
    */
    $scope._cbSSLConvert = function(data, status) {
        $scope.sent_form = $scope.form
        $scope.clearPage()
        $scope.ssl = data['result']
    }

    /*
    Called when key content download was requested.

    Download a file on the client side without touching the server.
    */
    $scope.onKeyDownload = function(content, name) {
        try {
            // Modern browser.
            var blob = new Blob([content], {type: 'text/plain;charset=utf-8'})
            saveAs(blob, name)
        } catch (e) {
            // Blob not supported, so we are on older browsers.
            var result = saveTextAs(content, name)
            if (!result[0]) {
                $scope.addErrorAlert(result[1])
            }
        }
    }

    /*
    Called when key content copy to clipboard was requested.

    We keep a separate variable for the test of each button.
    */
    var COPY_TO_CLIPBOARD = 'Copy to clipboard'
    var COPIED_TO_CLIPBOARD = 'Copied'
    var COPY_FAILED = 'Copy failed'
    $scope.clipboard_text = {
        'openssh_public': COPY_TO_CLIPBOARD,
        'openssh_v1_private': COPY_TO_CLIPBOARD,
        'openssh_private': COPY_TO_CLIPBOARD,
        'putty_private': COPY_TO_CLIPBOARD,
        'sshcom_public': COPY_TO_CLIPBOARD,
        'sshcom_private': COPY_TO_CLIPBOARD,
        'ssl_cert': COPY_TO_CLIPBOARD,
        'ssl_csr': COPY_TO_CLIPBOARD,
        'ssl_certificate': COPY_TO_CLIPBOARD,
        'ssl_key': COPY_TO_CLIPBOARD,
        'ssl_chain': COPY_TO_CLIPBOARD,
        'pgp_public': COPY_TO_CLIPBOARD,
        'pgp_private': COPY_TO_CLIPBOARD
    }
    $scope.onCopyToClipboard = function(content, name) {
        var copy_success = utils.writeClipboard(content)
        if (!copy_success) {
            // Most probalby not supported by browser.
            // Just show some feedback error and don't reset the text.
            $scope.clipboard_text[name] = COPY_FAILED
            return
        }
        $scope.clipboard_text[name] = COPIED_TO_CLIPBOARD
        $timeout(function() {
            $scope.clipboard_text[name] = COPY_TO_CLIPBOARD}, 5000)
    }

    /*
    Reset the whole page.
    */
    $scope.clearPage = function() {
        $scope.clearAlerts()
        $scope._resetForm()
        $scope.key = {}
        $scope.ssl = {}
        $scope.pkcs12 = {}
        $scope.pgp = {}
        $scope.$flow.cancel()
    }

    /*
    Called when we got the exported key.
    */
    $scope._cbSSHKeyExport = function(data, status) {
        $scope.sent_form = $scope.form
        $scope.clearAlerts()
        $scope.key = data['result']
    }

    /*
    Called when a new SSH key is requested.
    */
    $scope.onSSHKeyGenerate = function() {
        utils.jsonrpc({
            'method': 'ssh_key_generate',
            'params': {
                'type': $scope.form['generate_type'],
                'size': $scope.form['generate_size'],
                'password': $scope.form['generate_password']
                },
            'success': $scope._cbSSHKeyExport,
            'error': function(data, status) {
                $scope.clearPage()
                $scope.$parent.alertError(data['error'])
                },
            'alerts': $scope.$parent.alerts
        })
        $scope.clearAlerts()
        $scope.key = {}
        $scope.$parent.alerts.push({
            'type': 'info',
            'msg': 'Generating key (can take up to 5 minutes)...'
        })
    }

    /*
    Called when a new PGP key is requested.
    */
    $scope.onPGPKeyGenerate = function() {
        utils.jsonrpc({
            'method': 'pgp_key_generate',
            'params': {
                'type': $scope.form['pgp_generate_type'],
                'size': $scope.form['pgp_generate_size'],
                'name': $scope.form['pgp_generate_name'],
                'comment': $scope.form['pgp_generate_comment'],
                'email': $scope.form['pgp_generate_email'],
                'password': $scope.form['pgp_generate_password']
                },
            'success': $scope._cbPGPKeyExport,
            'error': function(data, status) {
                $scope.clearPage()
                $scope.$parent.alertError(data['error'])
                },
            'alerts': $scope.$parent.alerts
        })
        $scope.clearAlerts()
        $scope.pgp = {}
        $scope.$parent.alerts.push({
            'type': 'info',
            'msg': 'Generating key (can take up to 5 minutes)...'
        })
    }

    /*
    Called when type for the new SSH key to be generated is changed.
    */
    $scope.onSSHGenerateTypeChange = function() {
        // RSA key type by default.
        $scope.ssh_key_sizes = rsa_key_sizes
        $scope.form.generate_size = 3072

        if ($scope.form.generate_type == 'dsa') {
            $scope.ssh_key_sizes = dsa_key_sizes
        } else if ($scope.form.generate_type == 'ecdsa') {
            $scope.form.generate_size = 256
            $scope.ssh_key_sizes = ssh_ecdsa_key_sizes
        } else if ($scope.form.generate_type == 'ed25519') {
            $scope.form.generate_size = 256
            $scope.ssh_key_sizes = ssh_ed25519_key_sizes
        }
    }

    /*
    Called when type for the new key to be generated is changed.
    */
    $scope.onPGPGenerateTypeChange = function() {
        $scope.pgp_key_sizes = rsa_key_sizes
        if ($scope.form.pgp_generate_type !== 'rsa') {
            $scope.pgp_key_sizes = dsa_key_sizes
        }
        $scope.form.pgp_generate_size = 3072
    }

    /*
    Called when we got the PGP key.
    */
    $scope._cbPGPKeyExport = function(data, status) {
        $scope.sent_form = $scope.form
        $scope.clearPage()
        $scope.pgp = data['result']
    }

    /*
    Called when a new ssl key + csr is requested.
    */
    $scope.onSSLGenerate = function(form) {
        if (form.$invalid) {
            return
        }

        $scope.ssl = {}

        if ($scope.form['ssl_new_key'] == 'self') {
            return onSelfCertGenerate(form)
        }

        return onCSRGenerate(form)
    }

    /*
    Return the common paramtners for self signed or CSR generations.
    */
    var getCertGenerateParams = function() {
        return {
            'size': $scope.form['generate_size'],
            'password': $scope.form['generate_password'],
            'sign_algorithm': $scope.form['sign_algorithm'],
            'common_name': $scope.form['common_name'],
            'alternative_name': $scope.form['alternative_name'],
            'constraints': $scope.form['constraints'],
            'key_usage': $scope.form['key_usage'],
            'email': $scope.form['email'],
            'organization': $scope.form['organization'],
            'organization_unit': $scope.form['organization_unit'],
            'locality': $scope.form['locality'],
            'state': $scope.form['state'],
            'country': $scope.form['country']
            }

    }

    /*
    Handle success condition for self cert or CSR generations.
    */
    var onCertGenerateSuccess = function(data, status) {
        $scope.sent_form = $scope.form
        $scope.clearPage()
        $scope.ssl = data['result']
        }

    /*
    Handle error condition for self cert or CSR generations.
    */
    var onCertGenerateError = function(data, status) {
        $scope.clearAlerts()
        $scope.$parent.alertError(data['error'])
        }

    /*
    Called when a self signed certificate is requested.
    */
    var onSelfCertGenerate = function(form) {
        utils.jsonrpc({
            'method': 'generate_self_signed_certificate',
            'params': getCertGenerateParams(),
            'success': onCertGenerateSuccess,
            'error': onCertGenerateError,
            'alerts': $scope.$parent.alerts
        })
        $scope.clearAlerts()
        $scope.ssl = {}
        $scope.$parent.alerts.push({
            'type': 'info',
            'msg': (
                'Generating SSL key and certificate ' +
                '(can take up to 5 minutes)...'
                )
        })
    }

    /*
    Called when a new ssl key + csr is requested.
    */
    var onCSRGenerate = function(form) {

        if ($scope.form['ssl_new_key']) {
            $scope.form['ssl_key'] = ''
        }

        var params = getCertGenerateParams()
        params['key'] = $scope.form['ssl_key']

        utils.jsonrpc({
            'method': 'generate_csr',
            'params': params,
            'success': onCertGenerateSuccess,
            'error': onCertGenerateError,
            'alerts': $scope.$parent.alerts
        })
        $scope.clearAlerts()
        $scope.ssl = {}
        $scope.$parent.alerts.push({
            'type': 'info',
            'msg': 'Generating SSL key and CSR (can take up to 5 minutes)...'
        })
    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.keyCertCtrl = {} : exports))
