/*
Controller for login process.
*/
(function(exports){

exports.LoginCtrl = function($scope, $timeout, $location, utils) {
  var MANAGER_PAGE_URL = utils.constants.BASE_URL_PATH + '/manager'

  $scope.show_form = false
  $scope.alerts = []
  $scope.username = ''
  $scope.password = ''
  $scope.mfa_password = ''
  $scope.show_password = false

  $scope._redirect_url = MANAGER_PAGE_URL

  /*
  Close a specific alert message.
  */
  $scope.onAlertClose = function(index) {
    $scope.alerts.splice(index, 1)
  }

  /*
  Remove all outstanding alerts.
  */
  $scope.clearAlerts = function(index) {
    $scope.alerts.length = 0
  }

  $scope._setCriticalError = function(error) {
    $scope.show_form = false
    $scope.alerts.push({'type': 'error', 'msg': error})
  }

  /*
  Get CSS classes for alert type.
  alert-error alert-success alert-info
  */
  $scope.getAlertClasses = function(alert_type) {
    return 'alert alert-' + alert_type
  }

  $scope.__init__ = function() {
    if (!utils.areCookiesEnabled()) {
      $scope._setCriticalError(
        'To use this service, the browser must accept cookies ' +
        'from this site.'
        )
      return
    }

    $scope.show_password = false

    utils.jsonrpc({
      'method': 'is_valid_session',
      'params': {},
      'success': $scope._cbSessionChecked,
      'error': $scope._ebSessionChecked,
      'alerts': $scope.alerts
    })

  }

  /*
  Called to show or hide the password.
  */
  $scope.onTogglePasswordVisibility = function() {
    $scope.show_password = !$scope.show_password;
  }


  /*
  Called when JSON-RPC return a success response.
  */
  $scope._cbSessionChecked = function(data, status) {
    if (data.result && data.result.is_valid) {
        // Already a valid session. Go to manager page.
        utils.goToURL(MANAGER_PAGE_URL)
        return
    }

    // See if we have only errors from external interaction.
    // Like for example from OpenID connect or expired session.
    var search = $location.search()
    var error_code = search.backend_error
    if (error_code) {
        var error_message = utils.constants.LM_MESSAGE[error_code]
        if (!error_message) {
           error_message = 'Unknown error: ' + error_code
        }
        $scope.alerts.push({'type': 'error', 'msg': error_message})
    }
    // The session is now valid.
    $scope.show_form = true
    // We use $timeout to delay the execution only after angular has
    // processed all the changes caused by `show_form` state change.
    $timeout(
        function() {document.getElementById('input-username').focus()},
        0
    )
  }

  /*
  Called when JSON-RPC returns a failure.

  It this is called, it means that the JSON-PRC is broken or the server
  is not reachable.
  */
  $scope._ebSessionChecked = function(data, status) {
      // Show the login form on any errors.
      $scope.show_form = true
  }

  /*
  Called when the normal login is requested.
  */
  $scope.onLogin = function(valid) {

    $scope.clearAlerts()

    if (!valid) {
      return
    }

    utils.jsonrpc({
      'method': 'login',
      'params': {
        'username': $scope.username,
        'password': $scope.password + $scope.mfa_password
      },
      'success': $scope._onLoginResponse,
      'alerts': $scope.alerts
    })
  }

  // Start the initialization process;
  $scope._onLoginResponse = function(data, status) {
    $scope.clearAlerts()
    utils.goToURL($scope._redirect_url)
  }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.login = {} : exports))
