(function(exports){

var init = require('../../../js_commons/__init__.js', 'init')

exports.mainCtrl = function(
  $scope,
  $window,
  $route,
  $cookies,
  $location,
  $timeout,
  $rootScope,
  $anchorScroll,
  dayjs,
  utils
) {


  var LOGIN_PAGE_URL = utils.constants.BASE_URL_PATH + '/'
  // This is the login page with a preloaded error message.
  var SESSION_NOT_VALID_URL = (
    utils.constants.BASE_URL_PATH + '/#!?backend_error=session-not-valid')

  /*
  This is called each time we navigate the app.
  For the browser perspective, there is a single page.
  But the Angular router implements a custom navigation that simulates
  normal page navigation, with back and forward history.
  */
  $rootScope.$on('$routeChangeStart', function () {
      // Keep the session alive on each page navigation.
      $scope.onStartAutoUpdate()
      $window.scrollTo(0, 0)

      // Start by erasing the component from the current page,
      // since we are going to go out of the page.
      // The new page will set up its component.
      $scope.component = null
      $scope.runnable = false
  })

  /*
   Component categories.

   Examples of a component: service, location, group. Each component
   belongs to some category. When response from the server is
   received, it could be seen that a runnable and a configuration of a
   component belongs to some particular parent node. For example, a
   runnable and a configuration of a event handler belongs to
   `event_handler` node. The category actually represents this node.
   */
  $scope.component_categories = {
    'event_handlers': 'event_handlers',
    'resources': 'resources',
    'transfers': 'transfers',
    'services': 'services',
    'locations': 'locations',
    'authentications': 'authentications',
    'node_variables': 'node_variables',
    'accounts': 'accounts',
    'administrators': 'administrators',
    'groups': 'groups',
    'roles': 'roles'
  }

  // Here to make it easier to access the constants from the UI.
  $scope.utils = utils
  $scope.constant = utils.constants
  $scope.isAuthenticated = false
  $scope.documentation_target = '_blank'
  /*
   Extend `$scope.component_categories` so that it will contain not
   only data but functions by means of which it is easy to access to a
   component in context of a category.
   */
  for (var category_id in $scope.component_categories) {
    var category = $scope.component_categories[category_id]
    $scope.component_categories[category_id] = {
      name: category,
      getGroups: function() {
        if (!$scope.configuration) {
          return {}
        }
        return $scope.configuration.groups
      },
      add: function(component) {
        $scope.new_entities[this.name].push(component)
      },
      delete: function(component) {
        if (component.uuid) {
          var parts = this.name.split('/')
          if (parts.length > 1) {
            delete $scope.configuration[parts[0]][parts[1]][component.uuid]
          } else {
            delete $scope.configuration[this.name][component.uuid]
          }
        } else {
          var source = $scope.new_entities[this.name]
          angular.forEach(source, function(value, index) {
            if (component['name'] == value['name']) {
              source.splice(index, 1)
            }
          })
        }
      },
      goToConfiguration: function(component) {
        var category = this.name
        if (category.indexOf('/') !== -1) {
          var parts = category.split('/')
          category = parts[1]
        }
        var uri = '/' + category.replace('_', '-') + '/'

        if (uri) {
          $scope._goToComponent(component, uri)
        } else {
          $location.path('/')
        }
      }
    }
  }

  /*
   Map from runnable family name to the name of the runnable manager
   attribute on the process (i.e. "component category").
   */
  var family_to_category = {}
  family_to_category[utils.constants.TYPE_NAME.AUTHENTICATION] = (
      $scope.component_categories.authentications.name)
  family_to_category[utils.constants.TYPE_NAME.RESOURCE] = (
      $scope.component_categories.resources.name)
  family_to_category[utils.constants.TYPE_NAME.EVENT_HANDLER] = (
      $scope.component_categories.event_handlers.name)
  family_to_category[utils.constants.TYPE_NAME.LOCATION] = (
      $scope.component_categories.locations.name)
  family_to_category[utils.constants.TYPE_NAME.SERVICE] = (
      $scope.component_categories.services.name)
  family_to_category[utils.constants.TYPE_NAME.TRANSFER] = (
      $scope.component_categories.transfers.name)
  // This is here to allow using the length method in view expressions, where
  // filters are not supported.
  $scope.length = init.objectLength

  // ui alerts.
  // List of { type: 'error', msg: 'Message.' }
  // See list of supported types for bootstrap.
  $scope.alerts = []

  $scope.avatar = {
    'name': 'NOT-YET-VALIDATED',
    'uuid': '',
    'roles_uuid': [],
  }


  // The configuration object associated with the current page.
  // The runnable object associated with the configuration from the current
  // page.
  $scope.component = null
  $scope.runnable = false

  $scope.process = {'runnables': {}}

  // Configuration object.
  // This is used to collect changes and updated from the GUI.
  $scope.configuration = {}

  $scope.lets_encrypt = {
      'state': 'Unknown',
      'uuid': '',
      'name': "Let's Encrypt Validator"
  }

  // Configuration as currently defined on the server.
  // This is not updated from the GUI only from backend.
  $scope.upstream_configuration = {}

  // Cache for new entities that are not yet saved.
  $scope.new_entities = {}

  // Create start values for all new entities for the main configuration.
  $scope._initializeStandardNewEntities = function() {
    for (var category_id in $scope.component_categories) {
      var category = $scope.component_categories[category_id]
      $scope.new_entities[category.name] = []
    }
  }

  // Do a first initialization.
  $scope._initializeStandardNewEntities()

  // List of changes object pending review.
  $scope.changes = []

  $scope.product = {}
  $scope.product.name = utils.constants.PRODUCT_NAME
  $scope.product.version = utils.constants.PRODUCT_VERSION
  $scope.product.instance = $scope.product.name
  // Uptime as fuzzy human readable text: 3 days ago.
  $scope.product.uptime = 'no uptime yet'
  // ISO date time.
  $scope.product.start_date = 'no start date yet'

  // The CSS class of the sidebar.
  // Mainly used to show/hide it in full or condensed form.
  $scope.sidebar_css = ''

  /*
  Called each time the dashboard page is accessed.
  */
  $scope.__init__ = function() {
    utils.jsonrpc({
      'method': 'is_valid_session',
      'params': {},
      'success': $scope._cbSessionInitialize,
      'alerts': $scope.alerts
    })

    // Collapse sidebar panel if sidebar was condensed previously.
    if ($window.localStorage.getItem('sidebar_condensed') == 'yes') {
      $scope.sidebar_css = 'sidebar_condensed'
    }

  }

  /*
  Go to `path` or reload page is we are already on `path`
  */
  $scope.goToPage = function(path) {
    if ($location.path() == path) {
      $route.reload()
      return
    }
    $location.path(path)
  }

  /*
  Redirect to main page if configuration is not yet loaded.
  Return true if it was redirected.
  */
  $scope.redirectWithoutConfiguration = function() {
      if ($scope.configuration.server) {
          return false
      }
      $scope.goToPage('/')
      return true
  }

  /*
  Toggle condensed/full Sidebar menu.
  */
  $scope.toggleCondensedSidebar = function() {
      if (!$scope.sidebar_css) {
          $scope.sidebar_css = 'sidebar_condensed'
          $window.localStorage.setItem('sidebar_condensed', 'yes');
      } else {
          $scope.sidebar_css = ''
          $window.localStorage.setItem('sidebar_condensed', 'no');
      }
  }

  $scope._cbSessionInitialize = function(data, status) {
    if (!data.result || !data.result.is_valid) {
      // Session is not valid or we don't have a session.
      $scope.avatar.name = 'NOT-AUTHENTICATED'
      utils.goToURL(LOGIN_PAGE_URL)
      return
    }

    if ($scope.avatar.uuid) {
      // The session is valid and we have already initialized the process.
      return
    }

    $scope.avatar.name = data.result.username
    $scope.avatar.uuid = data.result.uuid
    $scope.avatar.roles_uuid = data.result.roles_uuid

    $scope._initializeStandardNewEntities()

    $scope.$watch(
      'configuration', $scope._onConfigurationChange, true)
    $scope.$watch(
      'new_entities', $scope._onNewEntitiesChange, true)
    $scope.$watch(
      'process.status.start_time', $scope._onStartTimeChange)

    utils.jsonrpc({
      'method': 'get_index',
      'params': {},
      'success': $scope._cbIndexReceived,
      'alerts': $scope.alerts
    })
    $scope.isAuthenticated = true
  }

  /*
  Close a specific alert message.
  */
  $scope.onAlertClose = function(index) {
    $scope.alerts.splice(index, 1)
  }

  /*
  Remove all outstanding alerts.
  */
  $scope.clearAlerts = function(index) {
    $scope.alerts.length = 0
  }

  /*
  Get CSS classes for alert type.
  alert-error alert-success alert-info
  */
  $scope.getAlertClasses = function(alert_type) {
    return 'alert alert-' + alert_type
  }

  /*
  Add an alert or type error.
  */
  $scope.addErrorAlert = function(message) {
    $scope.alerts.push({'type': 'error', 'msg': message})
  }

  /*
  Push an error alert, expanding 'error' data.
  */
  $scope.alertError = function(error) {
    $scope.alerts.push({
      'type': 'error',
      'msg': $scope.resolveError(error)
    })
  }

  /*
  JSON-RCP errors have a `code` and direct message
  Server JSON-RPC errors have an `id` and message is resolved from events db.
  */
  $scope.resolveError = function(error) {
    var event_id
    var event_content
    var expanded_message
    if (error['id']) {
      event_id = error['id']
      event_content = utils.constants.EVENTS_DB.events[event_id]
      expanded_message = event_content['message'].supplant(error['data'])

    } else {
      event_id = error['code']
      expanded_message = error['message']
    }

    return event_id + ': ' + expanded_message
  }

  /*
  Return the class for menu item.
  */
  $scope.getMenuItemClass = function(name) {
    var hash_path = $location.path()
    if (hash_path.startsWith('/' + name)) {
      return 'active'
    } else {
      return ''
    }
  }

  /*
  Return the text displayed for the startup if the entity.
  */
  $scope.getEntityStartupText = function(entity) {
    var prefix = ''
    if (!entity) {
      return 'UNKNOWN'
    }

    if (!entity['uuid']) {
      prefix = 'New - '
    }

    if (entity['enabled']) {
      return prefix + 'Start at startup'
    }
    return prefix + 'Leave stopped'
  }


  $scope.getEntityStateText = function(entity) {
    var prefix = ''
    if (!entity['uuid']) {
      prefix = 'New - '
    }

    if (entity['enabled']) {
      return prefix + 'Enabled'
    }
    return prefix + 'Disabled'
  }

  /*
  Return the css class for the badge applied to entity state.
  */
  $scope.getEntityStateBadgeClass = function(entity) {
    if (!entity) {
      return ''
    }
    if (entity['enabled']) {
      return 'label-success'
    } else {
      return ''
    }
  }

  $scope.getTemplatePath = function(base, type) {
      return 'partials/%(base)s_%(type)s.html?v=%(cache)s'.supplant({
          'type': type,
          'base': base,
          'cache': utils.cache_buster
          })
  }


  // How long to wait between auto-updates.
  $scope._AUTO_UPDATE_INTERVAL = 2000
  // How many times to auto-update, before stopping the auto-update.
  var _AUTO_UPDATE_COUNT = 30
  // Number of auto-retries left.
  $scope.auto_updates_left = _AUTO_UPDATE_COUNT
  // The current scheduled auto update call.
  $scope._pending_auto_update = null

  /*
  Called when a page is loaded or when an auto-update is manually requested.
  */
  $scope.onStartAutoUpdate = function() {
    $scope.auto_updates_left = _AUTO_UPDATE_COUNT
    if ($scope._pending_auto_update) {
      // An auto-update is already scheduled.
      return
    }
    _updateRunnables()
  }

  var _updateRunnables = function() {
    $scope._pending_auto_update = 'pending'
    utils.jsonrpc({
      'method': 'get_property',
      'params': {'path': '/runnables/'},
      // Update the runnables state on success.
      'success': function(data, status) {

        if ($scope.auto_updates_left < 1) {
          // Stop auto-updating.
          $scope._pending_auto_update = null
          return
        }

        // Schedule a new auto-update.
        $scope._pending_auto_update = $timeout(
          _updateRunnables, $scope._AUTO_UPDATE_INTERVAL)

        if (!$scope.process.runnables) {
          // This might be called during a refresh,
          // and we don't have the process setup yet.
          // We still want to have the auto-update active.
          // This is why this is called after scheduled the new
          // `_updateRunnables`.
          return
        }
        angular.merge($scope.process.runnables, data.result)
        $scope.auto_updates_left -= 1

      },
      // Logout with an error message.
      'error': function(data, status) {
        utils.goToURL(SESSION_NOT_VALID_URL)
      },
      'failure': function(response) {
        utils.goToURL(SESSION_NOT_VALID_URL)
      }
    })
  }

  $scope._cbIndexReceived = function(data, status) {
    if (!data.result) {
      $scope.addErrorAlert(
        'Critical error. Did not receive configuration from server.'
      )
      return
    }
    angular.copy(data.result, $scope.process)

    // Maybe this is already to late.
    // The name 'configuration' can be confusing as we also have
    // 'process.configuration'  but which is a different thing.
    // Maybe update this in the same PR as upstream_configuration.
    // Rename $scope.configuration to $scope.pending_configuration
    // or something like this, to make it clear that this is the configuration
    // not yet applied on the server.
    angular.copy(data.result['configuration'], $scope.configuration)
    // This is duplication and can be confusing.
    // The same info as $scope.upstream_configuration can be found in
    // $scope.process.configuration.
    angular.copy(data.result['configuration'], $scope.upstream_configuration)
  }

  /*
  Called when start time from the configuration object is changed.
  */
  $scope._onStartTimeChange = function(new_value, old_value) {
    if (!new_value) {
      return
    }
    // We add an extra 1 second to reduce automamted test in which
    // the start time is not in the past, but is the exact time now.
    var start_time = dayjs.unix(new_value - 1)
    $scope.product.uptime = start_time.fromNow()
    // ISO8601 format.
    $scope.product.start_date = start_time.format()
  }

  /*
  Called when configuration object change.
  */
  $scope._onConfigurationChange = function(new_value, old_value) {
    if (init.isEmpty(new_value)) {
      // Not ready.
      return
    }

    if (new_value.server) {
      if (new_value.server.name === 'chevah-selenium-testing') {
        // For selenium testing we open the documentation in the same page
        // to simplify testing.
        $scope.documentation_target = ''
      }
    }
    var instance_name = 'SFTPPlus'
    if (new_value.server) {
        instance_name = new_value.server.name
    }
    $scope.product.instance = instance_name
    $scope._updateChanges()
  }

  $scope._onNewEntitiesChange = function(new_value, old_value) {
    $scope._updateChanges()
  }

  $scope._updateChanges = function() {
    var changes = []
    var result = []
    var original = $scope.upstream_configuration
    var updated = $scope.configuration
    result = $scope._getChangesForUpdateAndDelete(original, updated, '')
    changes.push.apply(changes, result)

    result = $scope._getChangesForCreate()
    changes.push.apply(changes, result)

    // Reset changes and compute all diff from the start
    $scope.changes.length = 0
    $scope.changes.push.apply($scope.changes, changes)
  }


  $scope._getChangesForUpdateAndDelete = function(original, updated, root) {

    var change
    var key
    var result = []
    var original_kind

    for (key in original) {

      original_kind = init.typeOf(original[key])

      if (key.startsWith('$') || key.startsWith('_')) {
        continue
      }

      if (original_kind === 'function') {
        continue
      }

      /*
       * We need to check first for delete operation to avoid recursion in
       * deleted objects.
       */
      change = $scope._getChangeDelete(original, updated, key, root)
      if (change) {
        result.push(change)
        continue
      }

      if (original_kind === 'object') {
        var object_changes = $scope._getChangesForUpdateAndDelete(
          original[key], updated[key], root + key + '/')

        result.push.apply(result, object_changes)
        continue
      } else {
        change = $scope._getChangeUpdate(original, updated, key, root)
        if (change) {
          result.push(change)
          continue
        }
      }
    }

    return result
  }

  /*
  Return the list of all changes for new entities.
  */
  $scope._getChangesForCreate = function() {
    var result = []

    angular.forEach(
      $scope.new_entities,
      function(content, entity_cagetory) {
        angular.forEach(
          content,
          function(value, key) {
            var change = $scope._getChangeCreate(
              value, key, content, entity_cagetory + '/')
            result.push(change)
          })

      })

    return result
  }

  /*
  Return change for delete operation.
  Return null if no delete operation was done.
  */
  $scope._getChangeDelete = function(original, updated, key, root) {
    if (init.typeOf(updated[key]) !== 'undefined') {
      return null
    }
    // Need to make a copy, otherwise we will have the same object
    // and after restore both trees will have the same object and
    // changes function will no longer work.
    var old = angular.copy(original[key])
    return {
      'operation': 'delete',
      'path': root + key,
      'old': old,
      'value': ''
    }
  }

  $scope._getChangeUpdate = function(original, updated, key, root) {
    old_value = original[key]
    new_value = updated[key]

    if (angular.equals(old_value, new_value)) {
      return null
    }

    // Server uses None and empty string as equivalent values while on
    // the LM we only have empty string.
    if (old_value === null && new_value === '') {
      return null
    }

    // This need a copy to avoid putting same object in 2 trees.
    var old = angular.copy(old_value)
    return {
      'operation': 'update',
      'path': root + key,
      'old': old,
      'value': new_value
    }
  }

  $scope._getChangeCreate = function(value, index, parent, root) {

    return {
      'operation': 'create',
      'path': root,
      'old': '',
      'value': value
      }
  }

  /*
   Go to configuration page for the given `component`.
   */
  $scope._goToComponent = function(component, root) {
    var uuid
    if (init.typeOf(component) === 'string') {
      uuid = component
    } else if (component.uuid) {
      uuid = component.uuid
    } else {
      uuid = component.name
    }
    if (uuid) {
      $location.path(root + uuid)
    } else {
      $location.path('/')
    }
    $window.scrollTo(0, 0)
  }

    // Go to a specific section of the page.
    $scope.goToSection = function(hash) {
        // The actual offset might be updated, if we redesing the page.
        // anchorScroll also support calling it with a fuction to get a dynamic
        // yOffset, but for now we keep it simple with a fixed offset.
        $anchorScroll.yOffset = 150
        $anchorScroll(hash)

        // Update the active tab.
        var nav = document.querySelectorAll('[data-role="page-nav"] [data-tab]')
        angular.forEach(nav, function(node) {
          // The node should alwasy have `data-tab` since this is how we
          // use the selector.
          var data_tab = node.getAttribute('data-tab')

          if (data_tab != hash) {
            // Not the targeted tab.
            node.classList.remove("active")
            return
          }
          // We are at the tab associated with the current navigation.
          node.classList.add("active")
        })
    }

  /*
   Go to configuration page for the given `runnable`.
   */
  $scope.onGoToRunnable = function(runnable) {
    var category_name = family_to_category[runnable.family]
    var category = $scope.component_categories[category_name]
    category.goToConfiguration(runnable)
  }

    /*
     Go to account configuration page.
     */
    $scope.onGoToAccount = function(account_uuid) {
        var category = $scope.component_categories.accounts
        category.goToConfiguration({'uuid': account_uuid})
    }

    /*
    Go to administrator configuration page.
    */
    $scope.onGoToAdministrator = function(admin_uuid) {
      $location.path('/administrators/' + admin_uuid)
    }

    /*
    Go to location configuration page.
    */
    $scope.onGoToLocation = function(location_uuid) {
        var category = $scope.component_categories.locations
        category.goToConfiguration({'uuid': location_uuid})
    }

    /*
    Go to service configuration page.
    */
    $scope.onGoToService = function(uuid) {
      var category = $scope.component_categories.services
      category.goToConfiguration({'uuid': uuid})
  }

    /*
    Return the structure to show the tabs and content for identity objects.

    `type` can be 'accounts' | 'groups' | 'roles'...etc
    */
    $scope.createIdentityTabs = function(type) {
        var tab_index = 1
        var result = [
            {
                'index': 0,
                'uuid': 'main-configuration',
                'name': 'Main configuration',
                'accounts_path': 'accounts',
                'groups_path': 'groups',
                'default_group_uuids': ['DEFAULT_GROUP'],
                'container': $scope.component_categories[type]
                }
        ]

        angular.forEach(
            $scope.process.runnables.authentications,
            function(value, key) {
                if (value.type != utils.constants.TYPE_NAME.LOCAL_FILE) {
                    return
                }

                if (!value.started) {
                  // We don't show the identities for stopped auth methods.
                  return
                }

                if (value.startup_configuration.external_management) {
                  // Managed using external process.
                  // Nothing to show in Web Manager.
                  return
                }

                var entities_name = (
                    'authentications/' + value.uuid + '/' + type)
                if (!$scope.new_entities[entities_name]) {
                    $scope.new_entities[entities_name] = []
                }

                var container = $scope._getIdentityContainer(value, type)

                $scope.component_categories[value.uuid] = container

                var default_groups = []
                var groups_path = 'authentications/' + value.uuid + '/groups'
                var groups = $scope.getEntities(groups_path)
                if (groups.length > 0) {
                  default_groups = [groups[0].uuid]
                }
                result.push({
                    'index': tab_index,
                    'uuid': value.uuid,
                    'name': value.name,
                    'default_group_uuids': default_groups,
                    'accounts_path' : (
                      'authentications/' + value.uuid + '/accounts'),
                    'groups_path' : groups_path,
                    'container': container
                })
                tab_index++;
            })

        return result
    }

    /*
    Return the container for identity objects for an authentication method.
    */
    $scope._getIdentityContainer = function(method, type) {

        if (method.type != utils.constants.TYPE_NAME.LOCAL_FILE) {
            return
        }

        if (!method.uuid) {
            // Not yet created
            return
        }
        var entities_name = 'authentications/' + method.uuid + '/' + type

        if (!$scope.new_entities[entities_name]) {
            $scope.new_entities[entities_name] = []
        }

        var container = {
            name: entities_name,
            uuid: method.uuid,
            getGroups: function() {
              return $scope.configuration.authentications[method.uuid].groups
            },
            add: function(component) {
                if (type == 'accounts') {
                    component['type'] = 'application'
                }
                $scope.new_entities[this.name].push(component)
            },
            delete: function(component) {
                var configuration = (
                    $scope.configuration.authentications[this.uuid])
                if (component.uuid) {
                    delete configuration[type][component.uuid]
                    return
                }
                // We don't have an UUID yet, so this is a new entry.
                var source = $scope.new_entities[this.name]
                angular.forEach(
                    source,
                    function(value, index) {
                        if (component['name'] != value['name']) {
                            return
                        }
                        source.splice(index, 1)
                    })
            },
            goToConfiguration: function(component) {
                $scope._goToComponent(component, type + '/')
            }
        }

        return container
    }

  /*
  Return a list with all services.
  */
  $scope.getAllServices = function() {
    var result = []
    var sorted_result = []

    // Put new services at the start of the list.
    angular.forEach(
      $scope.new_entities['services'],
      function(value, index) {
        result.push(value)
      })

    angular.forEach(
      $scope.configuration.services,
      function(value, index) {
        if (!value) {
          // Service is missing... maybe it was deleted.
          return
        }
        result.push(value)
      })

    result.sort(init.sort_by('name', false, init.toUpperCaseFn))

    angular.forEach(
      result,
      function(value, index) {
        // Put administration service first.
        if (value['uuid'] === utils.constants['DEFAULT_LOCAL_MANAGER_UUID']) {
          sorted_result.splice(0, 0, value)
        }else {
          sorted_result.push(value)
        }
      })

    return sorted_result
  }

  /*
  Return a list with all accounts.
  */
  $scope.getAllAccounts = function() {
    var result = $scope.getEntities('accounts')
    angular.forEach(
      $scope.configuration.authentications,
      function(value, key) {
        var auth_accounts = $scope.getEntities(
          'authentications/' + value.uuid + '/accounts')
        result.push.apply(result, auth_accounts)
      })
    return result
  }

    /*
    Return a list with all application accounts defined on the main
    configuration.
    */
    $scope.getAllMainApplicationAccounts = function() {
        var result = []
        var configuration

        if (!$scope.configuration) {
            return result
        }

        configuration = $scope.configuration.accounts

        angular.forEach(
            configuration,
            function(value, key) {
                // FIXME:2585:
                // Should be replaced with `type`.
                // Here while the cleanup is not yet done.
                if (value.kind_name != 'application') {
                    return
                }
                result.push(value)
        })
        result.sort(init.sort_by('name', false, init.toUpperCaseFn))
        return result
    }

  /*
  Return a list with all the groups from the main configuration file.

  In lower-case alphabetical order.
  */
  $scope.getAllMainGroups = function() {
    var result = []

    if (!$scope.configuration) {
        // Configuration not yet loaded.
        return result
    }

    angular.forEach(
        $scope.new_entities['groups'],
        function(value, index) {
          result.push(value)
        }
    )

    angular.forEach(
        $scope.configuration.groups,
        function(value, key) {
            result.push(value)
        }
    )

    result.sort(init.sort_by('name', false, init.toUpperCaseFn))
    return result
  }

  /*
  Return a list with all roles.
  */
  $scope.getAllRoles = function() {
    var result = []
    var default_role
    if (!$scope.configuration) {
        // Configuration not yet loaded.
        return result
    }

    angular.forEach(
      $scope.new_entities['roles'],
      function(value, index) {
        result.push(value)
      })

    angular.forEach(
      $scope.configuration.roles,
      function(value, key) {
        if (value.uuid == utils.constants['CONFIGURATION_DEFAULT_ROLE']) {
          default_role = value
        } else {
          result.push(value)
        }
      })
    result.sort(init.sort_by('name', false, init.toUpperCaseFn))
    // Add default role to the top... if we have one.
    if (default_role) {
      result.splice(0, 0, default_role)
      }
    return result
  }

    /*
    Return a list with all administrators.
    */
    $scope.getAllAdministrators = function() {
        return $scope.getEntities('administrators')
    }

    /*
    Return a list with all event handlers.
    */
    $scope.getAllEventHandlers = function() {
        return $scope.getEntities('event_handlers')
    }


    /*
    Return the list of all transfers.

    This should include:
    * configured transfers on the server
    * pending transfers to be added
    FIXME:5628:
    * pending transfers to be removed
    */
    $scope.getAllTransfers = function() {
        return $scope.getEntities('transfers')
    }

  /*
  Return a list with all the available authentication methods,
  including the newly created ones.
  */
  $scope.getAllAuthentications = function() {
      return $scope.getEntities('authentications')
  }

    /*
    Return all the components that an event handler can use.

    FIXME:5616:
    Extend this to all components, so that we can use the same code for
    search.
    */
    $scope.getEventComponents = function() {
      var result = []

      angular.forEach(
        $scope.getEntities('transfers'),
        function(value, key) {
            var component = angular.copy(value)
            component['family'] = 'transfer'
            result.push(component)
        }
      )

      angular.forEach(
          $scope.getEntities('locations'),
          function(value, key) {
              var component = angular.copy(value)
              component['family'] = 'location'
              result.push(component)
          }
      )

      angular.forEach(
          $scope.getEntities('event_handlers'),
          function(value, key) {
              var component = angular.copy(value)
              component['family'] = 'handler'
              result.push(component)
          }
      )

      angular.forEach(
          $scope.getEntities('services'),
          function(value, key) {
                var component = angular.copy(value)
                component['family'] = 'service'
                result.push(component)
          }
      )

      angular.forEach(
          $scope.getEntities('resources'),
          function(value, key) {
              var component = angular.copy(value)
              component['family'] = 'resource'
              result.push(component)
          }
      )
      return result
  }


  /*
  Return all entities from a family, including those configured on server
  and newly created.

  We can have:

  * SET - -> configuration/SET
  * authentications/UUID/SET -? configuration.authentications.UUID.SET
  */
  $scope.getEntities = function(family) {
    var parts = family.split('/')
    var result = []
    var configuration

    if (!$scope.configuration[parts[0]]) {
      return result
    }

    if (parts.length == 3) {
      configuration = $scope.configuration[parts[0]][parts[1]][parts[2]]
    } else {
      configuration = $scope.configuration[family]
    }

    angular.forEach(
      $scope.new_entities[family],
      function(value, index) {
        result.push(value)
      })

    angular.forEach(
        configuration,
        function(value, key) {
        result.push(value)
    })
    result.sort(init.sort_by('name', false, init.toUpperCaseFn))
    return result
  }

  /*
  Return latest configuration object for `service_uuid`.
  */
  $scope.getServiceConfiguration = function(service_uuid) {
      return $scope.configuration.services[service_uuid]
  }

  /*
  Return configuration object for `service_uuid` as last defined on server.
  */
  $scope.getServiceUpstreamConfiguration = function(service_uuid) {
      return $scope.upstream_configuration.services[service_uuid]
  }

  /*
  Return true if service is allowed to be removed .
  */
  $scope.isServiceSpecialManager = function(service) {
      if (!service) {
        return false
      }
      if (service.uuid === utils.constants['DEFAULT_LOCAL_MANAGER_UUID']) {
          return true
      } else {
          return false
      }
  }

  /*
  Find entity with name or uuid `search` in the list of `all`.
  */
  $scope._findEntity = function(all, search) {
    var entity = null

    if (!search) {
      return null
    }

    if (!all || all.length < 1) {
      return null
    }

    angular.forEach(all, function(value, index) {
      if (!value['uuid'] && value['name'] == search) {
        entity = value
      }else if (value['uuid'] == search) {
        entity = value
      }
    })

    return entity
  }

  /*
  Return true if value is a valid umask format as string.
  */
  $scope.validateUmask = function(value) {
    if (!angular.isDefined(value)) {
      return true
    }

    var re_octal = new RegExp('^[0-7]{3}$')

    return re_octal.test(value)
  }

  /*
  Return true if value is a valid HTTP url.

  This is a naive check.
  */
  $scope.validateHTTPURL = function(value) {
    if (!value) {
      return true
    }

    if (value.startsWith('http://') || value.startsWith('https://')) {
      return true
    }

    return false
  }


  /*
  Text for show/hide clear text password button.
  */
  $scope.showPasswordButtonText = function(state) {
    if (state) {
      return 'Mask password'
    } else {
      return 'Show password'
    }
  }

  /*
  Disable password for `entity`.
  */
  $scope.disablePassword = function(entity, attribute) {
    var password_attr = attribute ? attribute : 'password_update'
    entity[password_attr] = utils.constants['CONFIGURATION_DISABLED_VALUE']
  }


  /*
  Return a list with all configured accounts and administrators names.
  */
  $scope._getAllAccountsAndAdministratorsNames = function() {
      var result = []
      if (!$scope.configuration) {
          return result
      }

      angular.forEach(
          $scope.getAllAccounts(),
          function(value, key) {
              result.push(value.name)
          })

      angular.forEach(
          $scope.configuration.administrators,
          function(value, key) {
              result.push(value.name)
          })

      return result
  }

  /*
  Return a list with all available service names.
  */
  $scope._getAllServiceNames = function() {
      var result = []
      if (!$scope.configuration.services) {
          return result
      }

      angular.forEach(
          $scope.configuration.services,
          function(value, key) {
              result.push(value.name)
          })

      return result
  }


  /*
  Return a list with all available transfer names.
  */
  $scope._getAllTransferNames = function() {
      var result = []
      if (!$scope.configuration.transfers) {
          return result
      }

      angular.forEach(
          $scope.configuration.transfers,
          function(value, key) {
              result.push(value.name)
          })

      return result
  }

  /*
  Return a list with all available location names.
  */
  $scope._getAllLocationNames = function() {
      var result = []
      if (!$scope.configuration.locations) {
          return result
      }

      angular.forEach(
          $scope.configuration.locations,
          // FIXME:4950:
          // The ClientAvatar should have the location name instead of
          // username.
          function(value, key) {
              result.push(value.username)
          })

      return result
  }

  /*
  Return `true` if service used a network interface.

  ie it has ip/port.
  */
  $scope.hasInterface = function(service) {
    if (!service) {
      return false
    }
    if (service.type == utils.constants['TYPE_NAME']['MONITOR']) {
      return false
    }
    return true
  }


  /*
  Return `false` if not in upstream server configuration.

  This is a LM GUI workaround to unify the resource status.
  */
  $scope.isInUpstreamConfiguration = function(resource) {
    if (!resource || !$scope.process) {
      return false
    }
    var family = resource.family
    if (!family) {
      return false
    }

    if (family == 'location') {
      return $scope.process.configuration.locations[resource.uuid]
    }

    // Fallback to resources.
    return $scope.process.configuration.resources[resource.uuid]
  }

  /*
  Return CSS for state label.
  */
  $scope.getStateClass = function(item) {
      if (!item) {
        return
      }
      var map = {}
      map[utils.constants.STATE.STARTED] = 'label-success'
      map[utils.constants.STATE.FAILED_AT_START] = 'label-important'
      map[utils.constants.STATE.FAILED_AT_STOP] = 'label-important'
      map[utils.constants.STATE.FAILED_AT_RUN] = 'label-important'
      map[utils.constants.STATE.CONNECTING] = 'label-warning'
      map[utils.constants.STATE.DISCONNECTED] = 'label-warning'
      map[utils.constants.STATE.DISCONNECTING] = 'label-warning'
      map[utils.constants.STATE.SOURCE_FAILED] = 'label-warning'
      map[utils.constants.STATE.SOURCE_STOPPED] = 'label-warning'
      map[utils.constants.STATE.DESTINATION_FAILED] = 'label-warning'
      map[utils.constants.STATE.SUSPENDED] = 'label-warning'
      var result = map[item.state]
      if (!result) {
        return ''
      }
      return result
  }


  $scope._unixTimeToISO8601 = function(input) {
      if (!input) {
        return 'no date yet'
      }
      var raw_date = dayjs(input * 1000)
      return raw_date.format()
  }

  /*
  Return the description of runnable state.
  */
  $scope.getStateDetails = function(item) {

    if (!item) {
        // Item not yet loaded.
        return ''
    }

    var STATE = utils.constants.STATE
    var state = item.state

    var get_time = function(timestamp) {
        var at_time = dayjs.unix(timestamp)
        return at_time.fromNow() + ' at ' + at_time.format()
    }

    if (state == STATE.STARTED) {
        return get_time(item.start_time)
    }

    var text = ''
    if (item.last_error) {
        text = $scope.resolveError(item.last_error)
    }

    if (utils.constants.STATES_STOP.contains(state)) {
      text = get_time(item.stop_time) + ' ' + text
    }

    return text

}


  /*
  Return an empty account that can be used in the UI when we fail to find
  a real account.
  */
  $scope.getAccountPlaceholder = function() {
    return {
        'uuid': '',
        'name': '',
        'description': ''
    }
  }

  /*
  Return an empty group that can be used in the UI when we fail to find
  a real group.
  */
  $scope.getGroupPlaceholder = function() {
    return {
        'uuid': '',
        'name': '',
        'description': ''
    }
  }

  /*
  Return an empty role that can be used in the UI when we fail to find
  a real role.
  */
  $scope.getRolePlaceholder = function() {
    // FIXME:5709:
    // Here we should build it based on the default values.
    // Maybe when we add a new role, we start with a placeholder.
    return {
        'uuid': '',
        'name': '',
        'description': '',
        'permissions': [],
        'source_ip_filter': []
    }
  }

  /*
   Get the list of runnables that require restart.
   */
  $scope.getRunnablesRequiringRestart = function() {
    var result = []
    angular.forEach(
      $scope.process.runnables,
      function(runnable_manager, key) {
        angular.forEach(
          runnable_manager,
          function(value, key) {
            if (!value.hasOwnProperty('restart_required')) {
              return
            }
            if (value['restart_required'].length > 0) {
              result.push(value)
            }
          }
        )
      })
    return result
  }

  /*
   Start given `runnable`.
   */
  $scope.onStartRunnable = function(runnable) {
    $scope._actOnRunnableManager(
      runnable,
      utils.constants.STATE.STARTING,
      'runnable_start'
    )
  }

  /*
   Stop given `runnable`.
   */
  $scope.onStopRunnable = function(runnable) {
    $scope._actOnRunnableManager(
      runnable,
      utils.constants.STATE.STOPPING,
      'runnable_stop'
    )
  }

  /*
   Restart given `runnable`.
   */
  $scope.onRestartRunnable = function(runnable) {
    $scope._actOnRunnableManager(
      runnable,
      utils.constants.STATE.RESTARTING,
      'runnable_restart'
    )
  }

  /*
   Change the state of given `runnable` to the given state `new_state`
   using `jsonrpc_method`.
   */
  $scope._actOnRunnableManager = function(
    runnable, new_state, jsonrpc_method) {

    var runnable_group = family_to_category[runnable.family]
    var uuid = runnable.uuid
    var initial_state = runnable.state
    // We need to inject the variables into the local scope as the JSON-RPC
    // method is executed in an external context.
    var parent = $scope.process.runnables[runnable_group]
    var configuration = $scope.configuration
    var upstream_configuration = $scope.upstream_configuration
    var new_entities = $scope.new_entities

    // While the JSON-RPC will not respond we will not have an updated state
    // for the runnable. This is why we force an initial update here.
    runnable.state = new_state
    runnable.operating = false
    runnable.started = true

    utils.jsonrpc({
      'method': jsonrpc_method,
      'params': {
        'uuid': uuid,
        'family': runnable_group
      },
      'success': function(data, status) {
        var result = data['result']
        angular.copy(result, parent[uuid])

        // This is a hack which syncs the local-file authentication runnable
        // configuration with the main configuration as the current local
        // manager does not support managing the configuration for a runnable.
        if (result.accounts !== undefined) {
            new_entities['authentications/' + result.uuid + '/accounts'] = []
            angular.copy(
                result.accounts,
                configuration.authentications[result.uuid].accounts
                )
            angular.copy(
                result.accounts,
                upstream_configuration.authentications[result.uuid].accounts
                )
        }
        if (result.groups !== undefined) {
            new_entities['authentications/' + result.uuid + '/groups'] = []
            angular.copy(
                result.groups,
                configuration.authentications[result.uuid].groups
                )
            angular.copy(
                result.groups,
                upstream_configuration.authentications[result.uuid].groups
                )
        }
      },
      'error': function(data, status) {
        runnable.state = initial_state
        $scope.alertError(data['error'])
      },
      'alerts': $scope.alerts
    })
    // Trigger the auto-update to check the status of the runnable.
    $scope.onStartAutoUpdate()
  }


/*
  Return true if value contains only valid ssl methods.
  */
  $scope.validateSSLMethod = function(value) {
      if (!value) {
          return true
      }
      var result = true
      var methods = value.split(' ')
      angular.forEach(methods, function(value, index) {
          value = value.trim()
          if (!utils.constants['SSL_SUPPORTED_METHODS'].contains(value)) {
              result = false
          }
      })
      return result
  }


  /*
  Send JSON-RCP request for applying changes.
  */
  $scope.applyChanges = function(changes) {
    $scope.alerts.push({type: 'info', msg: 'Applying changes...' })
    var requested_changes = $scope._getRequestedChanges(changes)
    utils.jsonrpc({
      'method': 'apply',
      'params': {'changes': requested_changes},
      'success': function(data, status) {
        $scope._cbApplyUpstreamChanges(data, status, requested_changes)
      },
      'error': $scope._ebApplyChanges,
      'alerts': $scope.alerts
    })
  }

  /*
  Critical error when sending apply changes JSON-RPC request.
  */
  $scope._ebApplyChanges = function(data, status) {
    $scope.clearAlerts()
    var error_message = (
        data['error']['code'].toString() + ' : ' + data['error']['message'])
    $scope.alerts.push({type: 'error', msg: error_message })
  }


  /*
  Convert UI changes into server request changes.

  If filters only attributes required to be sent to the server.
  */
  $scope._getRequestedChanges = function(changes) {
    var result = {}
    angular.forEach(changes, function(value, index) {
      result[index] = {
        'operation': value['operation'],
        'path': value['path'],
        'value': value['value']
        }
    })
    return result
  }

  /*
  Apply new configuration state after change request.
  */
  $scope._cbApplyUpstreamChanges = function(data, status, requested_changes) {
    $scope.clearAlerts()

    // This is critical error so just show an error and keep the current
    // configuration state.
    if (data['error']) {
      var error_message = (
        data['error']['code'].toString() + ' : ' + data['error']['message'])
      $scope.alerts.push({type: 'error', msg: error_message })
      return
    }

    // Start with the same state as upstream.
    // Later we will revert non-applied failed changes.
    angular.copy(
      data['result']['configuration'], $scope.upstream_configuration)
    angular.copy(data['result'], $scope.process)

    $scope.onRevertAll()

    $scope._restoreChanges(requested_changes, data['result']['results'])
    // Not sure why this forced update is needed here... something is fishy.
    $scope._updateChanges()

  }


  /*
  Revert all changes by copying upstream_configuration over the in work
  configuration.
  */
  $scope.onRevertAll = function() {
    angular.copy(
      $scope.upstream_configuration,
      $scope.configuration
      )
    angular.forEach(
      $scope.new_entities,
      function(value, key) { $scope.new_entities[key] = [] }
      )
  }

  /*
  Revert a single change as originally defined in running configuration.
  */
  $scope.onRevert = function(change) {
    var source
    var key
    var result
    if (change['operation'] == 'create') {
      source = $scope._getNewEntitiesSource(change['path'])
      source.splice(change['index'], 1)
      return
    }

    result = $scope._getConfigurationSource(change['path'])
    source = result[0]
    key = result[1]
    source[key] = change['old']
  }

  /*
  Get account from uuid.

  For now, this only works with accounts stored in the main config file.
  */
  $scope.resolveAccount = function(uuid) {
    // Start with a placeholder account.
    // We are relaxed to not break the UI.
    var result = $scope.getAccountPlaceholder()

    if (!$scope.configuration || !$scope.configuration.accounts) {
        // Configuration not yet loaded.
        return result
    }

    angular.forEach(
      $scope.configuration.accounts,
      function(value, key) {
          if (key == uuid) {
              result = value
          }
    })

    return result
  }


  /*
  Get group from group uuid.
  */
  $scope.resolveGroup = function(uuid, tab) {
    // Start with a placeholder group.
    // We are relaxed to not break the UI.
    var result =  $scope.getGroupPlaceholder()

    if (!$scope.configuration || !$scope.configuration.groups) {
        // Configuration not yet loaded.
        return result
    }

    var current_groups = $scope.configuration.groups
    if (tab) {
        current_groups = $scope.getEntities(tab.groups_path)
    }

    angular.forEach(
        current_groups,
        function(value, key) {
            if (value.uuid == uuid) {
                result = value
            }
        }
    )
    return result
  }

  /*
  Get role from role uuid.
  For now, this only works with roles stored in the main config file.
  */
  $scope.resolveRole = function(uuid) {
    // Start with a placeholder role.
    // We are relaxed to not break the UI.
    var result = $scope.getRolePlaceholder()

    if (!$scope.configuration || !$scope.configuration.roles) {
        // Configuration not yet loaded.
        return result
    }

    angular.forEach(
        $scope.configuration.roles,
        function(value, key) {
            if (key == uuid) {
                result = value
            }
        }
    )
    return result
  }



  /*
  When configuration was updated, but there are still changes that were not
  applied, we need to apply the pending changes on the new configuration
  object.
  */
  $scope._restoreChanges = function(changes, results) {
    angular.forEach(changes, function(change, key) {
      var error_message
      var response = results[key]

      if (!response) {
        error_message = (
          'Server did not send a response for ' + changes['path'])
        $scope.alerts.push({type: 'error', msg: error_message})
        $scope._restoreChangeRequest(change)
      } else if (response['error']) {
        $scope.alertError(response['error'])
        $scope._restoreChangeRequest(change)
      }

    })

    angular.forEach($scope.changes, function(change, index) {
      var change_requested = false
      angular.forEach(changes, function(requested_change, key) {
        if (change['operation'] == 'create') {
            if (change['value'] == requested_change['value']) {
              change_requested = true
            }
        } else if (change['path'] == requested_change['path']) {
          change_requested = true
        }
      })

      if (!change_requested) {
        $scope._restoreChangeRequest(change)
      }
    })
  }

  /*
  Apply requested change to the main configuration.
  */
  $scope._restoreChangeRequest = function(change_request) {
    var result
    var source
    var key
    var path = change_request['path']
    var value = change_request['value']

    if (change_request['operation'] === 'create') {
      source = $scope._getNewEntitiesSource(path)
      source.push(value)
    } else if (change_request['operation'] === 'delete') {
      result = $scope._getConfigurationSource(path)
      source = result[0]
      key = result[1];
      delete source[key]
    } else {
      result = $scope._getConfigurationSource(path)
      source = result[0]
      key = result[1]
      source[key] = value
    }
  }


  /*
  Return the array used for storing entities from category associated
  with `path`.
  */
  $scope._getNewEntitiesSource = function(path) {
    var result = path.split('/')
    var source = result[0]
    return $scope.new_entities[source]
  }


  /*
  Return a tuple of configuration parent and configuration key associated
  with `path`.
  */
  $scope._getConfigurationSource = function(path) {
    var result = path.split('/')
    var source = $scope.configuration
    var previous_source
    var key

    angular.forEach(result, function(value, index) {
      previous_source = source
      source = previous_source[value]
      key = value
    })
    return [previous_source, key]
  }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.mainCtrl = {} : exports))
