// This is named past_events.js as with activity_log.js is blocked by
// ad-blockers.
(function(exports){

exports.activityLogCtrl = function(
    $scope,
    $timeout,
    ng_table_mixin,
    dayjs,
    utils
) {

    var init = require('../../../js_commons/__init__.js', 'init')

    $scope.$parent.component = {
        'name': 'Activity log',
        'summary': 'Logs for current and past activity of this application.',
    }

    ng_table_mixin.NGTableMixin($scope)
    // The current selected DB.
    $scope.table_options.db_uuid = null
    // This is what goes to the JSON-RPC.
    $scope.table_options.criteria = []

    $scope.filter_ui = {
        // Filter rule pending to be added.
        'column': '',
        'comparison': '',
        'value': '',
        // Available comparison options.
        'options': {},
        // Current filter options, handled by the UI.
        // This is an object with each column name as a key.
        // The values are lists of [columns_name, comparison, value]
        'criteria': {},
    }


    /*
    Return the available comparison options for each column.
    */
    var _getComparisons = function(column) {
        if (column == 'event') {
            return [
                {'v': '=', 't': 'Equals'},
            ]
        }
        return [
            {'v': '=', 't': 'Equals'},
            {'v': '~', 't': 'Contains'},
        ]
    }

    /*
    Return true if the `column_name` has the `expected` comparison as a
    valid option.
    */
    var _hasComparison = function(column_name, expected) {
        var result = false
        angular.forEach(_getComparisons(column_name), function(comparison_ui) {
            if (comparison_ui.v == expected) {
                result = true
            }
        })
        return result
    }

    var _resetComparisons = function() {

        $scope.filter_ui.options = {
            'event': _getComparisons('event'),
            'peer': _getComparisons('peer'),
            'user': _getComparisons('user'),
            'text': _getComparisons('text'),
        }
        $scope.filter_ui.column = 'event'
        $scope.filter_ui.comparison = '='
    }

    _resetComparisons()

    // The current event for which we should show the details,
    // as offset inside the current batch.
    $scope.event_details_cursor = null
    $scope.event_ui = {
        'cursor': null,
        'have_previous': false,
        'have_next': false,
    }

    $scope.date_ui = {
        'duration': 'any date',
        'visible': false,
        'range_value': 0,
        'range_unit': 'hour'
    }

    var _TIMESTAMP_COLUMN = 'timestamp'
    var _PEER_COLUMN = 'client_peer'
    // Available sort options.
    $scope.columns = [
        {'value': 'status', 'text': ''},
        {'value': _TIMESTAMP_COLUMN, 'text': 'Date'},
        {'value': 'event_id', 'text': 'Event'},
        {'value': _PEER_COLUMN, 'text': 'Session'},
        {'value': 'message', 'text': 'Details'},
        {'value': 'link', 'text': ''},
    ]
    $scope.sort_options = {
        'default_column': _TIMESTAMP_COLUMN,
        'default_order': 'desc',
        'columns': [
            {'value': _TIMESTAMP_COLUMN, 'text': 'Date'},
            {'value': 'event_id', 'text': 'Event'},
            {'value': 'user_name', 'text': 'User'},
            {'value': _PEER_COLUMN, 'text': 'Session'},
        ],
        // These will be updated by the component.
        'reset': function() {
            $scope.table_options.sorting = {}
            $scope.table_options.sorting[_TIMESTAMP_COLUMN] = 'desc'},
        'sort_up': {},
        'sort_down': {},
    }

    $scope.__init__ = function() {
        $scope._updateIdentity('user_name')
        $scope._updateServices('service_name')
        _updateHandlers()
    }

    /*
    We watch for the config changes to show the content if we reload the page.
    On a refresh, there is no application state and the application state
    is loaded a bit later via JavaScript.

    Once we have the application state, we can request the activity log data.
    */
    $scope.$watch('configuration.services', function(new_value, old_value) {
        _updateHandlers()
        $scope._updateTable()

        if (new_value == old_value) {
            // Nothing is changed at initialization.
            return
        }
        $scope._updateServices()
    })

    $scope.$watch('date_start_ui.date_value', function(new_value, old_value) {

        if (!new_value) {
            $scope.date_end_ui.minDate = null
            return
        }

        $scope.date_end_ui.minDate = new_value
    })

    $scope.$watch('date_end_ui.date_value', function(new_value, old_value) {

        if (!new_value) {
            $scope.date_start_ui.maxDate = Date.now()
            return
        }

        $scope.date_start_ui.maxDate = new_value
    })

    /*
    Called when the "event" combo box is updated for the "Add filter" UI.
    */
    $scope.$watch('filter_ui.column', function(new_value, old_value) {
        if (!new_value) {
            // Most probably not loaded yet.
            $scope.filter_ui.comparison = '='
            return
        }

        if (_hasComparison(new_value, $scope.filter_ui.comparison)){
            // The current comparison is value for the new column.
            return
        }

        // Default to "equal" as the comparison for the new column.
        $scope.filter_ui.comparison = '='
    })

    /*
    Called for any key pressed in the add-filter value input.
    */
    $scope.onFilterValueKeyDown = function(event) {
        if (event.keyCode != 13) {
            // Not the Enter key.
            return
        }
        // Add the current filter on Enter key.
        $scope.onAddFilter()
    }

    var resetDateUI = function() {
        $scope.date_start_ui.date_value = null
        $scope.date_start_ui.time_value = null
        $scope.date_end_ui.date_value = null
        $scope.date_end_ui.time_value = null
        $scope.date_ui.duration = 'any date'
        $scope.date_ui.visible = false
    }

    /*
    Called when all the filters should be reset.
    */
    $scope.onReset = function() {
        resetDateUI()

        $scope.filter_ui.criteria = {}
        _resetComparisons()
        $scope.table_ui.range_start = 1
        $scope.table_ui.range_end = 50
        $scope.table_ui.total = 'many'
        $scope.table_options.criteria = []
        $scope.sort_options.reset()

        // We set the page to 0 to trigger a DB load.
        $scope.table_options.page = 0
    }

    /*
    Called when we should reset only the date filter.
    */
    $scope.onDateReset = function() {
        resetDateUI()
        // FIXME:6465:
        // Have date reset button outside of the dropdown, so that
        // we don't need this hack.
        $timeout(function() {$scope.date_ui.visible = false}, 50)

        // We update the page and this will trigger an update.
        $scope.table_options.page = 0
    }

    /*
    Called when the date filter button is pressed.
    */
    $scope.onDateFilter = function(range_value, range_unit) {
        $scope.date_ui.visible = false

        var start = null // We might not have a start date.
        var start_time = null
        var end = dayjs() // By default we end with now.
        var end_time = null
        var range_value = range_value
        var range_unit = range_unit

        if (!range_value) {
            // Reset the range values.
            range_value = $scope.date_ui.range_value
            range_unit = $scope.date_ui.range_unit
        }

        if (range_value) {
            // A fixed range value was requested.
            // It only has a start reference, without an end.
            //
            // Start by resetting to value, so that next time when we try to
            // update, it will not be enabled by default.
            $scope.date_ui.range_value = 0

            start = dayjs().subtract(range_value, range_unit)
            $scope.date_start_ui.date_value = start.toDate()
            $scope.date_start_ui.time_value = start.toDate()
            // We will filter until present.
            $scope.date_end_ui.date_value = null
            $scope.date_end_ui.time_value = null
        } else {
            // A custom range was selected.
            start_time = $scope.date_start_ui.time_value
            if (start_time) {
                start = dayjs($scope.date_start_ui.date_value
                    ).set('hour', start_time.getHours()
                    ).set('minute', start_time.getMinutes()
                    ).set('second', start_time.getSeconds())
            }

            end_time = $scope.date_end_ui.time_value
            if (end_time) {
                end = dayjs($scope.date_end_ui.date_value
                    ).set('hour', end_time.getHours()
                    ).set('minute', end_time.getMinutes()
                    ).set('second', end_time.getSeconds())
            }
        }

        // Update the duration, if we have a starting point.
        if (start) {
            $scope.date_ui.duration = start.to(end, true)
        } else if (!end_time) {
            // No start and end date selected.
            // Most probably the filter button was pressed
            // without selecting anything.
            // Show default UI without loading any extra data.
            $scope.date_ui.duration = 'any date'
            return
        } else {
            // We don't have a start date,
            // but we have an end date.
            // We can't compute the duration.
            $scope.date_ui.duration = ''
        }
        $scope._updateTable()
    }

    /*
    Update UI depending on database handlers configuration.
    */
    var _updateHandlers = function() {
        $scope.handlers = $scope._getDBHandlers()
        if ($scope.handlers.length > 0) {
            $scope.table_options.db_uuid = $scope.handlers[0].uuid
        }
    }

    /*
    Return a list with configurations of all event handler databases.
    */
    $scope._getDBHandlers = function() {
        var result = []

        // Return empty list if configuration is not ready yet.
        if (!$scope.configuration.event_handlers) {
            return result
        }

        angular.forEach(
            $scope.configuration.event_handlers,
            function(value, key) {
                // Build the list of database event handlers.
                if (value.type == 'database') {
                    result.push(value)
                }
            })

        result.sort(init.sort_by('name', false, init.toUpperCaseFn))
        return result
    }

    /* Load/refresh the table with backend data */
    $scope._updateTable = function(callback) {
        if (!$scope.table_options.db_uuid) {
            return
        }

        if (!$scope.table_options.page) {
            // We have an invalid page number.
            // A valid value will be auto-set soon.
            /// Don't do anything for now.
            $scope.table_options.page = 1
            return
        }

        $scope.in_progress = true
        $scope.table_options.criteria = $scope._sanitizeCriteria()
        $scope._dbFetch(utils, callback)
    }

    /*
    Called for each server-side entry of the table to convert and filter data.
    */
    $scope._updateEntryData = function(event) {
        var timestamp = event['timestamp']
        // Convert timestamp to datetime value for easy manipulation.
        event.datetime = dayjs(1000 * timestamp)
        event.date = event.datetime.format('YYYY-MM-DD')
        event.time = event.datetime.format('HH:mm:ss')
        event.main_group = 'informational'

        var event_definition = utils.constants.EVENTS_DB.events[event.event_id]
        var event_groups = []

        if (!event_definition) {
            // This is in case we get an event with an unknown event id.
            event.main_group = 'failure-critical'
        } else {
            event_groups = event_definition.groups
        }

        if (event_groups.contains('success')) {
            event.main_group = 'success'
        }
        if (event_groups.contains('failure')) {
            event.main_group = 'failure'
        }
        if (event_groups.contains('failure-high')) {
            event.main_group = 'failure-high'
        }
        if (event_groups.contains('failure-critical')) {
            event.main_group = 'failure-critical'
        }

        // Data is returned as a serialized JSON as stored in DB,
        // and we need to deserialize it.
        event.data = JSON.parse(event.data)
        // We remove the account as it should be displayed as part of the
        // details and not the data section.
        delete event.data['account']
    }

    $scope.onShowEventDetails = function(index) {
        var cursor = index
        // Start by assuming that we have presious or next elements.
        // We will update them later, as we get more data.
        $scope.event_ui.have_previous = true
        $scope.event_ui.have_next = true

        var callback = function() {
            $scope.event_ui.cursor = cursor
            $scope.event = $scope.entries[cursor]
            var entries_count = $scope.entries.length
            var limit = $scope.table_options.limit

            // Only check that we are on the very first entry.
            if (cursor === 0 && $scope.table_options.page == 1) {
                $scope.event_ui.have_previous = false
            }

            if (cursor == entries_count - 1 && entries_count < limit) {
                $scope.event_ui.have_next = false
            }
        }

        if (cursor < 0) {
            // Navigate to last event from previous page.
            $scope.table_options.page -= 1
            cursor = $scope.table_options.limit - 1
            // FIXME:6218:
            // By explicitly calling update here,
            // This will trigger the JSON-RPC twice,
            // as table_options.page already has a watching update.
            // See how we can call it only once.
            $scope._updateTable(callback)
        } else if (cursor >= $scope.table_options.limit) {
            // Navigate to first event on next page.
            $scope.table_options.page += 1
            cursor = 0
            // FIXME:6218:
            // This will trigger the JSON-RPC twice.
            // See how we can call it only once.
            $scope._updateTable(callback)
        } else {
            callback()
        }
    }

    /*
    Return the link to the Download CSV page.
    */
    $scope.getDownloadHREF = function() {
        var options = {}
        angular.copy($scope.table_options, options)

        // Convert filter to criteria.
        options.criteria = $scope._sanitizeCriteria()
        delete options.filter
        return (
            utils.constants.BASE_URL_PATH +
            '/activity-log-export/' + btoa(JSON.stringify(options))
        )
    }

    // Covert from UI column name to backend DB name.
    var _DB_COLUMN = {
        'event': 'event_id',
        'peer': 'client_peer',
        'user': 'user_name',
        'text': 'message',

    }
    // Covert from UI operation name to backend DB name.
    // We should update the backend to directly support this.
    var _DB_OPERATION = {
        '=': '=',
        '!=': '!=',
        '~': 'contains',
        '!~': '!contains',
    }

    /*
    Convert from UI values to JSON-RPC API.
    */
    $scope._sanitizeCriteria = function() {
        var criteria = []

        angular.forEach($scope.filter_ui.criteria, function(filters, key) {
            if (!filters || filters.length == 0) {
                // Somehow ended with an empty rule.
                return
            }
            var column_criterias = []
            angular.forEach(filters, function(filter_rule) {
                var column_name = _DB_COLUMN[filter_rule[0]]
                var operation = _DB_OPERATION[filter_rule[1]]
                var value = filter_rule[2]
                column_criterias.push([column_name, operation, value])
            })
            criteria.push(column_criterias)
        })

        if ($scope.date_start_ui.date_value) {
            var filter_date = new Date($scope.date_start_ui.date_value)
            if ($scope.date_start_ui.time_value) {
                filter_date.setHours(
                    $scope.date_start_ui.time_value.getHours(),
                    $scope.date_start_ui.time_value.getMinutes(),
                    $scope.date_start_ui.time_value.getSeconds()
                )
            }

            criteria.push([[
                'timestamp', '>=', filter_date.getTime() / 1000]])
        }

        if ($scope.date_end_ui.date_value) {
            var filter_date = new Date($scope.date_end_ui.date_value)
            if ($scope.date_end_ui.time_value) {
                filter_date.setHours(
                    $scope.date_end_ui.time_value.getHours(),
                    $scope.date_end_ui.time_value.getMinutes(),
                    $scope.date_end_ui.time_value.getSeconds()
                )
            }
            criteria.push([[
                'timestamp', '<=', filter_date.getTime() / 1000]])
        }

        return criteria
    }

    /*
    Called when we should exit from the event details page.
    */
    $scope.onShowEventsOverview = function() {
        $scope.event = null
    }

    /*
    Called when we should go to the details of the previous event.
    */
    $scope.onGoToPrevious = function() {
        $scope.onShowEventDetails($scope.event_ui.cursor - 1)
    }

    /*
    Called when we should go to the details of the next event.
    */
    $scope.onGoToNext = function() {
        $scope.onShowEventDetails($scope.event_ui.cursor + 1)
    }

    /*
    Called when the whole table should be filtered.
    */
    $scope.onFilter = function() {
        $scope.table_ui.total = 'many'
        $scope.onAddFilter()
        $scope._updateTable()
    }

    /*
    Called when a new filter rule should be added.
    */
    $scope.onAddFilter = function() {
        var column_name = $scope.filter_ui.column
        var comparison = $scope.filter_ui.comparison
        var value = $scope.filter_ui.value
        if (!value) {
            return
        }
        var column_filters = $scope.filter_ui.criteria[column_name]
        if (!column_filters) {
            column_filters = []
            $scope.filter_ui.criteria[column_name] = column_filters
        }
        column_filters.push([column_name,comparison, value])
        $scope.filter_ui.value = ''
    }

    /*
    Called when a current filter rule should be removed.
    */
    $scope.onDeleteFilter = function(rule) {
        var column_name = rule[0]
        $scope.filter_ui.criteria[column_name].remove(rule)
        if ($scope.filter_ui.criteria[column_name].length == 0) {
            // Delete this column category from the filter.
            delete $scope.filter_ui.criteria[column_name]
        }
    }
}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.activityLogCtrl = {} : exports))
