(function(exports){

exports.reportsCtrl = function(
    $scope,
    utils,
    dayjs,
    ng_table_mixin
) {

    ng_table_mixin.NGTableMixin($scope)

    $scope.$parent.component = {
        'name': 'Reports',
        'summary': (
            'The last successful authentication for users and administrators.'
        ),
    }

    var _ACCOUNT_COLUMN = 'account_name'
    var _PEER_COLUMN = 'last_login_successful_peer'
    var _LAST_LOGIN_COLUMN = 'last_login_successful_timestamp'
    var _SERVICE_COLUMN = 'last_login_successful_service'

    $scope.table_options.criteria = []
    $scope.table_options.sorting = {}
    $scope.table_options.sorting[_LAST_LOGIN_COLUMN] = 'desc'
    // Available sort options.
    $scope.sort_options = {
        'default_column': _LAST_LOGIN_COLUMN,
        'default_order': 'desc',
        'columns': [
            {'value': _ACCOUNT_COLUMN, 'text': 'Username'},
            {'value': _LAST_LOGIN_COLUMN, 'text': 'Date'},
            {'value': _PEER_COLUMN, 'text': 'Source IP'},
        ],
        // These will be updated by the component.
        'reset': function() {},
        'sort_up': {},
        'sort_down': {},
    }
    // Available time interval optional.
    $scope.date_dropdown = {'hide': function() {}}
    $scope.date_button_name = 'Date filter options'
    $scope.date_options = [
        {'value': 'all', 'text': 'Any date'},
        {'value': 'hour', 'text': 'Last hour'},
        {'value': 'day', 'text': 'Last day'},
        {'value': 'week', 'text': 'Last week'},
        {'value': 'month', 'text': 'Last month'},
        {'value': 'year', 'text': 'Last year'},
    ]
    // Will be created at init time, via reset
    $scope.current_filter = {
        'text': '',
        'time': null,
    }

    $scope.__init__ = function() {
        $scope.onReset()
        $scope._updateTable()
    }

    /*
    Called when a search filter is requested.
    */
    $scope.onTextFilter = function(value) {
        $scope._updateTable()
    }

    /*
    Called when the filter should be reset.
    */
    $scope.onReset = function() {
        // Updating the filters should automatically trigger a table update
        // no need
        $scope.current_filter.text = ''
        $scope.sort_options.reset()
        _setDateFilter(null)
    }

    /*
    Update the UI for date filter
    */
    var _setDateFilter = function(value) {
        if (value == 'hour') {
            $scope.current_filter.time = dayjs().subtract(1, 'hour').unix()
            $scope.date_button_name = "Showing last hour"
        } else if (value == 'day') {
            $scope.current_filter.time = dayjs().subtract(1, 'day').unix()
            $scope.date_button_name = "Showing last day"
        } else if (value == 'week') {
            $scope.current_filter.time = dayjs().subtract(7, 'day').unix()
            $scope.date_button_name = "Showing last week"
        } else if (value == 'month') {
            $scope.current_filter.time = dayjs().subtract(31, 'day').unix()
            $scope.date_button_name = "Showing last month"
        } else if (value == 'year') {
            $scope.current_filter.time= dayjs().subtract(356, 'day').unix()
            $scope.date_button_name = "Showing last year"
        } else {
            $scope.current_filter.time= null
            $scope.date_button_name = "Date filter options"
        }
    }


    /*
    Called when the date filter interal is updated.
    */
    $scope.onDateFilter = function(option) {
        _setDateFilter(option.value)
        $scope.date_dropdown.hide()
        $scope._updateTable()
    }

    $scope._updateTable = function(callback) {
        $scope.in_progress = true
        // For reports we use the default DB.
        $scope.table_options.db_uuid = null
        $scope.table_options.criteria = []
        if ($scope.current_filter.text) {
            $scope.table_options.criteria.push([
                [_ACCOUNT_COLUMN, 'contains', $scope.current_filter.text],
                [_PEER_COLUMN, 'contains', $scope.current_filter.text],
                [_SERVICE_COLUMN, 'contains', $scope.current_filter.text],
            ])

        }
        if ($scope.current_filter.time) {
            $scope.table_options.criteria.push([[
                _LAST_LOGIN_COLUMN, '>=', $scope.current_filter.time
            ]])
        }
        $scope._dbFetch(utils, callback)
    }

    /*
    Called for each server-side entry of the table to convert the data.
    */
    $scope._updateEntryData = function(data) {
        // Convert timestamp to datetime value and add it to entry data.
        var timestamp = data[_LAST_LOGIN_COLUMN]
        data['datetime'] = dayjs.unix(timestamp)
    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.reportsCtrl = {} : exports))
