(function(exports){

exports.resourceCtrl = function($scope, $routeParams, utils) {

    $scope.database = null
    $scope.configuration = null
    $scope.template_path = null
    var category = $scope.$parent.component_categories.resources
    $scope.section_uuid = ''
    // True when the resource has the check/test button.
    $scope.has_check = false
    $scope.has_start = true
    // The view-model used when checking a resource.
    $scope.check_result = {
        'message': '',
        'class_icon': '',
    }

    $scope.__init__ = function() {
        var current_resource = $scope._findEntity(
            $scope.getEntities('resources'), $routeParams.uuid)

        if (!current_resource) {
            // Try to see if we have a resource.
            current_resource = $scope._findEntity(
                $scope.getEntities('services'), $routeParams.uuid)
        }

        if (!current_resource) {
            $scope.$parent.goToPage('/resources')
            return
        }

        $scope.resource = current_resource
        $scope.$parent.component = current_resource

        // Names used by some shared UI.
        $scope.configuration = current_resource
        $scope.ssl_configuration = current_resource

        $scope.template_path = $scope.$parent.getTemplatePath(
            'resource', $scope.resource.type)

        $scope.is_delete_blocked = utils.isDeleteBlocked(
            current_resource, 'resource')

        var uuid = current_resource.uuid
        if (uuid) {
            // Only resources that are saved on the server have runnables.
            $scope.$parent.runnable = (
                $scope.$parent.process.runnables.resources[uuid])
            if (!$scope.$parent.runnable) {
                // Fallback to looking for services.
                $scope.$parent.runnable = (
                    $scope.$parent.process.runnables.services[uuid])
            }
            $scope.section_uuid = 'resources/' + uuid
        }

        $scope.has_enabled = (
            ($scope.resource.type == utils.constants.TYPE_NAME.LETS_ENCRYPT) ||
            ($scope.resource.type == utils.constants.TYPE_NAME.ANALYTICS) ||
            ($scope.resource.type == utils.constants.TYPE_NAME.MONITOR)
            )

        if ($scope.resource.type == utils.constants.TYPE_NAME.EMAIL_CLIENT) {
            $scope.has_check = true
        }


        if (!$scope.resource.uuid) {
            $scope.has_start = false
        }
    }

    /*
    Delete a resource.
    */
    $scope.onDeleteResource = function() {
        category.delete($scope.resource)
        $scope.goToPage('/resources')
    }

    /*
    Check the current unsaved configration for a resource.

    For not it only supports testing email resources,
    but we should design it to be used with any resource type.
    */
    var checking_resource = false
    $scope.onCheckResource = function() {
        if (checking_resource) {
            // Do nothing if the button is pressed multiple times while
            // we are waiting for a response from a previous check.
            return
        }
        checking_resource = true
        // Show a pending generic message.
        $scope.check_result.message = 'Checking the current configuration...'
        $scope.check_result.class_icon = 'fas fa-info-circle'

        utils.jsonrpc({
            'method': 'runnable_check',
            'params': {
                'family': 'resources',
                'configuration': $scope.resource,
            },
            'success': function(data) {
                checking_resource = false
                $scope.check_result.message = data.result
                $scope.check_result.class_icon = (
                    'fas fa-check-circle text--success')
                },
            'error': function(data) {
                checking_resource = false
                $scope.check_result.message = (
                    'Test failed. ' + $scope.resolveError(data.error))
                $scope.check_result.class_icon = (
                    'fas fa-times-circle text--important')
                }
        })

    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.resourceCtrl = {} : exports))
