(function(exports){

exports.reviewCtrl = function($scope, utils) {

  $scope.$parent.component = {
    'name': 'Review configuration changes',
    'summary': 'Review pending configuration changes.',
  }

  /*
  Return all changes.
  */
  $scope.getAllChanges = function() {
    return $scope.$parent.changes
  }


  /*
  Apply a single change.
  */
  $scope.onApply = function(change) {
    $scope.applyChanges([change])
  }

  /*
  Apply all changes.
  */
  $scope.onApplyAll = function() {
    $scope.applyChanges($scope.getAllChanges())
  }

  /*
  Go to the configuration page of the change.
  */
  $scope.onGoToChange = function(change) {
    var link = $scope._getChangeLink(change)
    $scope.$parent.goToPage(link)
  }

  $scope._getChangeLink = function(change) {
    var parts = change['path'].split('/')
    var link

    if (parts[0] == 'server' && parts[1] == 'authentications') {
        return 'authentications'
    }

    if (['server', 'reports', 'node_variables'].contains(parts[0])) {
        return 'server'
    }

    link = parts[0].replace('_', '-')
    if (parts[1]) {
        link = link + '/' + parts[1]
    }

    if (change['operation'] === 'create') {
        link = link + '/' + change['value']['name']
    }

    return link
  }

  /*
  Return the name of the component associated with this change.
  */
  var _getNodeVariablesChangeName = function(change) {

    var parts = change['path'].split('/')
    // We know the first part is `node_variables` so we discard it
    // and use the alternative separator to identity the name of the
    // associated component.
    parts = parts[1].split('|')

    if (parts[0] == 'server') {
        return 'Server > ' + parts[1]
    }
    return $scope.configuration[parts[0]][parts[1]].name + ' > ' + parts[2]

  }

  $scope.getChangeName = function(change) {

    if (change['path'].startsWith('node_variables')) {
      return _getNodeVariablesChangeName(change)
    }

    if (change['operation'] == 'create') {
      return change['value']['name']
    } else if (change['operation'] == 'delete') {
      return change['old']['name']
    } else {
      var entity = $scope._getChangeEntity(change)
      if (entity) {
        return '%(name)s > %(child)s'.supplant({
          'name': entity['name'],
          'child': change['path'].split('/').pop()
        })
      } else {
        return change['path'].replace('/', ' > ')
      }
    }
  }

  /*
  Return the component associated with this change.
  */
  $scope._getChangeEntity = function(change) {
    var path = change['path'].split('/')
    if (path[0] == 'reports') {
       return null
    } else if (path[0] == 'server') {
       return null
    } else {
      var section = $scope.$parent.configuration[path[0]]
      if (!section) {
        return null
      }
      return section[path[1]]
    }
  }

  /*
  Return human readable type for a change.
  */
  $scope.getChangeType = function(change) {
    if (change['path'].startsWith('services')) {
      return 'Protocol'
    } else if (change['path'].startsWith('administrators')) {
      return 'Administrator'
    } else if (change['path'].startsWith('roles')) {
      return 'Role'
    } else if (change['path'].startsWith('groups')) {
      return 'Group'
    } else if (change['path'].startsWith('accounts')) {
      return 'Account'
    } else if (change['path'].startsWith('event_handlers')) {
      return 'Event Handler'
    } else if (change['path'].startsWith('locations')) {
      return 'Location'
    } else if (change['path'].startsWith('transfers')) {
      return 'Transfer'
    } else if (change['path'].startsWith('authentications')) {
      return 'Authentication'
    } else if (change['path'].startsWith('resources')) {
      return 'Resource'
    } else if (change['path'].startsWith('node_variables')) {
      return 'Node variables'
    } else {
      return 'General'
    }
  }

  /*
  Return `true` if `changed` is for a value that should be masked.
  */
  var isChangeMasked = function(change) {
    return (change['path'].endsWith('password_update') ||
          change['path'].endsWith('password') ||
          change['path'].endsWith('multi_factor_authentication')
          )
  }

  $scope.getChangeOldValue = function(change) {
    var result
    if (change['operation'] === 'delete') {
      return 'Just removed.'
    } else {
      if (isChangeMasked(change)) {
        return 'Masked value'
      } else {
        result = change['old']
        if (result === '') {
          return 'EMPTY'
        } else if (result === null) {
          return 'NOT DEFINED'
        } else {
          return result
        }
      }
    }
  }

  $scope.getChangeNewValue = function(change) {
    var result
    if (change['operation'] === 'create') {
      return 'Just created.'
    } else {
      if (isChangeMasked(change)) {
        return 'Masked value'
      } else {
        result = change['value']
        if (result === '') {
          return 'EMPTY'
        } else if (result === null) {
          return 'NOT DEFINED'
        } else {
          return result
        }
      }
    }
  }

  /*
  Return the CSS class for operation label.
  */
  $scope.getOperationClass = function(change) {
    css_class = {
      'create': 'badge-success',
      'update': '',
      'delete': 'badge-important'
    }
    return css_class[change['operation']]
  }


}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.reviewCtrl = {} : exports))
