/*
Tests for role edit page.
*/
describe('roleCtrl', function() {


    /*
    It goes to the start page when there are no roles, as this is a sign
    that the configuration was not yet loaded, and there is no
    point in searching for the role with the UUID from the URL.
    */
    test('init_no_roles', function() {
        var sut = mk.getController('roleCtrl')

        sut.scope.$parent.configuration.roles = {}
        sut.scope.$parent.new_entities.roles = []
        sut.route.uuid = 'some-new-role'

        sut.scope.__init__()

        expect(sut.$location.path()).toEqual('/')

    })

    /*
    It goes to the roles list page when we have roles, but the one
    from the URL is not found.
    */
    test('init_role_not_found', function() {
        var sut = mk.getController('roleCtrl')

        sut.scope.$parent.configuration.roles = {}
        sut.scope.$parent.new_entities.roles = [{
            'name': 'new-group'
        }]
        sut.route.uuid = 'unknown-uuid'

        sut.scope.__init__()

        expect(sut.$location.path()).toEqual('/roles')

    })

    /*
    It load a role page from a new role based on the name extracted
    from the URL.
    */
    test('init_role_new', function() {
        var sut = mk.getController('roleCtrl')
        var new_role = {
            'name': 'new-role'
        }
        sut.scope.$parent.configuration.roles = {}
        sut.scope.$parent.new_entities.roles = [new_role]
        sut.route.uuid = 'new-role'
        expect(sut.scope.role).toBeEmpty()

        sut.scope.__init__()

        expect(sut.scope.role).toBe(new_role)
        // New roles should newer be created in secondary mode,
        // so they don't have UUID and a section_uuid.
        expect(sut.scope.section_uuid).toBe('')
    })

    /*
    It load a role page from an exsiting role based on the UUID extracted
    from the URL.
    */
    test('init_role_existing', function() {
        var sut = mk.getController('roleCtrl')
        var role = {
            'uuid': 'some-uuid',
            'name': 'existing role',
        }
        sut.scope.$parent.configuration.roles = {'some-uuid': role}
        sut.route.uuid = 'some-uuid'

        sut.scope.__init__()

        expect(sut.scope.role).toBe(role)
        expect(sut.scope.section_uuid).toBe('roles/some-uuid')
    })


})
