(function(exports){

    exports.rolesCtrl = function($scope, utils, $timeout) {

    var init = require('../../../js_commons/__init__.js', 'init')
    // The current list of filtered and sorted roles
    // Will be initialized later.
    $scope.filtered_roles = []
    // Used by the search input tag.
    $scope.filter_input = ''

    $scope.$parent.component = {
        'name': 'Roles',
        'summary': (
            'Groups of administrators. ' +
            'Used to define the access or permission level for administrators.'
        ),
    }

    var category = $scope.$parent.component_categories.roles

    /*
    This is called each time the page is accessed,
    including when using the browser back/forward.
    */
    $scope.__init__ = function() {
        // Trigger an initial filter that will show all.
        $scope.onFilterRoles()

        // The configuration might not be loaded so we force a dumb reload
        // We should find a better way to auto-reload the page but for
        // now it should be ok.
        // In an ideal case, we should use a watch on the configuration,
        // but that might have some side-effects and performance issues.
        // Most of the times, users should not land directly on the roles
        // page, so this hack should be ok.
        if (init.isEmpty($scope.filtered_roles)) {
            $timeout(function() {$scope.onFilterRoles()}, 3000)
        }
    }

    /*
    Called the update the list of roles to be displayed on the page.
    */
    $scope.onFilterRoles = function(filter_value) {
        var view_model
        $scope.filtered_roles = []

        if (!$scope.configuration.roles) {
            // Configuration not yet loaded.
            return
        }

        angular.forEach(
            $scope.getAllRoles(),
            function(role) {
                if (utils.isComponentExcluded(filter_value, role)) {
                    return
                }
                view_model = getRoleView(role)
                $scope.filtered_roles.push(view_model)
            })
    }

    /*
    Update the `roles` based on the current filter.
    */
    var getRoleView = function(configuration) {
        var uuid = configuration.uuid

        return {
            'uuid': uuid,  // To help with testing to identify the row.
            'details': getRoleDetails(configuration),
            'is_delete_blocked': utils.isDeleteBlocked(configuration, 'role'),
            'config': configuration,
        }
    }

    var getRoleDetails = function(configuration) {
        var source_ip_filter = []
        angular.forEach(
            configuration.source_ip_filter,
            function(value) {
                source_ip_filter.push(value[0] + '-' + value[1])
            }
        )

        result = [
            {
                'key': 'Source IPs',
                // FIXME:5729:
                // Better join for multi lines.
                // Here we should just pass a list, and the template
                // should handle the rendering.
                'value': source_ip_filter.join(' ')
            }
        ]
        return result
    }

    /*
    Create a new empty configuration for a role.
    */
    $scope.onAddRole = function() {
        var role = {}
        role['name'] = utils.getRandomMark('New Role ')
        role['enabled'] = true
        role['description'] = utils.maybeParseBoolean(
            utils.constants['CONFIGURATION_DEFAULTS']['description'])
        role['source_ip_filter'] = []
        role['permissions'] = []
        category.add(role)
        category.goToConfiguration(role)
    }

    /*
     Go to configuration page.
     */
    $scope.onGoToConfiguration = function(role) {
        category.goToConfiguration(role)
    }

    /*
     Delete a role.
     */
    $scope.onDeleteRole = function(role) {
        category.delete(role)
        // Update the table after one entry was removed.
        $scope.onFilterRoles($scope.filter_input)
    }
}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.rolesCtrl = {} : exports))
