(function(exports){

exports.serverCtrl = function($scope, utils) {

    var init = require('../../../js_commons/__init__.js', 'init')

    $scope.filtered_variables = []
    $scope.section_uuid = 'server'

    $scope.$parent.component = {
        'name': 'Configuration',
        'summary': 'General application configuration options.',
    }

    var category = $scope.$parent.component_categories.node_variables
    var all_sections = {}

    $scope.__init__ = function() {

        if (!$scope.configuration.server) {
            $scope.$parent.goToPage('/')
            return
        }

        $scope.server = $scope.configuration.server

        all_sections = get_all_sections($scope.configuration)
        $scope.onFilterVariables()

        var schemes = utils.constants.PASSWORD_HASHING_SCHEME_NAME
        $scope.hasing_schemes = []
        for (var option in schemes) {
            $scope.hasing_schemes.push(
                {'type': option, 'name': schemes[option]})
        }
    }

    /*
    Remove a node variable.
    */
    $scope.onDeleteVariable = function(variable_id) {
        category.delete(variable_id)
        $scope.onFilterVariables()
    }


    /*
    Generate a map to get the section name from the section ID
    */
    var get_all_sections = function(configuration) {
        var result = {
            'server': 'Server'
        }

        var accumulate = function(section, value) {
            angular.forEach(value, function(component, uuid) {
                result[section + '/' + uuid] = component.name
            })
        }

        angular.forEach(configuration, function(value, section) {
            /*
            Server section is plain.
            */
            if (section == 'server') {
                return
            }


            /*
            Need to go deep for identity.
            */
            if (section == 'identity') {
                angular.forEach(value, function(identities, kind) {
                    accumulate(kind, identities)
                })
                return
            }

            /*
            All other component collections.
            */
            accumulate(section, value)

        })
        return result
    }

    /*
    Called to setup the variales to be displayed on the page.
    */
    $scope.onFilterVariables = function(filter_value) {
        var result = []
        angular.forEach(
            $scope.configuration.node_variables,
            function(value, variable_id) {
                var section_name = all_sections[value.section]

                if (isFilteredOut(filter_value, section_name, value.key)) {
                    return
                }

                result.push({
                    // Set UUID and name to fit with other components.
                    'uuid': variable_id,
                    'name': value.key,
                    'section_name': section_name,
                    'model': value
                })
            }
        )

        result.sort(init.sort_by('section_name', false, init.toUpperCaseFn))
        $scope.filtered_variables = result
    }

    /*
    Return true if the entry should by filtered out.
    That is not included in the result.
    */
    var isFilteredOut = function(filter_value, section_name, key_name) {
        if (!filter_value) {
            return false
        }

        if (utils.containsInsensitive(section_name, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(key_name, filter_value)) {
            return false
        }

        return true
    }


}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.serverCtrl = {} : exports))
