/*
Tests for controller for server section from configuration.
*/

describe('ServerCtrl', function() {

    var scope
    var utils
    var ctrl
    var result
    var $location

    require('../app.js')
    beforeEach(angular.mock.module('app'))
    beforeEach(inject(function($injector, $rootScope, $controller) {
        /*
        Prepare the scope and controller before each test.
        */
        utils = mk.servicesUtils($injector)

        var parent_scope = $rootScope.$new();
        ctrl = $controller(
            'mainCtrl', {
            $scope: parent_scope,
            utils: utils
            })

        scope = parent_scope.$new();
        ctrl = $controller(
            'serverCtrl', {
            $scope: scope,
            utils: utils
            })
        $location = $injector.get('$location')
        // Run an initial digest to 'settle' all watchers.
        scope.$digest()
    }))


    test(
    'Go to start page if configuration is not yet ready.',
    function() {
        scope.$parent.configuration = {}

        scope.__init__()

        expect($location.path()).toBe('/')
    })

    test(
    'Update page with server section details.',
    function() {
        var section = {
            'name': 'some-name',
            'authentications': ['123', '345']
            }
        scope.$parent.configuration = {'server': section}

        scope.__init__()

        expect(scope.server).toBe(section)
    })


    test(
    'The node variables can be filtered in the table and the result \
    contains the section and key and value for each variable',
    function() {
        scope.$parent.configuration = {
            'server': {
                'name': 'some-name',
                'authentications': ['123', '345']
            },
            'services': {
                'service-1': {
                    'name': 'service sftp',
                    'uuid': 'service-1',
                    'configuration': {
                        'connection_count': 2
                    },
                },
            },
            'identity': {
                'accounts': {
                    'account-1': {
                        'uuid': 'account-1',
                        'name': 'Account 1'
                    }
                },
                'groups': {
                    'group-2': {
                        'uuid': 'group-2',
                        'name': 'group 2'
                    }
                }
            },
            'node_instances': {
                'server/name': {
                    'section': 'server',
                    'key': 'name',
                    'value': 'server override'
                },
                'services/service-1/c': {}
            }
        }


        scope.__init__()



    })


})
