(function(exports){

exports.servicesCtrl = function($scope, utils, componentTypeFilter, $timeout) {
    var details_getter = {}
    var category = $scope.$parent.component_categories.services
    $scope.filter_input = ''
    $scope.filtered_services = []
    $scope.STATE = utils.constants.STATE
    $scope.$parent.component = {
        'name': 'Protocols',
        'summary': (
            'File transfer servers of various protocols provided by the ' +
            'server process.'
        ),
    }

    // List of availabe services
    $scope.service_types = utils.getFamilyTypes(
        utils.constants.TYPE_NAME.SERVICE)

    $scope.__init__ = function() {
        // Trigger an initial filter that will show all.
        $scope.onFilterServices()

        // The configuration might not be loaded so we force a dumb reload
        // We should find a better way to auto-reload the page but for
        // now it should be ok.
        // In an ideal case, we should use a watch on the configuration,
        // but that might have some side-effects and performance issues.
        // Most of the times, users should not land directly on the services
        // page, so this hack should be ok.
        if (init.isEmpty($scope.filtered_services)) {
            $timeout(function() {$scope.onFilterServices()}, 3000)
        }
    }

    /*
    Called the update the list of services to be displayed on the page.
    */
    $scope.onFilterServices = function(filter_value) {
        var view_model
        $scope.filtered_services = []

        if (!$scope.configuration.services) {
            // Configuration not yet loaded.
            return
        }

        angular.forEach(
            getAllServices(),
            function(service) {
                if (utils.isComponentExcluded(filter_value, service)) {
                    return
                }
                view_model = getServiceView(
                    service, $scope.process.runnables.services)
                $scope.filtered_services.push(view_model)
            })
    }

    /*
    Return a list with configuration for all the services.
    */
    var getAllServices = function() {
        var services = $scope.getEntities('services')
        var result = []
        // Do not show services of type monitor in services table.
        angular.forEach(
            services,
            function(service) {
                if (service.type == 'monitor') {
                    return
                }
                result.push(service)
            })

        return result
    }

    /*
    Update the `services` based on the current filter.
    */
    var getServiceView = function(configuration, runnables) {
        var view_model = utils.getBaseViewModel(configuration, runnables)
        return {
            'uuid': view_model.uuid,
            'enabled_text': $scope.getEntityStartupText(configuration),
            'details': getServiceDetails(configuration),
            'config': configuration,
            'runnable': view_model.runnable,
            'is_delete_blocked': utils.isDeleteBlocked(
                configuration, 'service'),
            'just_created': view_model.just_created
        }
    }

    /*
    Return the details to be visible in the UI for each service type.
    */
    var getServiceDetails = function(configuration) {

        var result = [{
            'key': 'Type',
            'value': componentTypeFilter(
                configuration.type, utils.constants.TYPE_NAME.SERVICE),
        }]

        if ($scope.hasInterface(configuration)) {
            result.push({'key': 'Port', 'value': configuration.port})
            result.push({
                'key': 'Listening',
                'value': utils.getIPAddressDescription(configuration.address)
            })
            result.push({
                'key': 'Timeout',
                'value': configuration.idle_connection_timeout
            })
        }

        var get_details = details_getter[configuration.type]
        if (get_details) {
            Array.prototype.push.apply(
                result, get_details(configuration.configuration))
        }

        return result
    }

    details_getter.ftp = function(configuration) {
        var result = [
            {
                'key': 'FTPS Secure',
                'value': (
                    configuration.ftps_explicit_enabled ? 'Enabled' : 'Disabled'
                )
            },
            {
                'key': 'Passive address',
                'value': configuration.passive_address
            },
            {
                'key': 'Passive range',
                'value': configuration.passive_port_range.join(' - ')
            },
            {
                'key': 'Data timeout',
                'value': configuration.idle_data_connection_timeout + 's'
            }
        ]
        if (configuration.ssl_domains) {
            result.push({
                'key': 'Let\'s Encrypt',
                'value': configuration.ssl_domains.join(', ')
            })
        }
        return result
    }

    details_getter.ftpsi = function(configuration) {
        var result =  details_getter.ftp(configuration)
        result[0]['value'] = 'Yes'
        return result
    }

    details_getter.ssh = function(configuration) {
        return [
            {
                'key': 'SFTP',
                'value': configuration.sftp ? 'Enabled' : 'Disabled'
            },
            {
                'key': 'SCP',
                'value': configuration.scp ? 'Enabled' : 'Disabled'
            }
        ]
    }

    details_getter.http = function(configuration) {
        return [
            {
                'key': 'AS2 Enabled',
                'value': configuration.as2_receive_name
            }
        ]
    }

    details_getter.https = function(configuration) {

        var result = []
        if (configuration.ssl_domains) {
            result.push({
                'key': 'Let\'s Encrypt',
                'value': configuration.ssl_domains.join(', ')
            })
        }

        result.push({
                'key': 'AS2 Enabled',
                'value': configuration.as2_receive_name
        })
        return result
    }

    details_getter.manager = function(configuration) {

        var result = []
        if (configuration.ssl_domains) {
            result.push({
                'key': 'Let\'s Encrypt',
                'value': configuration.ssl_domains.join(', ')
            })
        }

        return result
    }
    details_getter._manager_unsecured = details_getter.manager

    details_getter['http-redirect'] = function(configuration) {
        return [
            {
                'key': 'URL',
                'value': configuration.redirect_url
            }
        ]
    }

    $scope.getServiceStatusClass = function(service) {
        var status = $scope.$parent.getEntityStateText(service)
        var css_class = {
            'New - Enabled': 'badge-warning',
            'New - Disabled': '',
            'Enabled': 'badge-success',
            'Disabled': 'badge-inverse'
            }
        return css_class[status]
    }

    $scope._getDefaultConfiguration = function(protocol) {
        var result = {}

        result['idle_connection_timeout'] = 300

        if (['ftp', 'ftpsi', 'https', 'manager'].contains(protocol)) {
            // Set default SSL values.
            result['ssl_certificate'] = ''
            result['ssl_cipher_list'] = 'secure'
        }

        if (['ftp', 'ftpsi'].contains(protocol)) {
            // FIXME:4553:
            // Fix bad default value for passive_port_range.
            result['passive_port_range'] = [0, 0]
            result['idle_data_connection_timeout'] = 30
            result['ftps_require_session_reuse'] = true
        } else if (protocol == 'ssh') {
            result['ssh_cipher_list'] = ['secure']
            result['dh_prime_size'] = ['1024', 'ideal']

        } else if (['http', 'https'].contains(protocol)) {
            result['public_name'] = 'public'
            result['public_account_uuid'] = ''
            result['as2_receive_name'] = 'as2receive'
            result['languages'] = ['en']
            result['login_footnotes'] = ''
            result['ui_version'] = utils.constants.TYPE_NAME.UI_GEN_2
        } else if (protocol == 'lets-encrypt') {
            result['acme_url'] = (
                utils.constants.CONFIGURATION_DEFAULTS.acme_url)
            result['contact_email'] = ''
            result['domains'] = [[
                'files.example.com', '1c17-44-878c', '17c9-7a-2f35']]
        }

        return result
    }

    /*
    Called when add service button is triggered.
    */
    $scope.onServiceAdd = function(protocol) {
        var service = {}
        service['name'] = 'New %(protocol)s Service %(random)s'.supplant({
            'protocol': protocol,
            'random': utils.getRandomMark()
            })
        service['type'] = protocol
        service['address'] = '0.0.0.0'
        service['port'] = 12345
        service['enabled'] = true
        service['configuration'] = $scope._getDefaultConfiguration(protocol)

        category.add(service)
        category.goToConfiguration(service)
    }

    /*
    Go to configuration page.
    */
    $scope.onGoToConfiguration = function(service) {
        category.goToConfiguration(service)
    }

    /*
    Delete service.
    */
    $scope.onDeleteService = function(service) {
        category.delete(service)
        $scope.onFilterServices($scope.filter_input)
    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.servicesCtrl = {} : exports))
