(function(exports){

exports.statusCtrl = function($scope, $window, utils) {

    $scope.active_services = []
    $scope.handlers = []
    $scope.STATE = utils.constants['STATE']
    $scope.failed_components = []
    $scope.$parent.component = {
        'name': 'Dashboard',
        'summary': 'Overall information about the state of the application.',
    }
    /*
    Watch product key configuration to update the available UI elements.
    */
    $scope.$watch(
      'configuration.server.product_key',
      function(new_value, old_value) {
        _updateLicenceType(new_value)
    })

    /*
    Watch process runnables to monitor failed components.
    */
    $scope.$watch('process', function(new_value, old_value) {
        $scope.failed_components.length = 0;
        $scope.failed_components.extend(getFailedComponents())
      },
      true)


    $scope.__init__ = function() {
        show_windows_legacy_service_warning()
    }

    /*
    Handled the display of a warning in the case SFPTplus runs as the
    legacy windows service.
    */
    var show_windows_legacy_service_warning = function() {
        if (!$scope.$parent.process.runnables) {
            // Backend data not loaded yet.
            return
        }
        if (!$scope.$parent.process.runnables.status) {
            // Backend data not loaded yet.
            return
        }
        if (!$scope.$parent.process.runnables.status.windows_legacy_service) {
            return
        }
        if ($window.sessionStorage.getItem(
                'windows_legacy_service_warning') == 'done') {
            // Only show the alert once per session.
            return
        }

        $scope.$parent.alerts.push({
            'type': 'warning',
            'msg': (
                'The SFTPPlus Windows service was started using the ' +
                'legacy manager. ' +
                'You should upgrade the Windows service. ' +
                'For now you can continue to use SFTPPlus. ' +
                'Upgrading the Windows service will be required when ' +
                'updating to any newer version of SFTPPlus. ' +
                'Get in touch with our support team to help you with the ' +
                'upgrade.'
            )
        })
        $window.sessionStorage.setItem('windows_legacy_service_warning', 'done')
    }

    /*
    Return a list with the view-model for all the failed components.
    */
    var getFailedComponents = function() {
        var result = []

        /*
        Return the information needed for the UI for one failed component.
        */
        var get_view = function(configuration, runnables) {
            var result = utils.getBaseViewModel(configuration, runnables)
            return result
        }

        /*
        Return all the UI info for all failed components from `family`.
        */
        var get_components = function(family) {
            angular.forEach(
                $scope.process.runnables[family],
                function(runnable) {
                    if (!runnable.last_error) {
                        return
                    }
                    var family_config = $scope.configuration[family]
                    if (!family_config) {
                        // Maybe the configuration is still loading.
                        return
                    }
                    var config = family_config[runnable.uuid]
                    if (!config) {
                        // Maybe the configuration is pending removal.
                        return
                    }
                    result.push(
                        get_view(config, $scope.process.runnables[family]))
                })
        }

        get_components('services')
        get_components('transfers')
        get_components('locations')
        get_components('authentications')
        get_components('event_handlers')
        get_components('resources')
        return result
    }

    $scope.isServiceStoppable = function(service_uuid) {
        if (service_uuid === utils.constants['DEFAULT_LOCAL_MANAGER_UUID']) {
            return false
        } else {
            return true
        }
    }

    /*
    Return configured interface for `service_uuid`.
    */
    $scope.getServiceInterface = function(service_uuid) {
        var configuration = $scope.getServiceStartupConfiguration(
            service_uuid)
        if (!$scope.hasInterface(configuration)) {
            return ''
        }
        return configuration['address'] + ':' + configuration['port']
    }

    $scope.getServiceInterface = function(service_uuid) {
        var configuration = $scope.getServiceStartupConfiguration(
            service_uuid)
        if (!$scope.hasInterface(configuration)) {
            return ''
        }
        return configuration['address'] + ':' + configuration['port']
    }

    /*
    Return latest configuration object for `service_uuid`.
    */
    $scope.getServiceConfiguration = function(service_uuid) {
        if (!$scope.$parent.configuration.services) {
            return {}
        }
        return $scope.$parent.configuration.services[service_uuid]
    }

    /*
    Return configuration object for `service_uuid` as last defined on server.
    */
    $scope.getServiceUpstreamConfiguration = function(service_uuid) {
        if (!$scope.$parent.upstream_configuration.services) {
            return {}
        }
        return $scope.$parent.upstream_configuration.services[service_uuid]
    }

    /*
    Return configuration at service startup for `service_uuid`.
    */
    $scope.getServiceStartupConfiguration = function(service_uuid) {
        var service = $scope.$parent.process.runnables.services[service_uuid]
        if (service && service.startup_configuration) {
            return service.startup_configuration
        }
        // No explicit startup configuration try to use upstream.
        if (!$scope.$parent.upstream_configuration.services) {
            // No upstream configuration yet.
            return {}
        }

        return $scope.$parent.upstream_configuration.services[service_uuid]
    }

    /*
    Called when user selects a specific license type.
    */
    $scope.onSetProductKey = function(usage_type) {
        // Temporarily set a product key value other than `ask` so that the
        // UI will hide the product usage dialog.
        $scope.upstream_configuration.server.product_key = 'updating'
        var change = {
            'operation': 'update',
            'path': 'server/product_key',
            'value': usage_type
        }
        $scope.applyChanges([change])
        _updateLicenceType(usage_type)
    }

    /*
    Update the UI view model related to product key type.
    */
    var _updateLicenceType = function(product_key) {
        // Start by enabling all.
        $scope.have_server_licence = true
        $scope.have_client_licence = true

        // Then disable based on product key type.
        if (product_key == 'server-only') {
            $scope.have_client_licence = false
        }
        if (product_key == 'client-only') {
            $scope.have_server_licence = false
        }
    }
}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.statusCtrl = {} : exports))
