/*
Tests for transfer edit page controller.
*/
describe('transferCtrl', function() {


    /*
    It goes to the start page when there are no transfer, as this is a sign
    that the configuration was not yet loaded, and there is no
    point in searching for the transfer with the UUID from the URL.
    */
    test('init_no_transfers', function() {
        var sut = mk.getController('transferCtrl')

        sut.scope.$parent.configuration.transfers = {}
        sut.scope.$parent.new_entities.transfers = []
        sut.route.uuid = 'some-new-admin'

        sut.scope.__init__()

        expect(sut.$location.path()).toEqual('/')

    })

    /*
    It goes to the transfers list page when we have transfer, but the one
    from the URL is not found.
    */
    test('init_transfer_not_found', function() {
        var sut = mk.getController('transferCtrl')

        sut.scope.$parent.configuration.transfers = {}
        sut.scope.$parent.new_entities.transfers = [{
            'name': 'new-transfer'
        }]
        sut.route.uuid = 'unknown-uuid'

        sut.scope.__init__()

        expect(sut.$location.path()).toEqual('/transfers')

    })

    /*
    It load the transfer page for a new transfers based on the name extracted
    from the URL.
    */
    test('init_transfer_new', function() {
        var sut = mk.getController('transferCtrl')

        sut.scope.$parent.configuration.transfers = {}
        sut.scope.$parent.new_entities.transfers = [{
            'name': 'new-transfer'
        }]
        sut.route.uuid = 'new-transfer'

        sut.scope.__init__()

        expect(sut.scope.runnable).toBeFalse()

    })

    /*
    It load the transfer page for an existing transfer based on the
    uuid extracted from the URL.
    */
    test('init_transfer_existing', function() {
        var sut = mk.getController('transferCtrl')

        sut.scope.$parent.configuration.transfers = {'some-uuid': {
            'name': 'existing transfer',
            'uuid': 'some-uuid',
        }}
        var runnable = {
            'name': 'existing transfer',
            'state': 'stopped',
        }
        sut.scope.$parent.new_entities.transfers = [{
            'name': 'new-transfer'
        }]
        sut.scope.$parent.process.runnables.transfers = {'some-uuid': runnable}
        sut.route.uuid = 'some-uuid'

        sut.scope.__init__()

        expect(sut.scope.runnable).toBe(runnable)

    })

    /*
    It can delete a transfer that was just created and not yet saved.
    */
    test('onDeleteTransfer_new', function() {
        var sut = mk.getController('transferCtrl')

        sut.scope.$parent.configuration.transfers = {
            'some-old-uuid': {'name': 'transfer in config'}}
        sut.scope.$parent.new_entities.transfers = [{
            'name': 'some-new-transfer'
        }]

        sut.route.uuid = 'some-new-transfer'
        sut.scope.__init__()

        sut.scope.onDeleteTransfer()

        // The new transfer is removed.
        expect(sut.scope.$parent.new_entities.transfers).toHaveLength(0)
        // Existing transfer is left.
        expect(sut.scope.$parent.configuration.transfers).toContainKey(
            'some-old-uuid')
    })

    /*
    For a transfer that is found in the config, it will remove it from the
    configuration object.
    */
    test('onDeleteTransfer_existing', function() {
        var sut = mk.getController('transferCtrl')

        sut.scope.$parent.configuration.transfers = {
            'some-old-uuid': {'name': 'old transfer', 'uuid': 'some-old-uuid'}}
        sut.scope.$parent.process.runnables.transfers = {
            'some-old-uuid': {
                'name': 'old transfer',
                'uuid': 'some-old-uuid',
                'state': 'started',
            }}
        sut.scope.$parent.new_entities.transfers = [{
            'name': 'some-new-transfer'
        }]
        // Transfer UUID is extracted from the URL route info.
        sut.route.uuid = 'some-old-uuid'
        sut.scope.__init__()

        sut.scope.onDeleteTransfer()

        // The new transfer is kept.
        expect(sut.scope.$parent.new_entities.transfers).toHaveLength(1)
        // Existing transfer is removed.
        expect(sut.scope.$parent.configuration.transfers).not.toContainKey(
            'some-old-uuid')
    })


})
