(function(exports){

exports.transfersCtrl = function($scope, utils, $timeout) {

    var init = require('../../../js_commons/__init__.js', 'init')
    // The current list of filtered and sorted transfers
    // Will be initialized later.
    $scope.filtered_transfers = []
    // Used by the search input tag.
    $scope.filter_input = ''

    $scope.$parent.component = {
        'name': 'Transfers',
        'summary': 'Transfer files to and from remote servers.',
    }

    var category = $scope.$parent.component_categories.transfers

    /*
    This is called each time the page is accessed,
    including when using the browser back/forward.
    */
    $scope.__init__ = function() {
        // Trigger an initial filter that will show all transfers.
        $scope.onFilterTransfers()

        // The configuration might not be loaded so we force a dumb reload
        // We should find a better way to auto-reload the page but for
        // now it should be ok.
        if (init.isEmpty($scope.filtered_transfers)) {
            $timeout(function() {$scope.onFilterTransfers()}, 3000)
        }
    }

    /*
    Called when add transfer button is triggered.
    */
    $scope.onAddTransfer = function() {
        var transfer = {}
        var location_uuid = utils.constants['DEFAULT_LOCAL_FILESYSTEM_UUID']
        transfer['enabled'] = true
        transfer['recursive'] = false
        transfer['delete_source_on_success'] = true
        transfer['source_uuid'] = location_uuid
        transfer['source_path'] = ''
        transfer['destination_uuid'] = location_uuid
        transfer['destination_path'] = ''
        transfer['name'] = 'New transfer %(random)s'.supplant({
            'random': utils.getRandomMark()
            })
        transfer['description'] = ''
        transfer['retry_count'] = 2
        transfer['execute_timeout'] = 30
        transfer['changes_poll_interval'] = 300
        transfer['retry_wait'] = 300
        transfer['stable_interval'] = 360
        transfer['minimum_transfer_count'] = 0
        transfer['minimum_transfer_interval'] = 86400

        category.add(transfer)
        category.goToConfiguration(transfer)
    }

    $scope.onGoToConfiguration = function(transfer) {
        category.goToConfiguration(transfer)
    }

    $scope.onDeleteTransfer = function(transfer) {
        category.delete(transfer)
        $scope.onFilterTransfers($scope.filter_input)
    }

    /*
    Called the update the list of transfers to be displayed on the page.
    */
    $scope.onFilterTransfers = function(filter_value) {
        $scope.filtered_transfers = Array()
        angular.forEach(
            $scope.getAllTransfers(),
            function(transfer) {
                addFilteredTransferView(
                    filter_value, transfer, $scope.filtered_transfers)
            })
    }

    /*
    Update the `transfers` based on the current filter.
    */
    var addFilteredTransferView = function(
            filter_value, configuration, transfers) {

        var view_model = utils.getBaseViewModel(
            configuration, $scope.process.runnables.transfers)
        var source = $scope.configuration.locations[
            configuration.source_uuid]
        var destination = $scope.configuration.locations[
            configuration.destination_uuid]

        if (isFilteredOut(filter_value, configuration, source, destination)) {
            return
        }

        transfers.push({
            'uuid': view_model.uuid,
            'config': configuration,
            'runnable': view_model.runnable,
            'source': source,
            'destination': destination,
            'enabled_text': $scope.getEntityStartupText(configuration),
            'just_created': view_model.just_created
        })
    }

    /*
    Return true if the entry should by filtered out.
    That is not included in the result.
    */
    var isFilteredOut = function(
            filter_value, configuration, source, destination) {
        if (!filter_value) {
            return false
        }

        if (utils.containsInsensitive(
                configuration.description, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(configuration.name, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(
                configuration.source_path, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(
                configuration.destination_path, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(source.name, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(destination.name, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(source.type, filter_value)) {
            return false
        }

        if (utils.containsInsensitive(destination.type, filter_value)) {
            return false
        }

        return true
    }

}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.transfersCtrl = {} : exports))
