/*
Tests for transfers controller.
*/


describe('TransfersCtrl', function() {

    var scope
    var utils
    var ctrl
    var result
    var $flushPendingTasks
    var $location

    require('../app.js')
    beforeEach(angular.mock.module('app'))
    beforeEach(inject(function($injector, $rootScope, $controller, $route) {
        /*
        Prepare the scope and controller before each test.
        */
        utils = mk.servicesUtils($injector)

        // FIXME:3932:
        // See how to test without removing  the default route.
        delete $route.routes[null]
        $flushPendingTasks = $injector.get('$flushPendingTasks')
        $location = $injector.get('$location')

        var parent_scope = $rootScope.$new();
        ctrl = $controller(
            'mainCtrl', {
            $scope: parent_scope,
            utils: utils
        })

        scope = parent_scope.$new();
        ctrl = $controller(
            'transfersCtrl', {
            $scope: scope,
            utils: utils
            })
    }))

    /*
    Will reload the table if at first no transfers are found
    since the configuration might not be yet ready.
    */
    test('__init__no_transfers', function() {
        scope.__init__()

        // No transfers loaded at start.
        expect(scope.filtered_transfers).toEqual([])

        // Meanwhile the configuration is updated.
        scope.$parent.configuration['transfers'] = {
            'transfer-uuid': {
                'name': 'transfer-name'
            }
        }
        scope.$parent.configuration['locations'] = {
            'DEFAULT-LOCAL-FILESYSTEM': {
                'uuid': 'DEFAULT-LOCAL-FILESYSTEM',
                'name': 'Local Files'
            }
        }

        $flushPendingTasks(3000)
        expect(scope.filtered_transfers.length).toBe(1)
        expect(scope.filtered_transfers[0].config.name).toEqual(
            'transfer-name')
    })

    test(
    'init will include all the transfer both new and existing on the ' +
    'server. The filter value changes will automatically update.',
    function() {

        // Meanwhile the configuration is updated.
        scope.$parent.configuration['transfers'] = {
            'transfer1-uuid': {
                'name': 'existing-name',
                'source_uuid': 'DEFAULT-LOCAL-FILESYSTEM',
                'destination_uuid': 'DEFAULT-LOCAL-FILESYSTEM'
            },
            'transfer2-uuid': {
                'name': 'Another Transfer',
                'source_uuid': 'DEFAULT-LOCAL-FILESYSTEM',
                'destination_uuid': 'DEFAULT-LOCAL-FILESYSTEM'
            }
        }
        scope.$parent.configuration['locations'] = {
            'DEFAULT-LOCAL-FILESYSTEM': {
                'uuid': 'DEFAULT-LOCAL-FILESYSTEM',
                'name': 'Local Files'
            }
        }
        scope.$parent.new_entities['transfers'] = [
            {'name': 'best new transfer'}
            ]

        // Page is loaded with existing configuration.
        scope.__init__()

        // Transfers are updated.
        expect(scope.filtered_transfers.length).toBe(3)
        expect(scope.filtered_transfers[0].config.name).toEqual(
            'Another Transfer')
        expect(scope.filtered_transfers[1].config.name).toEqual(
            'best new transfer')
        expect(scope.filtered_transfers[2].config.name).toEqual(
            'existing-name')

        // The filter is called by a separate directive.
        // For this unit test, we call it directly.
        scope.onFilterTransfers('tra')
        expect(scope.filtered_transfers.length).toBe(2)
        expect(scope.filtered_transfers[0].config.name).toEqual(
            'Another Transfer')
        expect(scope.filtered_transfers[1].config.name).toEqual(
            'best new transfer')
    })


    test(
    'onAddTransfer will add al new transfer in the cache and then go ' +
    'to its page.',
    function() {
        var new_transfers = scope.$parent.new_entities['transfers']

        scope.onAddTransfer()

        expect(new_transfers.length).toBe(1)
        expect(new_transfers[0]['enabled']).toBeTrue()
        expect(new_transfers[0]['uuid']).toBeUndefined()
        expect(new_transfers[0]['name']).toStartWith('New transfer ')
        expect($location.path()).toStartWith(
            '/transfers/New transfer ')
        expect(new_transfers[0]['recursive']).toBeFalse()
        expect(new_transfers[0]['delete_source_on_success']).toBeTrue()
        expect(new_transfers[0]['source_uuid']).toEqual(
            'DEFAULT-LOCAL-FILESYSTEM')
        expect(new_transfers[0]['destination_uuid']).toEqual(
            'DEFAULT-LOCAL-FILESYSTEM')
        expect(new_transfers[0]['description']).toEqual('')
        expect(new_transfers[0]['source_path']).toEqual('')
        expect(new_transfers[0]['destination_path']).toEqual('')
    })

    test(
    'onDeleteTransfer will remove the newly created transfer.',
    function() {
        var location = {
            'uuid': undefined,
            'name': mk.string(),
            'type': 'sftp'
            }
        scope.$parent.new_entities['transfers'].push(location)

        scope.onDeleteTransfer(location)

        expect(scope.$parent.new_entities['transfers'].length).toBe(0)
    })

    test(
    'onDeleteTransfer will remove an existing transfer from configuration.',
    function() {
        var uuid = mk.string()
        var transfer = {
            'uuid': uuid,
            'name': 'existing name',
            'type': 'copy'
            }
        scope.$parent.configuration = {'transfers': {}}
        scope.$parent.configuration['transfers'][uuid] = transfer

        scope.onDeleteTransfer(transfer)

        expect(scope.$parent.configuration['transfers']).toBeEmpty()
    })

})
