/*
Shared controller code for ng-table.

Users need to define:
* __init__
* _updateTable
* _updateEntryData

*/
(function(exports){

exports.Mixins = function() {
  var exports = {}

  exports.NGTableMixin = function($scope) {

    $scope.in_progress = true

    $scope.entries = []
    $scope.handlers = []

    // Event for which to show details.
    $scope.event = null

    $scope.typeaheads = {}

    $scope.table_options = {
        // There are used to interact with the DB.
        page: 1,  // We count from 1, but to DB we sent to 0.
        limit: 50,
        sorting: {
            timestamp: 'desc'
            },
        filter: []
        }
    $scope.table_ui = {
        total: 'many',
        range_start: 1,
        range_end: 50,
        have_previous: false,
        have_next: true,
        limits: [25, 50, 100, 500, 5000]
    }

    $scope.date_start_ui = {
        // Widget options
        startingDay: 1,
        maxDate: Date.now(),
        showWeeks: false,
        // Widget state.
        opened: false,
        // Widget model.
        // default to no date selected.
        date_value: null,
        time_value: null
    }

    $scope.date_end_ui = {
        // Widget options
        startingDay: 1,
        maxDate: Date.now(),
        showWeeks: false,
        // Widget state.
        opened: false,
        // Widget model.
        // default to no date selected.
        date_value: null,
        time_value: null
    }

    /*
    When start date is selected, we want to automatically select the end
    of the day, if no previous value is set.
    */
    $scope.$watch('date_start_ui.date_value', function(date) {
        if (!date) {
            // No new date.
            return
        }

        if ($scope.date_start_ui.time_value) {
            // We already have a time.
            return
        }

        var start_time = new Date(date)
        start_time.setHours(0, 0, 0)
        $scope.date_start_ui.time_value = start_time
    })

    /*
    When end date is selected, we want to automatically select the end
    of the day, if no previous value is set.
    */
    $scope.$watch('date_end_ui.date_value', function(date) {
        if (!date) {
            // No new date.
            return
        }

        if ($scope.date_end_ui.time_value) {
            // We already have a time.
            return
        }

        var end_time = new Date(date)
        end_time.setHours(23, 59, 59)
        $scope.date_end_ui.time_value = end_time
    })


    /*
    Load the content for `page` number.
    This is used as part of the table pagination next/previous.
    */
    $scope.onGoToPage = function(page) {
        if (page < 1) {
            var page = 1
        }
        $scope.table_options.page = page
        $scope._updateTable()

    }
    /*
    Reload table data on value change.
    */
    $scope._updateTableOnChange = function(new_value, old_value) {
        if (angular.equals(new_value, old_value)) {
            // Nothing is changed at initialization.
            return
        }

        $scope.onGoToPage(0)
    }

    /*
    Reload table data on when a new page was requested.
    */
    $scope._updateTableOnPage = function(new_value, old_value) {
        if (angular.equals(new_value, old_value)) {
            // Nothing is changed at initialization.
            return
        }
        $scope._updateTable()
    }

    $scope.$watch('table_options.limit', $scope._updateTableOnChange)
    $scope.$watch('table_options.sorting', $scope._updateTableOnChange)
    $scope.$watch('table_options.page', $scope._updateTableOnPage)

    /*
    Watch users configuration and refresh on change.
    */
    $scope.$watch('configuration.accounts', function(new_value, old_value) {
        if (new_value == old_value) {
            // Nothing is changed at initialization.
            return
        }

        $scope._updateIdentity()
    })

    /*
    Update UI depending on identity configuration.
    */
    $scope._updateIdentity = function(attribute_name) {
        var result = $scope._getAllAccountsAndAdministratorsNames()
        result.push.apply(result, $scope._getAllLocationNames())
        result.push.apply(result, $scope._getAllTransferNames())
        $scope.typeaheads[attribute_name] = result
    }

    /*
    Update UI depending on services configuration.
    */
    $scope._updateServices = function(attribute_name) {
        $scope.typeaheads[attribute_name] = (
            $scope._getAllServiceNames())
    }


    $scope._ebTableUpdate = function(data, status) {
        $scope.in_progress = false
        $scope.alertError(data['error'])
    }

    $scope._ebFailure = function(response) {
        var data = response.data
        var status = resource.status
        $scope.in_progress = false

        error = 'Server error (' + status.toString() + ') ' + data.toString()
        $scope.alerts.push({'type': 'error', 'msg': error})
    }

    /*
    Called when we got a response from the web service and we should extract
    the result from the response.

    It will delegate the data filtering and sanitization to _updateEntryData.
    */
    $scope._cbTableUpdate = function(data, callback) {
        $scope.in_progress = false
        $scope.table_ui.total = 'many'

        if (!data) {
            // Maybe a connection error.
            // Nothing to update on the UI.
            return
        }

        var page_size = data.result.data.length
        var limit_reached = page_size < $scope.table_options.limit

        // Convert from JSON-RPC data to view-model data.
        $scope.entries = []
        angular.forEach(
            data['result']['data'],
            function(data) {
              $scope.entries.push(data)
              $scope._updateEntryData(data)
            })

        if (page_size == 0 && $scope.table_options.page > 1) {
            // We reach the last page, as we are not on the first page
            // and no results were returned.
            $scope.entries = $scope._previous_entries
            $scope.table_options.page -= 1
            page_size = $scope.entries.length
            limit_reached = true
        }

        var page_index = $scope.table_options.page - 1
        var start = page_index * $scope.table_options.limit


        if (page_size == 0 && $scope.table_options.page == 1) {
            // No results... so we start and end at zero.
            $scope.table_ui.range_start = 1
        } else {
            // When we have results, we start counting with 1.
            $scope.table_ui.range_start = start + 1
        }

        $scope.table_ui.range_end = start + $scope.table_options.limit

        $scope.table_ui.have_previous = $scope.table_options.page > 1
        $scope.table_ui.have_next = true

        if (limit_reached) {
            var total = start + page_size
            $scope.table_ui.total = total
            $scope.table_ui.range_end = total
            $scope.table_ui.have_next = false
        }

        if (callback) {
            callback()
        }
    }

    $scope._dbFetch = function(utils, callback) {

        var params = {
            'db_uuid': $scope.table_options.db_uuid,
            // For the UI page starts at 1.
            // For DB, page starts at 0.
            'page': $scope.table_options.page - 1,
            'limit': $scope.table_options.limit,
            'criteria': $scope.table_options.criteria,
            'sorting': $scope.table_options.sorting
            }

        $scope._previous_entries = $scope.entries
        utils.jsonrpc({
            method: 'db_fetch',
            params: params,
            failure: $scope._ebFailure,
            success: function(data) { $scope._cbTableUpdate(data, callback) },
            error: $scope._ebTableUpdate
        })
    }

  }
  return exports
}

/* c8 ignore next */
}(typeof exports === 'undefined' ? this.ng_table_mixin = {} : exports))
