/*
Service providing helper functions.

All bad design code is collected here. :)

This is also the place where all browser specific dependencies are gathered,
so that we can mock them from a simple place in tests.
*/

(function(exports){

var Papa = require('@chevah/papaparse/papaparse', 'Papa')
var init = require('../../../js_commons/__init__.js', 'init')

exports.Utils = function(
  $http,
  $log,
  $cookies,
  $window,
  CACHE_BUSTER
) {

  var exports = {}

  // Store all dependencies as private members to simplify tests.
  // In the future this should be removed and the tests should get the $http
  // service via the injector.
  exports._http = $http

  // Store constants here so that it will be much easier to handle tests.
  exports.constants = CONSTANTS

  exports.getRandomMark = function(prefix) {
    var base = Math.floor((1 + Math.random()) * 0x10000)
               .toString(16)
               .substring(1)
    if (! prefix) {
      prefix = ''
    }
    return prefix + base
  }

  exports.getGUIDUnsafe = function() {
    return exports.getRandomMark() + exports.getRandomMark() + '-' +
      exports.getRandomMark() + '-' +
      exports.getRandomMark() + '-' +
      exports.getRandomMark() + '-' +
      exports.getRandomMark() + exports.getRandomMark() +
        exports.getRandomMark()
  }

  /*
  Navigate the browser's windows to the specified url.

  This is to be used for navigation outside of the AngjularJS app.
  For example when moving betwen the login app and the main app.
  */
  exports.goToURL = function(url) {
    $window.location.href = url
  }

  // Return true if cookies are enabled.
  exports.areCookiesEnabled = function() {
    var cookie_name = 'chevah_test_cookie'
    var cookie_value = exports.getGUIDUnsafe()

    // We use low-level call to check if cookies are enabled.
    // as the high-level $cookies are cached and are not synchronous.
    $cookies.put(cookie_name, cookie_value)
    if ($cookies.get(cookie_name) === cookie_value) {
        //Delete the test cookie.
        $cookies.remove(cookie_name)
        return true
    }
    return false
  }

  /*
  Call the JSON RPC `method` with `data`.

  The authentication is done via the session cookie.

  args = {
    'method': method_name,
    'params': method_params,
    'success': success_json_callback,
    'error': error_json_callback,
    'failure': failure_http_callback,
    'alerts': the array where to store default errors when no
              custom error or failure callbacks are provided.
    }
  */
  exports.jsonrpc = function(args) {

      var data = {
          'jsonrpc': '2.0',
          'method': args.method
          }

      var handleResponse = function(response) {
        var data = response.data
        var status = response.status
        if (data['error']) {
          return args.error(data, status)
        }
        args.success(data, status)
      }

      /*
      This is monster function that is used by default for both error
      handled as well as failure handling.
      */
      var logError = function(data, status) {

        // This can be called as either a `promise` failure and then
        // `data` is a `promise.response`.
        // It can also be called directly by our internal error handling
        /// and then the data and status are already extracted.
        if (!status) {
          // We might have a response as input argument.
          var status = data.status
          if (status == -1) {
            // This is a low-level HTTP ajax error.
            data = data.statusText
          } else {
            data = data.data
          }
        }

        var code
        var message
        if (data['error']) {
          // We have a normal JSON rpc error.
          code = '' + data['error']['code']
          message = '' + data['error']['message']
        } else {
          // This is low level HTTP error.
          code = '' + status  // Convert from number to text.
          if (data) {
            message = data
          } else {
            message = 'No response received from the server.'
          }
        }

        if (args['alerts']) {
          args['alerts'].push({
            'type': 'error',
            'msg': '%(status)s: %(message)s'.supplant({
              'status': code,
              'message': message
            })
          })
        } else {
            $log.log('Got response: ' + code)
            $log.log(message)
        }
      }

      if (args.params === undefined) {
          args.params = {}
      }
      data.params = args.params

      if (args.success === undefined) {
          args.success = function(data) {}
      } else {
          data.id = 1
      }

      if (args.error === undefined) {
          args.error = logError
      }

      if (args.failure === undefined) {
          args.failure = logError
      }

      exports._http({
        method: 'POST',
        url: exports.constants.BASE_URL_PATH + '/json',
        data: angular.toJson(data)
        }).then(handleResponse, args.failure)
  }

  /*
  Try to parse value as boolean, but return the exact value if it is not
  a boolean.
  */
  exports.maybeParseBoolean = function(value) {
    if (!angular.isString(value)) {
      return value
    }

    if (['yes', 'true'].contains(value.toLowerCase())) {
      return true
    } else if (['no', 'false'].contains(value.toLowerCase())) {
      return false
    } else {
      return value
    }
  }

  /*
  Convert a list of lines into single string text.
  */
  exports.multiLinesToText = function(multi_sequence) {
      if (!multi_sequence) {
        return ''
      }
      return multi_sequence.join('\n')
  }

  /*
  Convert a single text into array of lines.
  */
  exports.textToMultiLines = function(text) {
      if (!text) {
        return []
      }
      return text.split(/\r?\n/g)
  }


  /*
  Convert a list of lists into a comma separated multi line text.
  */
  exports.multiSequenceToText = function(multi_sequence) {
      if (!multi_sequence) {
        return ''
      }
      var result = Papa.unparse(
          multi_sequence,
          {
            newline: '\n',
            delimiter: ', '
          })
      return result
  }


  /*
  Convert a multi line comma separate lines into a list of lists.
  */
  exports.textToMultiSequence = function(raw_value) {
      var new_values = []

      var transform_value = function(value) {
        var result = value.trim()
        result = result.replace(/[\r\n]+/g, ' ')
        return result
      }

      var result = Papa.parse(
          raw_value,
          {
          delimiter: ',',
          skipEmptyLines: true,
          transform: transform_value
          })

      return result.data
  }

  exports.countries = [
      {'name': 'Afghanistan', 'x509_code': 'AF'},
      {'name': 'Albania', 'x509_code': 'AL'},
      {'name': 'Algeria', 'x509_code': 'DZ'},
      {'name': 'American Samoa', 'x509_code': 'AS'},
      {'name': 'Andorra', 'x509_code': 'AD'},
      {'name': 'Angola', 'x509_code': 'AO'},
      {'name': 'Anguilla', 'x509_code': 'AI'},
      {'name': 'Antarctica', 'x509_code': 'AQ'},
      {'name': 'Antigua and Barbuda', 'x509_code': 'AG'},
      {'name': 'Argentina', 'x509_code': 'AR'},
      {'name': 'Armenia', 'x509_code': 'AM'},
      {'name': 'Aruba', 'x509_code': 'AW'},
      {'name': 'Australia', 'x509_code': 'AU'},
      {'name': 'Austria', 'x509_code': 'AT'},
      {'name': 'Azerbaijan', 'x509_code': 'AZ'},
      {'name': 'Åland Islands', 'x509_code': 'AX'},
      {'name': 'Bahamas', 'x509_code': 'BS'},
      {'name': 'Bahrain', 'x509_code': 'BH'},
      {'name': 'Bangladesh', 'x509_code': 'BD'},
      {'name': 'Barbados', 'x509_code': 'BB'},
      {'name': 'Belgium', 'x509_code': 'BE'},
      {'name': 'Belize', 'x509_code': 'BZ'},
      {'name': 'Benin', 'x509_code': 'BJ'},
      {'name': 'Bermuda', 'x509_code': 'BM'},
      {'name': 'Bhutan', 'x509_code': 'BT'},
      {'name': 'Bolivia', 'x509_code': 'BO'},
      {'name': 'Bosnia and Herzegovina', 'x509_code': 'BA'},
      {'name': 'Botswana', 'x509_code': 'BW'},
      {'name': 'Bouvet Island', 'x509_code': 'BV'},
      {'name': 'Brazil', 'x509_code': 'BR'},
      {'name': 'British Indian Ocean Territory', 'x509_code': 'IO'},
      {'name': 'Brunei Darussalam', 'x509_code': 'BN'},
      {'name': 'Bulgaria', 'x509_code': 'BG'},
      {'name': 'Burkina Faso', 'x509_code': 'BF'},
      {'name': 'Burundi', 'x509_code': 'BI'},
      {'name': 'Cambodia', 'x509_code': 'KH'},
      {'name': 'Cameroon', 'x509_code': 'CM'},
      {'name': 'Canada', 'x509_code': 'CA'},
      {'name': 'Cape Verde', 'x509_code': 'CV'},
      {'name': 'Cayman Islands', 'x509_code': 'KY'},
      {'name': 'Central African Republic', 'x509_code': 'CF'},
      {'name': 'Chad', 'x509_code': 'TD'},
      {'name': 'Chile', 'x509_code': 'CL'},
      {'name': 'China', 'x509_code': 'CN'},
      {'name': 'Christmas Island', 'x509_code': 'CX'},
      {'name': 'Cocos (Keeling) Islands', 'x509_code': 'CC'},
      {'name': 'Colombia', 'x509_code': 'CO'},
      {'name': 'Comoros', 'x509_code': 'KM'},
      {'name': 'Cook Islands', 'x509_code': 'CK'},
      {'name': 'Costa Rica', 'x509_code': 'CR'},
      {'name': 'Cote D\'Ivoire (Ivory Coast)', 'x509_code': 'CI'},
      {'name': 'Croatia (Hrvatska)', 'x509_code': 'HR'},
      {'name': 'Cyprus', 'x509_code': 'CY'},
      {'name': 'Czech Republic', 'x509_code': 'CZ'},
      {'name': 'Czechoslovakia (former)', 'x509_code': 'CS'},
      {'name': 'Denmark', 'x509_code': 'DK'},
      {'name': 'Djibouti', 'x509_code': 'DJ'},
      {'name': 'Dominica', 'x509_code': 'DM'},
      {'name': 'Dominican Republic', 'x509_code': 'DO'},
      {'name': 'East Timor', 'x509_code': 'TP'},
      {'name': 'Ecuador', 'x509_code': 'EC'},
      {'name': 'Egypt', 'x509_code': 'EG'},
      {'name': 'El Salvador', 'x509_code': 'SV'},
      {'name': 'Equatorial Guinea', 'x509_code': 'GQ'},
      {'name': 'Eritrea', 'x509_code': 'ER'},
      {'name': 'Estonia', 'x509_code': 'EE'},
      {'name': 'Ethiopia', 'x509_code': 'ET'},
      {'name': 'Falkland Islands (Malvinas)', 'x509_code': 'FK'},
      {'name': 'Faroe Islands', 'x509_code': 'FO'},
      {'name': 'Fiji', 'x509_code': 'FJ'},
      {'name': 'Finland', 'x509_code': 'FI'},
      {'name': 'France', 'x509_code': 'FR'},
      {'name': 'France, Metropolitan', 'x509_code': 'FX'},
      {'name': 'French Guiana', 'x509_code': 'GF'},
      {'name': 'French Polynesia', 'x509_code': 'PF'},
      {'name': 'French Southern Territories', 'x509_code': 'TF'},
      {'name': 'Gabon', 'x509_code': 'GA'},
      {'name': 'Gambia', 'x509_code': 'GM'},
      {'name': 'Georgia', 'x509_code': 'GE'},
      {'name': 'Germany', 'x509_code': 'DE'},
      {'name': 'Ghana', 'x509_code': 'GH'},
      {'name': 'Gibraltar', 'x509_code': 'GI'},
      {'name': 'Greece', 'x509_code': 'GR'},
      {'name': 'Greenland', 'x509_code': 'GL'},
      {'name': 'Grenada', 'x509_code': 'GD'},
      {'name': 'Guadeloupe', 'x509_code': 'GP'},
      {'name': 'Guam', 'x509_code': 'GU'},
      {'name': 'Guatemala', 'x509_code': 'GT'},
      {'name': 'Guernsey', 'x509_code': 'GG'},
      {'name': 'Guinea', 'x509_code': 'GN'},
      {'name': 'Guinea-Bissau', 'x509_code': 'GW'},
      {'name': 'Guyana', 'x509_code': 'GY'},
      {'name': 'Haiti', 'x509_code': 'HT'},
      {'name': 'Heard and McDonald Islands', 'x509_code': 'HM'},
      {'name': 'Honduras', 'x509_code': 'HN'},
      {'name': 'Hong Kong', 'x509_code': 'HK'},
      {'name': 'Hungary', 'x509_code': 'HU'},
      {'name': 'Iceland', 'x509_code': 'IS'},
      {'name': 'India', 'x509_code': 'IN'},
      {'name': 'Indonesia', 'x509_code': 'ID'},
      {'name': 'Ireland', 'x509_code': 'IE'},
      {'name': 'Isle of Man', 'x509_code': 'IM'},
      {'name': 'Israel', 'x509_code': 'IL'},
      {'name': 'Italy', 'x509_code': 'IT'},
      {'name': 'Jamaica', 'x509_code': 'JM'},
      {'name': 'Japan', 'x509_code': 'JP'},
      {'name': 'Jersey', 'x509_code': 'JE'},
      {'name': 'Jordan', 'x509_code': 'JO'},
      {'name': 'Kazakhstan', 'x509_code': 'KZ'},
      {'name': 'Kenya', 'x509_code': 'KE'},
      {'name': 'Kiribati', 'x509_code': 'KI'},
      {'name': 'Korea (South)', 'x509_code': 'KR'},
      {'name': 'Kuwait', 'x509_code': 'KW'},
      {'name': 'Kyrgyzstan', 'x509_code': 'KG'},
      {'name': 'Laos', 'x509_code': 'LA'},
      {'name': 'Latvia', 'x509_code': 'LV'},
      {'name': 'Lesotho', 'x509_code': 'LS'},
      {'name': 'Libya', 'x509_code': 'LY'},
      {'name': 'Liechtenstein', 'x509_code': 'LI'},
      {'name': 'Lithuania', 'x509_code': 'LT'},
      {'name': 'Luxembourg', 'x509_code': 'LU'},
      {'name': 'Macau', 'x509_code': 'MO'},
      {'name': 'Macedonia', 'x509_code': 'MK'},
      {'name': 'Madagascar', 'x509_code': 'MG'},
      {'name': 'Malawi', 'x509_code': 'MW'},
      {'name': 'Malaysia', 'x509_code': 'MY'},
      {'name': 'Maldives', 'x509_code': 'MV'},
      {'name': 'Mali', 'x509_code': 'ML'},
      {'name': 'Malta', 'x509_code': 'MT'},
      {'name': 'Marshall Islands', 'x509_code': 'MH'},
      {'name': 'Martinique', 'x509_code': 'MQ'},
      {'name': 'Mauritania', 'x509_code': 'MR'},
      {'name': 'Mauritius', 'x509_code': 'MU'},
      {'name': 'Mayotte', 'x509_code': 'YT'},
      {'name': 'Mexico', 'x509_code': 'MX'},
      {'name': 'Micronesia', 'x509_code': 'FM'},
      {'name': 'Moldova', 'x509_code': 'MD'},
      {'name': 'Monaco', 'x509_code': 'MC'},
      {'name': 'Mongolia', 'x509_code': 'MN'},
      {'name': 'Montenegro', 'x509_code': 'ME'},
      {'name': 'Montserrat', 'x509_code': 'MS'},
      {'name': 'Morocco', 'x509_code': 'MA'},
      {'name': 'Mozambique', 'x509_code': 'MZ'},
      {'name': 'Myanmar', 'x509_code': 'MM'},
      {'name': 'Namibia', 'x509_code': 'NA'},
      {'name': 'Nauru', 'x509_code': 'NR'},
      {'name': 'Nepal', 'x509_code': 'NP'},
      {'name': 'Netherlands Antilles', 'x509_code': 'AN'},
      {'name': 'Netherlands', 'x509_code': 'NL'},
      {'name': 'Neutral Zone', 'x509_code': 'NT'},
      {'name': 'New Caledonia', 'x509_code': 'NC'},
      {'name': 'New Zealand (Aotearoa)', 'x509_code': 'NZ'},
      {'name': 'Nicaragua', 'x509_code': 'NI'},
      {'name': 'Niger', 'x509_code': 'NE'},
      {'name': 'Nigeria', 'x509_code': 'NG'},
      {'name': 'Niue', 'x509_code': 'NU'},
      {'name': 'Norfolk Island', 'x509_code': 'NF'},
      {'name': 'Northern Mariana Islands', 'x509_code': 'MP'},
      {'name': 'Norway', 'x509_code': 'NO'},
      {'name': 'Oman', 'x509_code': 'OM'},
      {'name': 'Pakistan', 'x509_code': 'PK'},
      {'name': 'Palau', 'x509_code': 'PW'},
      {'name': 'Palestinian Territory', 'x509_code': 'PS'},
      {'name': 'Panama', 'x509_code': 'PA'},
      {'name': 'Papua New Guinea', 'x509_code': 'PG'},
      {'name': 'Paraguay', 'x509_code': 'PY'},
      {'name': 'Peru', 'x509_code': 'PE'},
      {'name': 'Philippines', 'x509_code': 'PH'},
      {'name': 'Pitcairn', 'x509_code': 'PN'},
      {'name': 'Poland', 'x509_code': 'PL'},
      {'name': 'Portugal', 'x509_code': 'PT'},
      {'name': 'Puerto Rico', 'x509_code': 'PR'},
      {'name': 'Qatar', 'x509_code': 'QA'},
      {'name': 'Reunion', 'x509_code': 'RE'},
      {'name': 'Romania', 'x509_code': 'RO'},
      {'name': 'Russian Federation', 'x509_code': 'RU'},
      {'name': 'Rwanda', 'x509_code': 'RW'},
      {'name': 'S. Georgia and S. Sandwich Isls.', 'x509_code': 'GS'},
      {'name': 'Saint Kitts and Nevis', 'x509_code': 'KN'},
      {'name': 'Saint Lucia', 'x509_code': 'LC'},
      {'name': 'Saint Vincent and the Grenadines', 'x509_code': 'VC'},
      {'name': 'Samoa', 'x509_code': 'WS'},
      {'name': 'San Marino', 'x509_code': 'SM'},
      {'name': 'Sao Tome and Principe', 'x509_code': 'ST'},
      {'name': 'Saudi Arabia', 'x509_code': 'SA'},
      {'name': 'Senegal', 'x509_code': 'SN'},
      {'name': 'Serbia', 'x509_code': 'RS'},
      {'name': 'Seychelles', 'x509_code': 'SC'},
      {'name': 'Sierra Leone', 'x509_code': 'SL'},
      {'name': 'Singapore', 'x509_code': 'SG'},
      {'name': 'Slovak Republic', 'x509_code': 'SK'},
      {'name': 'Slovenia', 'x509_code': 'SI'},
      {'name': 'Solomon Islands', 'x509_code': 'SB'},
      {'name': 'South Africa', 'x509_code': 'ZA'},
      {'name': 'Spain', 'x509_code': 'ES'},
      {'name': 'Sri Lanka', 'x509_code': 'LK'},
      {'name': 'St. Helena', 'x509_code': 'SH'},
      {'name': 'St. Pierre and Miquelon', 'x509_code': 'PM'},
      {'name': 'Suriname', 'x509_code': 'SR'},
      {'name': 'Svalbard and Jan Mayen Islands', 'x509_code': 'SJ'},
      {'name': 'Swaziland', 'x509_code': 'SZ'},
      {'name': 'Sweden', 'x509_code': 'SE'},
      {'name': 'Switzerland', 'x509_code': 'CH'},
      {'name': 'Taiwan', 'x509_code': 'TW'},
      {'name': 'Tajikistan', 'x509_code': 'TJ'},
      {'name': 'Tanzania', 'x509_code': 'TZ'},
      {'name': 'Thailand', 'x509_code': 'TH'},
      {'name': 'Togo', 'x509_code': 'TG'},
      {'name': 'Tokelau', 'x509_code': 'TK'},
      {'name': 'Tonga', 'x509_code': 'TO'},
      {'name': 'Trinidad and Tobago', 'x509_code': 'TT'},
      {'name': 'Tunisia', 'x509_code': 'TN'},
      {'name': 'Turkey', 'x509_code': 'TR'},
      {'name': 'Turkmenistan', 'x509_code': 'TM'},
      {'name': 'Turks and Caicos Islands', 'x509_code': 'TC'},
      {'name': 'Tuvalu', 'x509_code': 'TV'},
      {'name': 'Uganda', 'x509_code': 'UG'},
      {'name': 'Ukraine', 'x509_code': 'UA'},
      {'name': 'United Arab Emirates', 'x509_code': 'AE'},
      {
        'name': 'United Kingdom of Great Britain and Northern Ireland (GB)',
        'x509_code': 'GB'
      },
      {'name': 'United States of America', 'x509_code': 'US'},
      {'name': 'Uruguay', 'x509_code': 'UY'},
      {'name': 'US Minor Outlying Islands', 'x509_code': 'UM'},
      {'name': 'USSR (former)', 'x509_code': 'SU'},
      {'name': 'Uzbekistan', 'x509_code': 'UZ'},
      {'name': 'Vanuatu', 'x509_code': 'VU'},
      {'name': 'Vatican City State (Holy See)', 'x509_code': 'VA'},
      {'name': 'Venezuela', 'x509_code': 'VE'},
      {'name': 'Viet Nam', 'x509_code': 'VN'},
      {'name': 'Virgin Islands (British)', 'x509_code': 'VG'},
      {'name': 'Virgin Islands (U.S.)', 'x509_code': 'VI'},
      {'name': 'Wallis and Futuna Islands', 'x509_code': 'WF'},
      {'name': 'Western Sahara', 'x509_code': 'EH'},
      {'name': 'Yemen', 'x509_code': 'YE'},
      {'name': 'Zambia', 'x509_code': 'ZM'},

      {
        'name': 'Exceptional reservation - Ascension Island',
        'x509_code': 'AC'
      },
      {
        'name': 'Exceptional reservation - Clipperton Island',
        'x509_code': 'CP'
      },
      {'name': 'Exceptional reservation - Diego Garcia', 'x509_code': 'DG' },
      {'name': 'Exceptional reservation - Ceuta, Melilla', 'x509_code': 'EA'},
      {'name': 'Exceptional reservation - European Union', 'x509_code': 'EU'},
      {'name': 'Exceptional reservation - Eurozone', 'x509_code': 'EZ'},
      {
        'name': 'Exceptional reservation - France, Metropolitan',
        'x509_code': 'FX'
      },
      {'name': 'Exceptional reservation - Canary Islands', 'x509_code': 'IC'},
      {'name': 'Exceptional reservation - USSR', 'x509_code': 'SU'},
      {
        'name': 'Exceptional reservation - Tristan da Cunha',
        'x509_code': 'TA'
      },
      {
        'name': 'Exceptional reservation - United Kingdom (UK)',
        'x509_code': 'UK'
      },
      {'name': 'Exceptional reservation - United Nations', 'x509_code': 'UN'}

      ]

    /*
    Generate a new OTPAUTH URL with a new
    BASE32 random secret as for RFC4648 alfpabet.
    */
    exports.get_new_otpauth_url = function(name, service) {
        // Sanitize the service name to make it URL friendly.
        var service = service.replace(' ', '-')
        var alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ234567'
        var secret = ''
        for (var i = 0; i < 16; i++) {
          secret += alphabet.charAt(Math.floor(Math.random() * 32))
        }

        return ('otpauth://totp/' + service + ':' +
            name +
            '?secret=' +
            secret +
            '&issuer=' + service
            )

    }

  /*
  Return true if `text` contains the  `needle`.

  Designed to be used as a helper for filters.

  When `needle` is empty or null it always returns True.
  */
  exports.containsInsensitive = function(text, needle) {
      if (!needle) {
        return true
      }
      if (!text) {
        return false
      }
      var needle = needle.toLowerCase()
      return text.toLowerCase().indexOf(needle) !== -1
  }

  exports.setPasswordMeter = function(scope, target) {
    return function(current, old) {
        if (!current) {
            scope.password_meter = {
              'score': 0
            }
            return
        }
        var minimum_length = scope.configuration.server.password_minimum_length
        if (current.length < minimum_length) {
            scope.password_meter = {
              'score': 0,
              'feedback': {'warning': 'Password is too short.'}
            }

            return
        }

        var user_inputs = _getUserInputs(scope[target])
        var zxcvbn = require('zxcvbn', 'zxcvbn')
        scope.password_meter = zxcvbn(current, user_inputs)
    }
  }

  /*
  Return a message if config delete operation should be blocked.
  Return empty string otherwise.
  */
  exports.isDeleteBlocked = function(config, target) {
    if (exports.constants.DEFAULT_UUIDS.contains(config.uuid)) {
        return 'Default %(target)s can\'t be removed.'.supplant({
          'target': target})
    }
    return ''
  }


  /*
  Return IE Version.

  We set the _user_agent here to make it easier to mock in  unit tests.
  All other tests should mock the public `exports.ie_version` value.
  */

  exports._user_agent = navigator.userAgent
  exports.getIEVersion = function() {
      // Set defaults
      var value = {
          IsIE: false,
          TrueVersion: 0,
          ActingVersion: 0,
          CompatibilityMode: false
      }

      // Try to find the Trident version number
      var trident = exports._user_agent.match(/Trident\/(\d+)/)
      if (trident) {
          value.IsIE = true
          //Convert from the Trident version number to the IE version number
          value.TrueVersion = parseInt(trident[1], 10) + 4
      }

      // Try to find the MSIE number
      var msie = exports._user_agent.match(/MSIE (\d+)/)
      if (msie) {
          value.IsIE = true;
          // Find the IE version number from the user agent string
          value.ActingVersion = parseInt(msie[1]);
      } else {
          // Must be IE 11 in "edge" mode
          value.ActingVersion = value.TrueVersion
      }

      // If we have both a Trident and MSIE version number, see if they're
      // different
      if (value.IsIE && value.TrueVersion > 0 && value.ActingVersion > 0) {
          // In compatibility mode if the trident number doesn't match up with
          // the MSIE number
          value.CompatibilityMode = value.TrueVersion != value.ActingVersion;
      }

      return value
  }


  // Helper to avoid importing the constant.
  exports.cache_buster = CACHE_BUSTER
  // Helper to reduce calling the fuction.
  // It can also help with testing.
  exports.ie_version = exports.getIEVersion()

  /*
  Return list with user input values
  */
  var _getUserInputs = function(model) {
    var result = [
      'SFTPPlus',
      'sftpplus',
      'ftp',
      'ftps',
      'sftp',
      'FTP',
      'FTPS',
      'SFTP'
      ]
    result.push(model.name)
    angular.forEach(
      model.description.split(' '),
      function(value) {
        var candidate = value.trim()
        if (!candidate) {
          return
        }
        result.push(candidate.toLowerCase())
      })
    return result
  }

  /*
  Retrun true if `configuration` for component is excluded from `filter_value`.
  */
  exports.isComponentExcluded = function(filter_value, configuration) {
      if (!filter_value) {
          return false
      }

      if (exports.containsInsensitive(configuration.name, filter_value)) {
          return false
      }

      if (exports.containsInsensitive(
              configuration.description, filter_value)) {
          return false
      }

      return true
  }

  /*
   Return an empty runnable that is used to represent components that
   don't yet have a runnable.
   For example, if their configuration was not yet applied to the server.
   */
   exports.getRunnablePlaceholder = function() {
      return {
        'started': false,
        'operating': false,
        'state': 'Pending add',
        'restart_required': []
      }
    }

    exports.getBaseViewModel = function(configuration, runnables) {
        var result = {
            'uuid': configuration.uuid,
            'name': configuration.name,
            'config': configuration,
            // True if this is a newly created service that was not applied.
            'just_created': false,
            // The service's runnable representation.
            'runnable': {}
            }

        if (!result.uuid) {
            // This is a new component, without a runnable.
            result.just_created = true
            result.uuid = configuration.name
            // Not yet applied
            result.runnable = exports.getRunnablePlaceholder()
            return result
        }

        // Set the direct runnable for automatic update of the UI.
        result.runnable = runnables[result.uuid]
        return result
    }


  /*
  Convert all the dict/object members into an array having "key" and "value"
  as members.
  */
  exports.keyValueArray = function(input) {
      var result = []

      for (var key in input) {
          result.push({'key': key, 'value': input[key]})
      }

      result.sort(init.sort_by('value', false, init.toUpperCaseFn))
      return result
  }

/*
Return a list of types sorted by their human readable text,
for all available components from family.
*/
exports.getFamilyTypes = function(family) {
    var family_info = exports.constants[
      family.toUpperCase().replace('-', '_') + '_NAMES']

    var values = []
    for (var key in family_info) {
        values.push({'type': key, 'name': exports.getReadableType(family, key)})
    }

    values.sort(init.sort_by('name', false, init.toUpperCaseFn))

    var result = []
    angular.forEach(
    values,
    function(value, index) {
      result.push(value['type'])
    })

    return result
}

exports.getIPAddressDescription = function(address) {

  if (address == '0.0.0.0') {
      return 'All IPv4'
  }

  if (address == '::') {
      return 'All IPv6'
  }
  return address
}

/*
Return the human readable representation for an component type.
*/
exports.getReadableType = function(family, entity_type) {
    if (!entity_type) {
        return 'NO-TYPE-YET'
    }

    var family_info = exports.constants[
      family.toUpperCase().replace('-', '_') + '_NAMES']

    if (!family_info) {
        // We don't know this family.
        // Just default to raw value.
        return entity_type.toUpperCase()
    }

    var name = family_info[entity_type]
    if (!name) {
        // We know the family, but the type is not found.
        // Default to raw raw.
        name = entity_type.toUpperCase()
    }
    return name
}

/*
Write `text` to clipboard.

Does nothing if browser has no clipboard API.
*/
exports.writeClipboard = function(text) {
    if ($window.navigator.clipboard != undefined) {
        // Modern browser.
        $window.navigator.clipboard.writeText(text)
        return true
    }

    if ($window.clipboardData) {
        // IE 11
        $window.clipboardData.setData("Text", text)
        return true
    }
    return false
}

/*
Return True if value is on of the inherited values.
*/
exports.isInherited = function(value) {

  if (exports.constants.CONFIGURATION_INHERIT.contains(value)) {
    return true
  }
  return false
}


return exports


}
/* c8 ignore next */
}(typeof exports === 'undefined' ? this.utils = {} : exports))
