/*
Manufacture is our test object factory. It is called mk for short tests.
*/

var getManufacture = function(){

var exports = {}
var _unique_id = 0

exports.uniqueNumber = function() {
    _unique_id += 1
    return _unique_id
}
// Shorter alias.
exports.number = exports.uniqueNumber

exports.uniqueString = function() {
    var uid = exports.uniqueNumber()
    return 'string-' + uid.toString()
}
// Shorter alias.
exports.string = exports.uniqueString

exports.password = function() {
    return 'Str0ng!-' + exports.string()
}

exports.randomMark = function() {
    return Math.floor((1 + Math.random()) * 0x10000).toString(16)
}

var _s4 = function() {
    return exports.makeRandomMark.substring(1)
}

exports.uuid4 = function() {
    return _s4() + _s4() + '-' + _s4() + '-' + _s4() + '-' +
           _s4() + '-' + _s4() + _s4() + _s4()
}


/*
Simulate updating the value from the view that is attached to `scope` and
then triggering an event.
*/
exports.getViewUpdate = function(scope) {
    return function (target, value, event_type) {
        if (!event_type) {
            event_type = 'input'
        }
        angular.element(target).val(value)
        // We trigger the event at the low-level JS.
        if (!target.dispatchEvent) {
            // We have a high level angular element.
            target = target[0]
        }
        target.dispatchEvent(new Event(event_type))
    }
}


/*
Return a new instance for Utils service.
*/
exports.servicesUtils = function($injector) {
    var utils = require('../services/utils.js', 'utils')

    var utils = utils.Utils(
        $injector.get('$http'),
        $injector.get('$log'),
        $injector.get('$cookies'),
        $injector.get('$window'),
        '%(cache)s'
        )
    // Create a copy of the constants, as the test might mess with the
    // constants.
    utils.constants = JSON.parse(JSON.stringify(utils.constants))

    return utils
}


/*
Set up the scope to the minimal backend configuration
*/
exports.minimalProcess = function(sut) {

    var default_lm = sut.utils.constants['DEFAULT_LOCAL_MANAGER_UUID']
    var default_auth = sut.utils.constants[
        'CONFIGURATION_DEFAULT_AUTHENTICATION']

    sut.scope.process.runnables = {}

    sut.scope.configuration = sut.scope.$parent.configuration
    sut.scope.configuration.accounts = {}
    sut.scope.configuration.groups = {}
    sut.scope.configuration.administrators = {}
    sut.scope.configuration.roles = {}

    sut.scope.configuration.locations = {
        'DEFAULT-FILESYSTEM': {
            'uuid': 'DEFAULT-FILESYSTEM',
            'name': 'Default filesytem',
            'type': 'filesystem',
        }
    }
    sut.scope.process.runnables.locations = {
        'DEFAULT-FILESYSTEM': {
            'uuid': 'DEFAULT-FILESYSTEM',
            'name': 'Default filesytem',
            'type': 'filesystem',
            'state': 'stopped',
        }
    }

    sut.scope.configuration.authentications = {
        'DEFAULT-AUTHENTICATION': {
            'uuid': 'DEFAULT-AUTHENTICATION',
            'name': 'Default auht',
            'type': 'application',
        }
    }
    sut.scope.process.runnables.authentications =  {
        'DEFAULT-AUTHENTICATION': {
            'uuid': 'DEFAULT-AUTHENTICATION',
            'name': 'Default auht',
            'type': 'application',
            'state': 'stopped',
        }
    }
    // DEFAULT_ANALYTICS_UUID = 'DEFAULT-ANALYTICS'
    sut.scope.configuration.resources = {
        'DEFAULT-EMAIL-CLIENT': {
            'uuid': 'DEFAULT-EMAIL-CLIENT',
            'name': 'Email client',
            'type': 'email-client',
            'email_to_recipients': ['john@example.com', 'marry@yo.lo']
        },
        'DEFAULT-LETS-ENCRYPT': {
            'uuid': 'DEFAULT-LETS-ENCRYPT',
            'name': 'Lets Encrypt',
            'type': 'lets-encrypt',
        },
        'DEFAULT-SQLITE': {
            'uuid': 'DEFAULT-SQLITE',
            'name': 'SQlite DB',
            'type': 'sqlite',
        },
        'DEFAULT-ANALYTICS': {
            'uuid': 'DEFAULT-ANALYTICS',
            'name': 'Analitics resources',
            'type': 'analytics',
        },
    }

    sut.scope.process.runnables.resources = {
        'DEFAULT-EMAIL-CLIENT': {
            'uuid': 'DEFAULT-EMAIL-CLIENT',
            'name': 'Email client',
            'type': 'email-client',
            'state': 'stopped',
        },
        'DEFAULT-LETS-ENCRYPT': {
            'uuid': 'DEFAULT-LETS-ENCRYPT',
            'name': 'Lets Encrypt',
            'type': 'lets-encrypt',
            'state': 'stopped',
        },
        'DEFAULT-SQLITE': {
            'uuid': 'DEFAULT-SQLITE',
            'name': 'SQlite DB',
            'type': 'sqlite',
            'state': 'stopped',
        },
        'DEFAULT-ANALYTICS': {
            'uuid': 'DEFAULT-ANALYTICS',
            'name': 'Analitics resources',
            'type': 'analytics',
            'state': 'stopped',
        },
    }

    sut.scope.configuration.services = {}
    sut.scope.process.runnables.services = {}
    sut.scope.process.runnables.status = {}

    sut.scope.configuration.services[default_lm] = {
        'uuid': default_lm,
        'name': 'Zoro Default Web Manager',
        'type': 'manager',
        'configuration': {
            'ssl_domains': ['adm.ex.com', 'admin.ex.org'],
        }
    }
    sut.scope.process.runnables.services[default_lm] = {
        'uuid': default_lm,
        'name': 'Zoro Default Web Manager',
        'type': 'manager',
    }

    sut.scope.configuration.authentications[default_auth] = {
        'uuid': default_auth,
        'name': 'A Default Auth Method',
        'type': 'application',
    }
    sut.scope.process.runnables.authentications[default_auth] = {
        'uuid': default_auth,
        'name': 'A Default Auth Method',
        'type': 'application',
    }
}


/*
Reset our internal window state.
*/
exports.resetWindow = function(jsdom_window) {
    jsdom_window._clipboard_content = ''
    jsdom_window.sessionStorage.clear()
    jsdom_window.localStorage.clear()
}

/*
Return a mock Flowbite dropdown JS object.
*/
exports.flowbiteDropdown = function (visible) {

    var result = {}
    result._visible = visible
    result.show = function() { result._visible = true}
    result.hide = function() { result._visible = false}
    result.isVisible = function() { return result._visible}
    result.toggle = function() { result._visible = !result._visible}
    return result
}

exports.getComponent = function(target, bindings) {
    var ctrl
    var utils
    var $window
    var dayjs
    var scope

    require('../app.js')
    angular.mock.module('app')

    angular.mock.inject(function($injector, _$componentController_) {
        utils = exports.servicesUtils($injector)
        $window = $injector.get('$window')
        dayjs = $injector.get('dayjs')
        var locals = {
            'utils': utils,
            '$window': $window,
        }
        // Try to generate bindings as close as possible to the real
        // usage.
        var resolved_bindings = {}
        angular.forEach(bindings, function(value, key) {
            resolved_bindings[key] = {}
        })
        // Make sure we don't use the expiration from the current
        // build, as for JS testing we want to have full control.
        utils.constants.EXPIRATION_TIMESTAMP = 0
        ctrl = _$componentController_(target, locals, resolved_bindings)

    })

    return {
        'ctrl': ctrl,
        'utils': utils,
        '$window': $window,
        'dayjs': dayjs
    }
}

/*
Return a dayjs with a fixed current data.
*/
exports.getDayJS = function () {
    var dayjs = require('dayjs')
    // We make sure to return a fixed date
    // to make it easy to test.
    var mockDayJS = function(value) {
        if (!value) {
            // This should be 1234567890
            return dayjs('2009-02-13T23:31:30.000Z')
        }
        return dayjs(value)
    }
    mockDayJS.unix = function(value) {return dayjs.unix(value)}
    mockDayJS.test = true
    return mockDayJS

}
/*
Return a new controller for testing.

This is designed to be called as the start of each test.
It should try to use as fewer global objects as possible.
For example, an `utils` dependency is generated for each call.

But the angular dependency are global/singleton.
*/
exports.getController = function(target) {
    var scope
    var utils
    var route = {}
    var injector
    var $location
    var $httpBackend
    var $flushPendingTasks

    require('../app.js')
    angular.mock.module('app')

    angular.mock.inject(function($rootScope, $controller, $injector) {
        var parent_scope = $rootScope.$new()
        injector = $injector
        utils = exports.servicesUtils($injector)
        $httpBackend = injector.get('$httpBackend')
        $flushPendingTasks = $injector.get('$flushPendingTasks')
        $location = injector.get('$location')
        var ctrl = $controller(
            'mainCtrl', {
            $scope: parent_scope,
            utils: utils
        })

        scope = parent_scope.$new();
        ctrl = $controller(
            target, {
            $scope: scope,
            $routeParams: route,
            utils: utils,
            'dayjs': exports.getDayJS(),
        })

        // Run an initial digest to 'settle' all watchers.
        scope.$digest()
    })

    // Helper to navigate to a separate path and check for page loading.
    var changeRoute= function(path) {
        exports.expectDefaultRouter($httpBackend)
        $location.path(path)
        window.location.href = path
        $httpBackend.flush()
    }

    var window = injector.get('$window')
    exports.resetWindow(window)
    return {
        'scope': scope,
        'utils': utils,
        'route': route,
        '$httpBackend': $httpBackend,
        '$flushPendingTasks': $flushPendingTasks,
        '$location': $location,
        'changeRoute': changeRoute,
        '$window': window,
        'dayjs': exports.getDayJS(),
        'expectDefaultRouter': () => {exports.expectDefaultRouter($httpBackend)}
    }
}


/*
Setup the assertion for loading the default router.

This is here to help with a bug in our testing code.
*/
exports.expectDefaultRouter = function($httpBackend) {
    $httpBackend.expectPOST('/rp/path/json', {
        'jsonrpc': '2.0',
        'method': 'get_property',
        'params': {'path': '/runnables/'},
        'id': 1
    }).respond({})
    $httpBackend.expectGET(/partials\/status.html\?v=.+/).respond('')
}

/*
Create a new fake dropdown menu.
*/
exports.Dropdown = function() {
    function MockDropdown() {
        this._visible = 'unset'
        this.show = () => {this._visible = true}
        this.hide = () => {this._visible = false}
        this.isVisible = () => {return this._visible}
    }

    return new MockDropdown()
}

return exports
}

global.mk = getManufacture()
