# Copyright (c) 2014 Adi Roiban.
# See LICENSE for details.
"""
Configuration for client side components.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

from zope.interface import implementer

from chevah.server.commons.constant import (
    CONFIGURATION_SECTION_TRANSFERS,
    DEFAULT_LOCAL_FILESYSTEM_UUID,
    TYPE_NAME,
    )
from chevah.server.commons.interface import (
    ITransferConfiguration,
    ITransfersConfiguration,
    )

from chevah.server.configuration.option import (
    EntityConfiguration,
    LocationWatchConfigurationMixin,
    )
from chevah.server.configuration.model import (
    WritableBoolean,
    WritableInteger,
    WritableLines,
    WritableNumber,
    WritableString,
    WritableStringCaseInsensitive,
    WritableCSVLines,
    WritableCSVLinesUnique,
    WritableStringOrEmpty,
    )
from chevah.server.configuration.set import SetConfiguration


@implementer(ITransfersConfiguration)
class TransfersConfiguration(SetConfiguration):
    """
    See: `ITransfersConfiguration`.
    """
    SECTION_PREFIX = CONFIGURATION_SECTION_TRANSFERS
    __type_interface__ = ITransferConfiguration


@implementer(ITransferConfiguration)
class TransferConfiguration(
        EntityConfiguration,
        LocationWatchConfigurationMixin,
        ):
    """
    See: ITransferConfiguration.
    """

    SECTION_PREFIX = CONFIGURATION_SECTION_TRANSFERS

    destination_uuid = WritableString(
        'destination_uuid', default=DEFAULT_LOCAL_FILESYSTEM_UUID)
    destination_path = WritableStringOrEmpty('destination_path')
    destination_fallback_path = WritableString('destination_fallback_path')
    destination_temporary_suffix = WritableStringOrEmpty(
        'destination_temporary_suffix')
    batch_interval = WritableNumber('batch_interval', default=0)
    batch_marker_path = WritableLines('batch_marker_path')
    execute_timeout = WritableNumber(
        'execute_timeout',
        default=30,
        allow_negative=False,
        allow_zero=False,
        )
    execute_before = WritableStringOrEmpty('execute_before')
    execute_after_success = WritableStringOrEmpty('execute_after_success')
    execute_after_failure = WritableStringOrEmpty('execute_after_failure')
    execute_on_destination_before = WritableStringOrEmpty(
        'execute_on_destination_before')
    execute_on_destination_after_success = WritableStringOrEmpty(
        'execute_on_destination_after_success')
    execute_on_destination_after_failure = WritableStringOrEmpty(
        'execute_on_destination_after_failure')
    archive_success_path = WritableStringOrEmpty('archive_success_path')
    archive_failure_path = WritableStringOrEmpty('archive_failure_path')
    archive_retention_period = WritableNumber(
        'archive_retention_period', default=0)
    archive_format = WritableStringCaseInsensitive(
        'archive_format', default=TYPE_NAME.TIMESTAMP_ALWAYS)
    retry_count = WritableInteger('retry_count', default=2)
    retry_wait = WritableNumber('retry_wait', default=60)
    # It is represented at the low level as a multi-line value.
    # At the app layer the lines are merged and re-arranged as a single
    # list of actions.
    schedule = WritableCSVLinesUnique('schedule')
    overwrite_rule = WritableString('overwrite_rule')
    destination_path_actions = WritableCSVLines(
        'destination_path_actions')
    destination_content_actions = WritableCSVLines(
        'destination_content_actions')
    transfer_memory_duration = WritableNumber(
        'transfer_memory_duration', default=0)
    minimum_transfer_count = WritableInteger(
        'minimum_transfer_count', default=0)
    minimum_transfer_interval = WritableNumber(
        'minimum_transfer_interval', default=86400)
    ignore_duplicate_paths = WritableBoolean('ignore_duplicate_paths')
    source_filter_age = WritableNumber('source_filter_age', default=0)
    delete_source_on_success = WritableBoolean('delete_source_on_success')
    delete_source_parent_delay = WritableNumber(
        'delete_source_parent_delay', default=0)
